Package: syntenet
Title: Inference And Analysis Of Synteny Networks
Version: 1.15.0
Date: 2022-03-28
Authors@R: 
    c(
    person(given = "Fabrício",
           family = "Almeida-Silva",
           role = c("aut", "cre"),
           email = "fabricio_almeidasilva@hotmail.com",
           comment = c(ORCID = "0000-0002-5314-2964")),
    person(given = "Tao",
           family = "Zhao",
           role = "aut",
           email = "tao.zhao@nwafu.edu.cn",
           comment = c(ORCID = "0000-0001-7302-6445")),
    person(given = "Kristian K",
           family = "Ullrich",
           role = "aut",
           email = "ullrich@evolbio.mpg.de",
           comment = c(ORCID = "0000-0003-4308-9626")),
    person(given = "Yves",
           family = "Van de Peer",
           role = "aut",
           email = "yves.vandepeer@psb.vib-ugent.be",
           comment = c(ORCID = "0000-0003-4327-3730"))
    )
Description: syntenet can be used to infer synteny networks from 
    whole-genome protein sequences and analyze them. Anchor pairs
    are detected with the MCScanX algorithm, which was ported to this package
    with the Rcpp framework for R and C++ integration. 
    Anchor pairs from synteny analyses are
    treated as an undirected unweighted graph (i.e., a synteny network),
    and users can perform: i. network clustering; ii. phylogenomic profiling 
    (by identifying which species contain which clusters) and; 
    iii. microsynteny-based phylogeny reconstruction with maximum likelihood.
License: GPL-3
URL: https://github.com/almeidasilvaf/syntenet
BugReports: https://support.bioconductor.org/t/syntenet
biocViews: Software, NetworkInference, FunctionalGenomics,
        ComparativeGenomics, Phylogenetics, SystemsBiology,
        GraphAndNetwork, WholeGenome, Network
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.2
Imports: Rcpp (>= 1.0.8), BiocParallel, GenomicRanges, rlang,
        Biostrings, utils, methods, igraph, stats, grDevices,
        RColorBrewer, pheatmap, ggplot2, ggnetwork, intergraph
Suggests: rtracklayer, BiocStyle, ggtree, labdsv, covr, knitr,
        rmarkdown, testthat (>= 3.0.0), xml2, networkD3
Config/testthat/edition: 3
VignetteBuilder: knitr
LinkingTo: Rcpp, testthat
NeedsCompilation: yes
Depends: R (>= 4.2)
LazyData: false
git_url: https://git.bioconductor.org/packages/syntenet
git_branch: devel
git_last_commit: aa99fce
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
Packaged: 2026-04-29 01:30:34 UTC; biocbuild
Author: Fabrício Almeida-Silva [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-5314-2964>),
  Tao Zhao [aut] (ORCID: <https://orcid.org/0000-0001-7302-6445>),
  Kristian K Ullrich [aut] (ORCID:
    <https://orcid.org/0000-0003-4308-9626>),
  Yves Van de Peer [aut] (ORCID: <https://orcid.org/0000-0003-4327-3730>)
Maintainer: Fabrício Almeida-Silva <fabricio_almeidasilva@hotmail.com>

# Generated by roxygen2: do not edit by hand

export(binarize_and_transpose)
export(check_input)
export(cluster_network)
export(collapse_bidirectional_hits)
export(collapse_protein_ids)
export(create_species_id_table)
export(diamond_is_installed)
export(export_sequences)
export(fasta2AAStringSetlist)
export(find_GS_clusters)
export(gff2GRangesList)
export(infer_microsynteny_phylogeny)
export(infer_syntenet)
export(interspecies_synteny)
export(intraspecies_synteny)
export(iqtree_is_installed)
export(iqtree_version)
export(last_is_installed)
export(make_bidirectional)
export(parse_collinearity)
export(phylogenomic_profile)
export(plot_network)
export(plot_profiles)
export(process_input)
export(profiles2phylip)
export(rcpp_mcscanx_file)
export(read_diamond)
export(run_diamond)
export(run_last)
import(Rcpp)
import(intergraph)
importFrom(BiocParallel,SerialParam)
importFrom(BiocParallel,bplapply)
importFrom(Biostrings,readAAStringSet)
importFrom(Biostrings,subseq)
importFrom(Biostrings,width)
importFrom(Biostrings,writeXStringSet)
importFrom(GenomicRanges,GRangesList)
importFrom(GenomicRanges,makeGRangesFromDataFrame)
importFrom(GenomicRanges,mcols)
importFrom(RColorBrewer,brewer.pal)
importFrom(ggnetwork,geom_edges)
importFrom(ggnetwork,geom_nodes)
importFrom(ggnetwork,ggnetwork)
importFrom(ggnetwork,theme_blank)
importFrom(ggplot2,aes)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,guides)
importFrom(ggplot2,scale_color_manual)
importFrom(grDevices,colorRampPalette)
importFrom(igraph,cluster_infomap)
importFrom(igraph,graph_from_data_frame)
importFrom(igraph,simplify)
importFrom(methods,is)
importFrom(pheatmap,pheatmap)
importFrom(rlang,.data)
importFrom(stats,ave)
importFrom(stats,dist)
importFrom(stats,hclust)
importFrom(stats,reshape)
importFrom(stats,setNames)
importFrom(utils,capture.output)
importFrom(utils,combn)
importFrom(utils,head)
importFrom(utils,read.csv)
importFrom(utils,read.table)
importFrom(utils,write.table)
useDynLib(syntenet, .registration = TRUE)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# syntenet <img src="man/figures/logo.png" align="right" height="138" />

<!-- badges: start -->

[![GitHub
issues](https://img.shields.io/github/issues/almeidasilvaf/syntenet)](https://github.com/almeidasilvaf/syntenet/issues)
[![Lifecycle:
stable](https://img.shields.io/badge/lifecycle-stable-brightgreen.svg)](https://lifecycle.r-lib.org/articles/stages.html#stable)
[![R-CMD-check-bioc](https://github.com/almeidasilvaf/syntenet/workflows/R-CMD-check-bioc/badge.svg)](https://github.com/almeidasilvaf/syntenet/actions)
[![Codecov test
coverage](https://codecov.io/gh/almeidasilvaf/syntenet/branch/devel/graph/badge.svg)](https://codecov.io/gh/almeidasilvaf/syntenet?branch=devel)
<!-- badges: end -->

The goal of `syntenet` is to infer synteny networks from whole-genome
protein sequence data and analyze them. Anchor pairs from synteny
analyses are treated as an undirected unweighted graph (i.e., a synteny
network), and users can perform:

- **Synteny detection** using a native implementation of the [MCScanX
  algorithm](https://doi.org/10.1093/nar/gkr1293), a C++ program that
  has been modified and ported to R with Rcpp. This way, users do not
  need to install MCScanX beforehand, because `syntenet` has its own
  implementation of the same algorithm.
- **Synteny network inference** by treating anchor pairs as edges of a
  graph;
- **Network clustering** using the Infomap algorithm;
- **Phylogenomic profiling**, which consists in identifying which
  species contain which clusters. This analysis can reveal highly
  conserved synteny clusters and taxon-specific ones (e.g., family- and
  order-specific clusters);
- **Microsynteny-based phylogeny reconstruction** with maximum
  likelihood, which can be achieved by inferring a phylogeny from a
  binary matrix of phylogenomic profiles with IQTREE.

## Installation instructions

Get the latest stable `R` release from
[CRAN](http://cran.r-project.org/). Then install `syntenet` from
[Bioconductor](http://bioconductor.org/) using the following code:

``` r
if (!requireNamespace("BiocManager", quietly = TRUE)) {
    install.packages("BiocManager")
}

BiocManager::install("syntenet")
```

And the development version from
[GitHub](https://github.com/almeidasilvaf/syntenet) with:

``` r
BiocManager::install("almeidasilvaf/syntenet")
```

## Citation

Below is the citation output from using `citation('syntenet')` in R.
Please run this yourself to check for any updates on how to cite
**syntenet**.

``` r
print(citation('syntenet'), bibtex = TRUE)
#> 
#> To cite syntenet in publications, use:
#> 
#>   Almeida-Silva, F., Zhao, T., Ullrich, K.K., Schranz, M.E. and Van de
#>   Peer, Y. syntenet: an R/Bioconductor package for the inference and
#>   analysis of synteny networks. Bioinformatics, 39(1), p.btac806.
#>   (2023). https://doi.org/10.1093/bioinformatics/btac806
#> 
#> A BibTeX entry for LaTeX users is
#> 
#>   @Article{,
#>     title = {syntenet: an R/Bioconductor package for the inference and analysis of synteny networks},
#>     author = {Fabricio Almeida-Silva and Tao Zhao and Kristian K. Ullrich and M. Eric Schranz and Yves {Van de Peer}},
#>     journal = {Bioinformatics},
#>     year = {2023},
#>     volume = {39},
#>     number = {1},
#>     pages = {btac806},
#>     url = {https://academic.oup.com/bioinformatics/article/39/1/btac806/6947985},
#>     doi = {10.1093/bioinformatics/btac806},
#>   }
```

Please note that `syntenet` was only made possible thanks to many other
R and bioinformatics software authors, which are cited either in the
vignettes and/or the paper(s) describing this package.

## Code of Conduct

Please note that the `syntenet` project is released with a [Contributor
Code of Conduct](http://bioconductor.org/about/code-of-conduct/). By
contributing to this project, you agree to abide by its terms.

## Development tools

- Continuous code testing is possible thanks to [GitHub
  actions](https://www.tidyverse.org/blog/2020/04/usethis-1-6-0/)
  through *[usethis](https://CRAN.R-project.org/package=usethis)*,
  *[remotes](https://CRAN.R-project.org/package=remotes)*, and
  *[rcmdcheck](https://CRAN.R-project.org/package=rcmdcheck)* customized
  to use [Bioconductor’s docker
  containers](https://www.bioconductor.org/help/docker/) and
  *[BiocCheck](https://bioconductor.org/packages/3.15/BiocCheck)*.
- Code coverage assessment is possible thanks to
  [codecov](https://codecov.io/gh) and
  *[covr](https://CRAN.R-project.org/package=covr)*.
- The [documentation website](http://almeidasilvaf.github.io/syntenet)
  is automatically updated thanks to
  *[pkgdown](https://CRAN.R-project.org/package=pkgdown)*.
- The code is styled automatically thanks to
  *[styler](https://CRAN.R-project.org/package=styler)*.
- The documentation is formatted thanks to
  *[devtools](https://CRAN.R-project.org/package=devtools)* and
  *[roxygen2](https://CRAN.R-project.org/package=roxygen2)*.

For more details, check the `dev` directory.

This package was developed using
*[biocthis](https://bioconductor.org/packages/3.15/biocthis)*.

#' Collapse protein IDs into gene IDs in sequence names of AAStringSet objects
#' 
#' This function can be used if the sequence names of the AAStringSet objects
#' contain protein IDs instead of gene IDs (what syntenet requires)
#'
#' @param seq A list of AAStringSet objects, each list element containing
#' protein sequences for a given species. This list must have names 
#' (not NULL), and names of each list element must match the names of
#' list elements in \strong{protein2gene}.
#' @param protein2gene A list of 2-column data frames containing
#' protein-to-gene ID correspondences, where the first column contains 
#' protein IDs, and the second column contains gene IDs. Names of list elements
#' must match names of \strong{seq}.
#' 
#' @return A list of AAStringSet objects as in \strong{seq}, but with
#' protein IDs replaced with gene IDs.
#' 
#' @details
#' For each species, this function will replace the protein IDs in sequence
#' names with gene IDs using the protein-to-gene correspondence table in
#' \strong{protein2gene}. After replacing protein IDs with gene IDs, if
#' there are multiple sequences with the same gene ID (indicating different
#' isoforms of the same gene), only the longest sequence is kept, so that
#' the number of sequences is not greater than the number of genes.
#' 
#' @importFrom Biostrings width
#' @export
#' @rdname collapse_protein_ids
#' @examples
#' # Load data
#' seq_path <- system.file(
#'     "extdata", "RefSeq_parsing_example", package = "syntenet"
#' )
#' seq <- fasta2AAStringSetlist(seq_path)
#' annot <- gff2GRangesList(seq_path)
#' 
#' # Clean sequence names
#' names(seq$Aalosa) <- gsub(" .*", "", names(seq$Aalosa))
#' 
#' # Create a correspondence data frame
#' cor_df <- as.data.frame(annot$Aalosa[annot$Aalosa$type == "CDS", ])
#' cor_df <- cor_df[, c("Name", "gene")]
#' 
#' # Create a list of correspondence data frames
#' protein2gene <- list(Aalosa = cor_df)
#' 
#' # Collapse IDs
#' new_seqs <- collapse_protein_ids(seq, protein2gene)
collapse_protein_ids <- function(seq, protein2gene = NULL) {
    
    if(!is(protein2gene, "list") | !is(protein2gene[[1]], "data.frame")) {
        stop("`protein2gene` must be a list of data frames.")
    }
    
    if(!all(names(seq) %in% names(protein2gene))) {
        stop("All species in `seq` must be in `protein2gene`.")
    }
    
    new_seqs <- lapply(seq_along(seq), function(x) {
        
        species <- names(seq)[x]
        
        nseq <- seq[[species]]
        pgene <- protein2gene[[species]]
        
        ## Get indices of matches
        idx <- unique(match(
            names(nseq), 
            pgene[, 1]
        ))
        if(length(idx) <= 1) {
            
            m <- paste0(
                "None of the protein IDs in column 1 of `protein2gene[['",
                species, "']]` match sequence names in `seq[['", 
                species, "']]`."
                
            )
            stop(m)
        }
        
        ## Replace names
        names(nseq) <- pgene[, 2][idx]
        
        # Keep only longest protein for each gene
        nseq <- nseq[order(Biostrings::width(nseq), decreasing = TRUE), ]
        nseq <- nseq[!duplicated(names(nseq))]
        
        return(nseq)
    })
    names(new_seqs) <- names(seq)
    
    return(new_seqs)
    
}



#' Check if input objects are ready for further analyses
#' 
#' @param seq A list of AAStringSet objects, each list element containing
#' protein sequences for a given species. This list must have names 
#' (not NULL), and names of each list element must match the names of
#' list elements in \strong{annotation}.
#' @param annotation A GRangesList, CompressedGRangesList, or list of
#' GRanges with the annotation for the sequences in \strong{seq}. This list must
#' have names (not NULL), and names of each list element must match the names
#' of list elements in \strong{seq}.
#' @param gene_field Character, name of the column in the GRanges objects
#' that contains gene IDs. Default: "gene_id".
#'
#' @details 
#' This function checks the input data for 3 required conditions:
#' \enumerate{
#'   \item Names of \strong{seq} list (i.e., \code{names(seq)}) match 
#'   the names of \strong{annotation} GRangesList/CompressedGRangesList
#'   (i.e., \code{names(annotation)})
#'   \item For each species (list elements), the number of sequences
#'   in \strong{seq} is not greater than the number of genes 
#'   in \strong{annotation}. This is a way to ensure users do not input
#'   the translated sequences for multiple isoforms of the same gene (generated
#'   by alternative splicing). Ideally, the number of sequences in \strong{seq} 
#'   should be equal to the number of genes in \strong{annotation}, but
#'   this may not always stand true because of non-protein-coding genes.
#'   \item For each species, sequence names (i.e., \code{names(seq[[x]])},
#'   equivalent to FASTA headers) match gene names in \strong{annotation}.
#' }
#' @return TRUE if the objects pass the check.
#' @export
#' @rdname check_input 
#' @examples 
#' data(annotation) 
#' data(proteomes)
#' check_input(proteomes, annotation)
check_input <- function(seq = NULL, annotation = NULL, gene_field = "gene_id") {
    
    valid <- valid_seq(seq) & valid_annot(annotation)

    check1 <- check_list_names(seq, annotation)
    check2 <- check_ngenes(seq, annotation)
    check3 <- check_gene_names(seq, annotation, gene_field = gene_field)
    
    return(TRUE)
}


#' Process sequence data
#'
#' @param seq A list of AAStringSet objects, each list element containing
#' protein sequences for a given species. This list must have names 
#' (not NULL), and names of each list element must match the names of
#' list elements in \strong{annotation}.
#' @param annotation A GRangesList, CompressedGRangesList, or list of
#' GRanges with the annotation for the sequences in \strong{seq}. This list must
#' have names (not NULL), and names of each list element must match the names
#' of list elements in \strong{seq}.
#' @param gene_field Character, name of the column in the GRanges objects
#' that contains gene IDs. Default: "gene_id".
#' @param filter_annotation Logical indicating whether \strong{annotation}
#' should be filtered to keep only genes that are also in \strong{seq}. This
#' is particularly useful if users want to remove information on 
#' non-protein coding genes from \strong{annotation}, since such genes are
#' typically not present in sets of whole-genome protein sequences. 
#' Default: FALSE.
#'
#' @return A list of 2 elements:
#' \describe{
#'   \item{seq}{The processed list of AAStringSet objects from \strong{seq}.}
#'   \item{annotation}{The processed GRangesList or CompressedGRangesList
#'   object from \strong{annotation}.}
#' }
#' 
#' @details 
#' This function processes the input sequences and annotation to:
#' \enumerate{
#'   \item Remove whitespace and anything after it in sequence names 
#'   (i.e., \code{names(seq[[x]])}, which is equivalent to FASTA headers), if
#'   there is any.
#'   \item Add a unique species identifier to sequence names. The species 
#'   identifier consists of the first 3-5 strings of the element name.
#'   For instance, if the first element of the \strong{seq} list is named
#'   "Athaliana", each sequence in it will have an identifier "Atha_" added
#'   to the beginning of each gene name (e.g., Atha_AT1G01010).
#'   \item If sequences have an asterisk (*) representing stop codon, remove it.
#'   \item Add a unique species identifier (same as above) to 
#'   gene and chromosome names of each element of the \strong{annotation}
#'   GRangesList/CompressedGRangesList.
#'   \item Filter each element of the \strong{annotation} 
#'   GRangesList/CompressedGRangesList to keep only seqnames, ranges, and
#'   gene ID.
#' }
#' @importFrom Biostrings width subseq
#' @importFrom GenomicRanges makeGRangesFromDataFrame
#' @export
#' @rdname process_input
#' @examples 
#' data(annotation)
#' data(proteomes)
#' seq <- proteomes
#' clean_data <- process_input(seq, annotation)
process_input <- function(seq = NULL, annotation = NULL, 
                          gene_field = "gene_id", filter_annotation = FALSE) {
    
    check <- check_input(seq, annotation, gene_field = gene_field)
    
    seq_ids <- create_species_id_table(names(seq))$species_id
    annot_ids <- create_species_id_table(names(annotation))$species_id
    
    # Clean 'seq' object first
    clean_seqs <- lapply(seq_along(seq), function(x) {
        sequence <- seq[[x]]
        gene_names <- names(sequence)
        
        ## 1. Remove everything after space
        gene_names <- gsub(" .*", "", gene_names)
        
        ## 2. Add unique identifier in front of gene names
        gene_names <- paste(seq_ids[x], gene_names, sep = "_")
        names(sequence) <- gene_names
        
        ## 3. Remove "*" as stop codon (if any)
        last_aa <- sequence[[1]][Biostrings::width(sequence)[1]]
        if(identical(toString(last_aa), "*")) {
            sequence <- Biostrings::subseq(sequence, 1, width(sequence)-1)
        }
        return(sequence)
    })
    names(clean_seqs) <- names(seq)
    
    # Clean annotation object
    clean_annotation <- lapply(seq_along(annotation), function(x) {
        
        ## 5. Add unique identifier in front of chromosome and gene names
        annot_df <- annotation[[x]][annotation[[x]]$type == "gene"]
        annot_df <- as.data.frame(annot_df)
        annot_df$seqnames <- paste(annot_ids[x], annot_df$seqnames, sep = "_")
        annot_df$gene <- paste(annot_ids[x], annot_df[[gene_field]], sep = "_")
        
        # 6. Keep only seqnames, ranges, and gene ID
        annot_df <- annot_df[, c("seqnames", "start", "end", "strand", "gene")]
        
        # Keep in annotation only genes in `seq`
        if(filter_annotation) {
            genes_in_seq <- names(clean_seqs[[x]])
            annot_df <- annot_df[annot_df$gene %in% genes_in_seq, ]
        }
        
        return(makeGRangesFromDataFrame(annot_df, keep.extra.columns = TRUE))
    })
    names(clean_annotation) <- names(annotation)
    
    flist <- list(seq = clean_seqs, annotation = clean_annotation)
    return(flist)
}

#----Load data------------------------------------------------------------------
data(proteomes)
data(annotation)

## Simulate a GRangesList object without "gene_id"
annotation2 <- lapply(annotation, function(x) {
    x$gene_id <- NULL
    return(x)
})

#----Start tests----------------------------------------------------------------
test_that("check_input() returns TRUE if everything is OK", {
    
    c <- check_input(proteomes, annotation)
    
    expect_equal(c, TRUE)
    expect_equal(class(c), "logical")
})

test_that("process_input() returns clean seq and annotation objects", {
    
    clean <- process_input(proteomes, annotation)
    expect_error(process_input(proteomes, annotation2))
    
    expect_equal(class(clean), "list")
    expect_false(identical(clean$seq, proteomes))
    expect_false(identical(clean$annotation, annotation))

})


test_that("collapse_protein_ids() works", {

    seq_path <- system.file(
        "extdata", "RefSeq_parsing_example", package = "syntenet"
    )
    seq <- fasta2AAStringSetlist(seq_path)
    annot <- gff2GRangesList(seq_path)
    names(seq$Aalosa) <- gsub(" .*", "", names(seq$Aalosa))
    cor_df <- as.data.frame(annot$Aalosa[annot$Aalosa$type == "CDS", ])
    cor_df <- cor_df[, c("Name", "gene")]
    protein2gene <- list(Aalosa = cor_df)
    new_seqs <- collapse_protein_ids(seq, protein2gene)
    
    # Start tests
    expect_true(is(new_seqs, "list"))
    expect_true(is(new_seqs[[1]], "AAStringSet"))
    
    expect_error(collapse_protein_ids(seq, protein2gene$Aalosa))
    
    seq2 <- seq
    names(seq2) <- "random_name"
    expect_error(collapse_protein_ids(seq2, protein2gene))
    
    pgene <- protein2gene
    pgene$Aalosa$Name <- pgene$Aalosa$gene
    expect_error(collapse_protein_ids(seq, pgene))
})

#' Plot a heatmap of phylogenomic profiles
#'
#' @param profile_matrix A matrix of phylogenomic profiles obtained
#' with \code{phylogenomic_profile}.
#' @param species_annotation A 2-column data frame with species IDs in 
#' the first column (same as column names of profile matrix), and species
#' annotation (e.g., higher-level taxonomic information) in the second column.
#' @param palette A character vector of colors or a character scalar with
#' the name of an RColorBrewer palette. Default: "RdYlBu".
#' @param dist_function Function to use to calculate a distance matrix for
#' synteny clusters. Popular examples include \code{stats::dist},
#' \code{labdsv::dsvdis}, and \code{vegan::vegdist}. Default: stats::dist.
#' @param dist_params A list with parameters to be passed to the function
#' specified in parameter \strong{dist_function}. 
#' Default: list(method = "euclidean").
#' @param clust_function Function to use to cluster the distance matrix
#' returned by the function specified in \code{dist_function}. Examples include
#' \code{stats::hclust} and \code{Rclusterpp::Rclusterpp.hclust}.
#' Default: stats::hclust.
#' @param clust_params A list with additional parameters (if any) to be
#' passed to the function specified in parameter \strong{clust_function}.
#' Default: list(method = "ward.D").
#' @param cluster_species Either a logical scalar (TRUE or FALSE) or 
#' a character vector with the order in which species should be arranged. 
#' TRUE or FALSE indicate whether hierarchical clustering should be applied
#' to rows (species). Ideally, the character vector
#' should contain the order of species in a phylogenetically meaningful way.
#' If users pass a named vector, vector names will be used to rename species.
#' If users have a species tree, they can read it 
#' with \code{treeio::read.tree()}, plot it with \code{ggtree::ggtree()}, 
#' and get the species order from the ggtree object 
#' with \code{ggtree::get_taxa_name()}. Default: FALSE.
#' @param show_colnames Logical indicating whether to show column names (i.e.,
#' cluster IDs) or not. Showing cluster IDs can be useful when visualizing
#' a small subset of them. When visualizing all clusters, cluster IDs are
#' impossible to read. Default: FALSE.
#' @param discretize Logical indicating whether to discretize clusters in
#' 4 categories: 0, 1, 2, and 3+. If FALSE, counts will be
#' log2 transformed. Default: TRUE.
#' @param ... Additional parameters to \code{pheatmap::pheatmap()}.
#'
#' @return A pheatmap object.
#' 
#' @export
#' @rdname plot_profiles
#' @importFrom pheatmap pheatmap
#' @importFrom grDevices colorRampPalette 
#' @importFrom RColorBrewer brewer.pal
#' @importFrom stats dist hclust
#' @examples 
#' data(clusters)
#' profile_matrix <- phylogenomic_profile(clusters)
#' species_order <- c(
#'     "vra", "van", "pvu", "gma", "cca", "tpr", "mtr", "adu", "lja",
#'     "Lang", "car", "pmu", "ppe", "pbr", "mdo", "roc", "fve",
#'     "Mnot", "Zjuj", "jcu", "mes", "rco", "lus", "ptr"
#' )
#' species_names <- c(
#'     "V. radiata", "V. angularis", "P. vulgaris", "G. max", "C. cajan",
#'     "T. pratense", "M. truncatula", "A. duranensis", "L. japonicus",
#'     "L. angustifolius", "C. arietinum", "P. mume", "P. persica",
#'     "P. bretschneideri", "M. domestica", "R. occidentalis", 
#'     "F. vesca", "M. notabilis", "Z. jujuba", "J. curcas",
#'     "M. esculenta", "R. communis", "L. usitatissimum", "P. trichocarpa"
#' )
#' names(species_order) <- species_names
#' species_annotation <- data.frame(
#'    Species = species_order,
#'    Family = c(rep("Fabaceae", 11), rep("Rosaceae", 6),
#'               "Moraceae", "Ramnaceae", rep("Euphorbiaceae", 3), 
#'               "Linaceae", "Salicaceae")
#' )
#' p <- plot_profiles(profile_matrix, species_annotation, 
#'                    cluster_species = species_order)
#'                    
#' p <- plot_profiles(profile_matrix, species_annotation, 
#'                    cluster_species = species_order, 
#'                    discretize = FALSE)
plot_profiles <- function(
        profile_matrix = NULL, 
        species_annotation = NULL,
        palette = "Greens",
        dist_function = stats::dist,
        dist_params = list(method = "euclidean"),
        clust_function = stats::hclust,
        clust_params = list(method = "ward.D"),
        cluster_species = FALSE,
        show_colnames = FALSE,
        discretize = TRUE, ...
) {
    # Set defaults
    breaks <- NA
    legend_labels <- NA
    palette <- colorRampPalette(brewer.pal(7, palette))(100)
    annot_row <- NA
    annot_colors <- NA
    
    # Handle colors and legend for species metadata
    if(is.data.frame(species_annotation)) {
        annot_row <- species_colors(species_annotation)$species_annotation
        annot_colors <- species_colors(species_annotation)$annotation_colors
    }
    
    # Should counts be discretized?
    if(discretize) {
        profile_matrix[profile_matrix >= 3] <- 3
        palette <- c("grey95", "#8FB0D7", "#FDC87A", "#F70C0E")
        breaks <- c(0, 1, 2, 3, 4)
        legend_labels <- c("0", "1", "2", "3+", "")
    } else {
        profile_matrix <- log2(profile_matrix + 1)
    }
    
    # Cluster columns
    ## Get a distance matrix
    dparams <- c(
        list(x = profile_matrix), dist_params
    )
    dist_mat <- do.call(dist_function, dparams)
    ## Cluster based on the distance matrix
    cparams <- c(
        list(d = dist_mat), clust_params
    )
    cluster_columns <- do.call(clust_function, cparams)

    # Handle reordering of species names and change names for better viz
    if(is.character(cluster_species)) {
        if(!all(cluster_species %in% colnames(profile_matrix))) {
            stop("'cluster_species' must match column names of profile matrix.")
        }
        profile_matrix <- profile_matrix[, cluster_species]
        if(!is.null(names(cluster_species))) {
            colnames(profile_matrix) <- names(cluster_species)
            if(is.data.frame(annot_row)) {
                annot_row <- annot_row[cluster_species, , drop = FALSE]
                rownames(annot_row) <- names(cluster_species)
            }
        }
        cluster_species <- FALSE
    }
    # Plot heatmap
    p <- pheatmap::pheatmap(
        t(profile_matrix), color = palette,
        cluster_cols = cluster_columns,
        cluster_rows = cluster_species,
        show_colnames = show_colnames,
        legend_breaks = breaks,
        legend_labels = legend_labels,
        annotation_row = annot_row,
        annotation_colors = annot_colors,
        ...
    )
    return(p)
}


#' Plot network
#' 
#' @param network The synteny network represented as an edge list, which is
#' a 2-column data frame with each member of the anchor pair in a column.
#' @param clusters A 2-column data frame with the variables \strong{Gene} 
#' and \strong{Cluster} representing gene ID and cluster ID, respectively,
#' exactly as returned by \code{cluster_network}.
#' @param cluster_id Character scalar or vector with cluster ID. If more than one 
#' cluster is passed as input, clusters are colored differently. 
#' @param color_by Either "cluster" or a 2-column data frame with 
#' gene IDs in the first column and variable to be used for 
#' coloring (e.g., taxonomic information) in the second column.
#' @param interactive Logical scalar indicating whether to display an 
#' interactive network or not. Default: FALSE.
#' @param dim_interactive Numeric vector of length 2 with the window 
#' dimensions of the interactive plot. If \strong{interactive} is set to FALSE,
#' this parameter is ignored.
#' 
#' @return A ggplot object with the network.
#' 
#' @importFrom ggnetwork ggnetwork theme_blank geom_edges geom_nodes
#' @importFrom igraph graph_from_data_frame
#' @importFrom ggplot2 aes ggplot scale_color_manual guides
#' @importFrom grDevices colorRampPalette
#' @import intergraph
#' @export
#' @rdname plot_network
#' @examples 
#' data(network)
#' data(clusters)
#' # Option 1: 1 cluster
#' cluster_id <- 25
#' plot_network(network, clusters, cluster_id)
#' 
#' # Option 2: 2 clusters
#' cluster_id <- c(25, 1089)
#' plot_network(network, clusters, cluster_id)
#' 
#' # Option 3: custom annotation for coloring
#' species_order <- c(
#'     "vra", "van", "pvu", "gma", "cca", "tpr", "mtr", "adu", "lja",
#'     "Lang", "car", "pmu", "ppe", "pbr", "mdo", "roc", "fve",
#'     "Mnot", "Zjuj", "jcu", "mes", "rco", "lus", "ptr"
#' ) 
#' species_annotation <- data.frame(
#'    Species = species_order,
#'    Family = c(rep("Fabaceae", 11), rep("Rosaceae", 6),
#'               "Moraceae", "Ramnaceae", rep("Euphorbiaceae", 3), 
#'               "Linaceae", "Salicaceae")
#')
#' genes <- unique(c(network$node1, network$node2))
#' gene_df <- data.frame(
#'     Gene = genes,
#'     Species = unlist(lapply(strsplit(genes, "_"), head, 1))
#' )
#' gene_df <- merge(gene_df, species_annotation)[, c("Gene", "Family")]
#' 
#' plot_network(network, clusters, cluster_id = 25, color_by = gene_df)
plot_network <- function(network = NULL, clusters = NULL, 
                         cluster_id = NULL, color_by = "cluster",
                         interactive = FALSE, 
                         dim_interactive = c(600, 600)) {
    
    requireNamespace("intergraph", quietly=TRUE)
    names(network) <- c("node1", "node2")
    #----Handle node attributes------------------------------------------------
    genes <- clusters[clusters$Cluster %in% cluster_id, ] # Att. 1: Cluster
    fedges <- network[network$node1 %in% genes$Gene, ]
    fedges <- fedges[fedges$node2 %in% genes$Gene, ]
    deg <- as.data.frame(table(c(fedges$node1, fedges$node2)))
    names(deg) <- c("Gene", "Degree")
    genes <- merge(genes, deg) # Attr. 2: Degree
    genes$Group <- as.factor(genes$Cluster) # Attr. 3: Group
    if(is.data.frame(color_by)) {
        genes$Group <- NULL
        names(color_by) <- c("Gene", "Group")
        genes <- merge(genes, color_by, by = "Gene")
        genes$Group <- as.factor(genes$Group)
    }
    graph <- graph_from_data_frame(fedges, directed = FALSE, vertices = genes)
    palette <- custom_palette(1)[seq_len(nlevels(genes$Group))]
    if(nlevels(genes$Group) > 20) {
        palette <- colorRampPalette(custom_palette(1))(nlevels(genes$Group))
    }
    if(interactive) {
        if(!requireNamespace("networkD3", quietly = TRUE)) {
            stop(
                "Package 'networkD3' is required for interactive visualizations.\n",
                "You can install it with `BiocManager::install('networkD3')`."
            )
        }
        graph_d3 <- networkD3::igraph_to_networkD3(graph, group = genes)
        d <- dim_interactive
        p <- networkD3::forceNetwork(
            Links = graph_d3$links, Nodes = graph_d3$nodes,
            Source = 'source', Target = 'target',
            NodeID = 'name', Group = 'Group', charge = -10,
            height = d[2], width = d[1], Nodesize = 'Degree',
            opacity=0.8, zoom = TRUE, fontSize = 12
        )
        
    } else {
        n <- ggnetwork::ggnetwork(graph, arrow.gap = 0)
        # Plot graph
        p <- ggplot2::ggplot(
            n, aes(x = .data$x, y = .data$y, xend = .data$xend, yend = .data$yend)
        ) +
            ggnetwork::geom_edges(color = "grey75", alpha = 0.5, 
                                  show.legend = FALSE) +
            ggnetwork::geom_nodes(
                aes(size = .data$Degree, color = .data$Group)
            ) +
            ggplot2::guides(size = "none") +
            ggplot2::scale_color_manual(values = palette) +
            ggnetwork::theme_blank()
    }
    return(p)
}

#----Load data------------------------------------------------------------------
data(network)
data(clusters)
profiles <- phylogenomic_profile(clusters)

## Data frame of species annotation
species_order <- c(
    "vra", "van", "pvu", "gma", "cca", "tpr", "mtr", "adu", "lja",
    "Lang", "car", "pmu", "ppe", "pbr", "mdo", "roc", "fve",
    "Mnot", "Zjuj", "jcu", "mes", "rco", "lus", "ptr"
)
species_names <- c(
    "V. radiata", "V. angularis", "P. vulgaris", "G. max", "C. cajan",
    "T. pratense", "M. truncatula", "A. duranensis", "L. japonicus",
    "L. angustifolius", "C. arietinum", "P. mume", "P. persica",
    "P. bretschneideri", "M. domestica", "R. occidentalis",
    "F. vesca", "M. notabilis", "Z. jujuba", "J. curcas",
    "M. esculenta", "R. communis", "L. usitatissimum", "P. trichocarpa"
)
names(species_order) <- species_names
species_annotation <- data.frame(
   Species = species_order,
   Family = c(rep("Fabaceae", 11), rep("Rosaceae", 6),
              "Moraceae", "Ramnaceae", rep("Euphorbiaceae", 3),
              "Linaceae", "Salicaceae")
)

# color_by data frame
genes <- unique(c(network$node1, network$node2))
gene_df <- data.frame(
    Gene = genes,
    Species = unlist(lapply(strsplit(genes, "_"), head, 1))
)
gene_df <- merge(gene_df, species_annotation)[, c("Gene", "Family")]
gene_df$Family[1:20] <- paste0("Random", 1:20)
    
#----Start tests----------------------------------------------------------------
test_that("plot_profiles() returns a heatmap", {
    
    p1 <- plot_profiles(profiles, discretize = FALSE)
    p2 <- plot_profiles(
        profiles, 
        species_annotation = species_annotation,
        cluster_species = species_order,
        discretize = TRUE
    )
    p3 <- plot_profiles(
        profiles, 
        species_annotation <- species_annotation[-1, ],
        discretize = FALSE
    )
    
    cspecies <- c(species_order[1:5], "fake_species")
    expect_error(
        plot_profiles(
            profiles, 
            species_annotation = species_annotation,
            cluster_species = cspecies,
            discretize = TRUE
        )
    )
    
    expect_true("pheatmap" %in% class(p1))
    expect_true("pheatmap" %in% class(p2))
    expect_true("pheatmap" %in% class(p3))
})


test_that("plot_network() returns a ggplot object", {
    
    p1 <- plot_network(
        network, 
        clusters, 
        cluster_id = c(
            227, 470, 277, 1059, 759, 685, 102, 210, 196, 48, 895, 635, 
            1097, 521, 1297, 303, 873, 258, 1175, 1212
        ), 
        color_by = gene_df
    )
    p2 <- plot_network(network, clusters, cluster_id = clusters$Cluster[1:6])
    p3 <- plot_network(network, clusters, cluster_id = 1, interactive = TRUE)
    
    expect_true("ggplot" %in% class(p1))
    expect_true("ggplot" %in% class(p2))
    expect_true("forceNetwork" %in% class(p3))
})

#' Detect intraspecies synteny
#'
#' @param blast_intra A list of BLAST/DIAMOND data frames for 
#' intraspecies comparisons as returned by \code{run_diamond()}.
#' @param annotation A processed GRangesList or CompressedGRangesList object 
#' as returned by \code{process_input()}. 
#' @param intra_dir Path to output directory where .collinearity files will
#' be stored.
#' @param anchors Numeric indicating the minimum required number of genes
#' to call a syntenic block. Default: 5.
#' @param max_gaps Numeric indicating the number of upstream and downstream
#' genes to search for anchors. Default: 25.
#' @param is_pairwise Logical indicating if only pairwise blocks should be 
#' reported. Default: TRUE.
#' @param verbose Logical indicating if log messages should be printed on
#' screen. Default: FALSE.
#' @param bp_param BiocParallel back-end to be used. 
#' Default: \code{BiocParallel::SerialParam()}.
#' @param ... Any additional arguments to the `MCScanX` algorithm. For a
#' complete list of all available options, see the man page 
#' of \code{rcpp_mcscanx_file()}.
#'
#' @return Paths to .collinearity files.
#' @importFrom methods is
#' @importFrom BiocParallel SerialParam bplapply
#' @rdname intraspecies_synteny
#' @export
#' @examples 
#' # Load data
#' data(scerevisiae_annot)
#' data(scerevisiae_diamond)
#' 
#' # Detect intragenome synteny
#' intra_syn <- intraspecies_synteny(
#'     scerevisiae_diamond, scerevisiae_annot
#' )
#' 
intraspecies_synteny <- function(
        blast_intra = NULL, 
        annotation = NULL, 
        intra_dir = file.path(tempdir(), "intra"),
        anchors = 5, max_gaps = 25, is_pairwise = TRUE,
        verbose = FALSE, 
        bp_param = BiocParallel::SerialParam(),
        ...
) {
    
    valid <- valid_blast(blast_intra) & valid_annot(annotation)
    if(!dir.exists(intra_dir)) { dir.create(intra_dir, recursive = TRUE) }
    
    # Get species ID length
    example_gene <- as.character(blast_intra[[1]][1, 1])
    species_id_length <- nchar(gsub("_.*", "", example_gene))
    
    intrasyn <- BiocParallel::bplapply(seq_along(blast_intra), function(x) {
        
        # Get species name
        blast_comp <- names(blast_intra)[x]
        all_species <- names(annotation)
        counts <- unlist(lapply(all_species, function(y) { 
            match1 <- grepl(paste0(y, "_"), blast_comp) 
            match2 <- grepl(paste0(y, "$"), blast_comp)
            return(match1 + match2)
        }))
        species <- all_species[counts == 2]
        
        # Write files
        ## BLAST output
        blast_file <- file.path(intra_dir, paste0(species, ".blast"))
        write.table(
            blast_intra[[x]], file = blast_file, quote = FALSE,
            row.names = FALSE, col.names = FALSE, sep = "\t"
        )
        
        ## Annotation (as a 4-column table)
        annot_df <- as.data.frame(annotation[[species]])
        annot_df <- annot_df[, c("seqnames", "gene", "start", "end")]
        
        gff_file <- file.path(intra_dir, paste0(species, ".gff"))
        write.table(
            annot_df, file = gff_file, quote = FALSE,
            row.names = FALSE, col.names = FALSE, sep = "\t"
        )
        
        # Detect synteny
        input <- file.path(intra_dir, species)
        rcpp_mcscanx_file(
            blast_file = blast_file, gff_file = gff_file, prefix = species,
            outdir = intra_dir, match_size = anchors, max_gaps = max_gaps,
            is_pairwise = is_pairwise, in_synteny = 1, verbose = verbose, 
            species_id_length = species_id_length, ...
        )
        
        # Delete intermediate files
        unlink(c(blast_file, gff_file))
        
        synf <- file.path(intra_dir, paste0(species, ".collinearity"))
        
        return(synf)
    }, BPPARAM = bp_param)
    intrasyn <- unlist(intrasyn)
    intrasyn <- intrasyn[!is.null(intrasyn)]
    
    return(intrasyn)
}


#' Detect interspecies synteny
#'
#' @param blast_inter A list of BLAST/DIAMOND data frames for 
#' interspecies comparisons as returned by \code{run_diamond()}.
#' @param annotation A processed GRangesList or CompressedGRangesList object 
#' as returned by \code{process_input()}.
#' @param inter_dir Path to output directory where .collinearity files will
#' be stored.
#' @param anchors Numeric indicating the minimum required number of genes
#' to call a syntenic block. Default: 5.
#' @param max_gaps Numeric indicating the number of upstream and downstream
#' genes to search for anchors. Default: 25.
#' @param is_pairwise specify if only pairwise blocks should be reported
#' Default: TRUE.
#' @param verbose Logical indicating if log messages should be printed on
#' screen. Default: FALSE.
#' @param bp_param BiocParallel back-end to be used. 
#' Default: \code{BiocParallel::SerialParam()}.
#' @param ... Any additional arguments to the `MCScanX` algorithm. For a
#' complete list of all available options, see the man page 
#' of \code{rcpp_mcscanx_file()}.
#'
#' @return Paths to .collinearity files.
#' 
#' @importFrom utils combn
#' @importFrom methods is
#' @importFrom BiocParallel bplapply SerialParam
#' @rdname interspecies_synteny
#' @export
#' @examples 
#' # Load data
#' data(proteomes)
#' data(blast_list)
#' data(annotation)
#' 
#' # Get DIAMOND and processed annotation lists
#' blast_inter <- blast_list[2]
#' annotation <- process_input(proteomes, annotation)$annotation
#'
#' # Detect interspecies synteny
#' intersyn <- interspecies_synteny(blast_inter, annotation)
interspecies_synteny <- function(
        blast_inter = NULL, 
        annotation = NULL,
        inter_dir = file.path(tempdir(), "inter"), 
        anchors = 5, max_gaps = 25, is_pairwise = TRUE,
        verbose = FALSE, 
        bp_param = BiocParallel::SerialParam(),
        ...
) {
    
    valid <- valid_blast(blast_inter) & valid_annot(annotation)
    if(!dir.exists(inter_dir)) { dir.create(inter_dir, recursive = TRUE) }
    species <- names(annotation)
    unique_comp <- combn(species, 2, simplify = FALSE)
    
    # Create a list with BLAST tables and annotation data frames to export
    minter <- lapply(unique_comp, function(x) {
        n1 <- x[1]
        n2 <- x[2]
        
        ## rbind GRanges into a single annotation data frame
        amerged <- suppressWarnings(c(annotation[[n1]], annotation[[n2]]))
        amerged <- as.data.frame(amerged)
        amerged <- amerged[, c("seqnames", "gene", "start", "end")]
        
        ## rbind BLAST tables into a single data frame
        blast1 <- paste0(n1, "_", n2)
        blast2 <- paste0(n2, "_", n1)
        bmerged <- rbind(blast_inter[[blast1]], blast_inter[[blast2]])
        
        res_list <- list(blast_table = bmerged, gff = amerged)
        return(res_list)
    })
    names(minter) <- unlist(lapply(unique_comp, function(x) {
        return(paste0(x[1], "_", x[2]))
    }))
    
    # Get species ID length
    example_chr <- as.character(minter[[1]]$gff[1, 1])
    species_id_length <- nchar(gsub("_.*", "", example_chr))
    
    # Detect synteny
    intersyn <- BiocParallel::bplapply(seq_along(minter), function(x) {
        
        sp <- names(minter)[x]
        blast_file <- file.path(inter_dir, paste0(sp, ".blast"))
        write.table(
            minter[[x]]$blast_table, file = blast_file, quote = FALSE,
            row.names = FALSE, col.names = FALSE, sep = "\t"
        )
        
        gff_file <- file.path(inter_dir, paste0(sp, ".gff"))
        write.table(
            minter[[x]]$gff, file = gff_file, quote = FALSE,
            row.names = FALSE, col.names = FALSE, sep = "\t"
        )
        
        # Detect synteny
        input <- file.path(inter_dir, sp)
        rcpp_mcscanx_file(
            blast_file = blast_file, gff_file = gff_file, prefix = sp,
            outdir = inter_dir, match_size = anchors, max_gaps = max_gaps,
            is_pairwise = is_pairwise, in_synteny = 2, verbose = verbose, 
            species_id_length = species_id_length, ...
        )
        
        # Delete intermediate files
        unlink(c(blast_file, gff_file))
        synf <- file.path(inter_dir, paste0(sp, ".collinearity"))
        return(synf)
    }, BPPARAM = bp_param)
    intersyn <- unlist(intersyn)
    intersyn <- intersyn[!is.null(intersyn)]
    
    return(intersyn)
}

#' Check if the names of list of sequences and annotations match.
#' 
#' @param seq A list of AAStringSet objects.
#' @param annotation A GRangesList, CompressedGRangesList, or list of
#' GRanges with the annotation for the sequences in \strong{seq}.
#' 
#' @return TRUE if the objects pass the check.
#' @noRd
#' @examples 
#' data(annotation)
#' data(proteomes)
#' seq <- proteomes
#' check_list_names(seq, annotation)
check_list_names <- function(seq = NULL, annotation = NULL) {
    
    annot_names <- names(annotation)
    seq_names <- names(seq)
    
    # Check for differences in both sets
    diff_seq <- setdiff(seq_names, annot_names)
    diff_annot <- setdiff(annot_names, seq_names)
    
    check <- TRUE
    if(is.null(annot_names) | is.null(seq_names)) {
        stop("List-like arguments 'seq' and 'annotation' must have names.")
    } else if(length(diff_seq) != 0 & length(diff_annot) == 0) {
        stop(
            "The following elements in `seq` were not found in `annotation`:\n",
            paste0(diff_seq, collapse = "\n")
        )
    } else if(length(diff_seq) == 0 & length(diff_annot) != 0) {
        stop(
            "The following elements in `annotation` were not found in `seq:`\n",
            paste0(diff_annot, collapse = "\n")
        )
    } else if(length(diff_seq) != 0 & length(diff_annot) != 0) {
        stop(
            "Element in `seq` but not in `annotation`: \n",
            paste0(diff_seq, collapse = "\n"),
            "\n\nElements in `annotation` but not in `seq`: \n",
            paste0(diff_annot, collapse = "\n")
        )
    }
    
    return(check)
}


#' Check if the number of sequences is less than the number of genes
#'
#' @param seq A list of AAStringSet objects.
#' @param annotation A GRangesList or CompressedGRangesList object 
#' with the annotation for the sequences in \strong{seq}.
#'
#' @return TRUE if the objects pass the check.
#' @noRd 
#' @importFrom utils capture.output
#' @examples 
#' data(proteomes)
#' data(annotation)
#' seq <- proteomes
#' check_ngenes(seq, annotation)
check_ngenes <- function(seq = NULL, annotation = NULL) {
    
    # Data frame of species and gene count based on annotation
    gene_count <- Reduce(rbind, lapply(seq_along(annotation), function(x) {
        count <- length(annotation[[x]][annotation[[x]]$type == "gene"])
        count_df <- data.frame(species = names(annotation)[x], ngenes = count)
        return(count_df)
    }))
    
    # Data frame of species and gene count based on sequences
    seq_count <- Reduce(rbind, lapply(seq_along(seq), function(x) {
        count <- length(seq[[x]])
        count_df <- data.frame(species = names(annotation)[x], nseqs = count)
        return(count_df)
    }))
    
    # Check if number of sequences is <= gene count (accounting for ncRNAs)
    counts <- merge(gene_count, seq_count, by = "species")
    check_count <- counts[counts$nseqs > counts$ngenes, ]
    if(nrow(check_count) > 0) {
        msg <- paste0(
            "One or more species have more sequences in `seq` than ", 
            "there are genes in `annotation`.\n",
            "Did you remember to keep only one protein isoform per gene?\n",
            "Problematic species:\n"
        )
        out <- capture.output(print(check_count, row.names = FALSE))
        stop(paste(c(msg, out), collapse = "\n"))
    }
    
    return(TRUE)
}


#' Check if sequence names match gene names from annotation
#'
#' @param seq A list of AAStringSet objects.
#' @param annotation A GRangesList or CompressedGRangesList object 
#' with the annotation for the sequences in \strong{seq}.
#' @param gene_field Character, name of the column in the GRanges objects
#' that contains gene IDs. Default: "gene_id".
#'
#' @return TRUE if the objects pass the check.
#' @noRd 
#' @importFrom GenomicRanges mcols
#' @importFrom utils capture.output head
#' @examples
#' data(annotation)
#' data(proteomes)
#' seq <- proteomes
#' check_gene_names(seq, annotation)
check_gene_names <- function(
        seq = NULL, annotation = NULL, gene_field = "gene_id"
) {
    
    seq_names <- lapply(seq, names)
    gene_names <- lapply(annotation, function(x) {
        ranges_cols <- GenomicRanges::mcols(x[x$type == "gene"])
        if(!gene_field %in% names(ranges_cols)) {
            stop("Could not find column '", gene_field, "' in GRanges.")
        }
        nam <- unique(ranges_cols[[gene_field]])
        
        return(nam)
    })
    
    # Check if names in `seq` match gene names in `annotation`
    check_names <- lapply(seq_along(seq_names), function(x) {
        sp <- names(seq_names)[x]
        diff <- seq_names[[x]][!seq_names[[x]] %in% gene_names[[sp]]]
        return(diff)
    })
    names(check_names) <- names(seq_names)
    
    # If there at least one species with a mismatch, show species name + info
    n_mismatch <- lengths(check_names)
    if(any(n_mismatch > 0)) {
        m <- names(n_mismatch[n_mismatch > 0])
        
        firstn <- function(x, n = 2) {
            return(lapply(x, function(y) paste(head(y, n), collapse = ",")))
        }
        m_df <- data.frame(
            species = m,
            sample_seqs = unlist(firstn(check_names[m])),
            sample_genes = unlist(firstn(gene_names[m]))
        )
        out <- capture.output(print(m_df, row.names = FALSE))
        msg <- paste0(
            "Sequence names in `seq` do not match gene names ", 
            "in `annotation` for the following ", length(m), " species:\n"
        )
        stop(paste(c(msg, out), collapse = "\n"))
    }
    
    return(TRUE)
}


#' Create a data frame of species IDs (3-5-character abbreviations)
#' 
#' @param species_names A character vector of names extracted from 
#' the \strong{seq} or \strong{annotation} lists, which can be extracted with
#' \code{names(seq)} or \code{names(annotation)}.
#' 
#' @return A 2-column data frame with the following variables:
#' \describe{
#'   \item{species_id}{Character, species ID consisting of 3-5 characters.}
#'   \item{species_name}{Character, original names passed as input.}
#' }
#' @export
#' @importFrom stats ave
#' @rdname create_species_id_table
#' @examples
#' # Load 'seq' list (list of AAStringSet objects)
#' data(proteomes)
#' 
#' # Create ID table
#' create_species_id_table(names(proteomes))
create_species_id_table <- function(species_names) {
    
    # Replace space with 'X'
    list_names <- gsub(" ", "X", species_names)
    
    # Get the minimum number of characters possible
    for(n in seq(3, 6)) { 
        abbrev <- substr(list_names, 1, n)
        count <- table(abbrev)
        if (!any(count > 1)) break
    }
    
    # Create a data frame of species IDs and names
    abbrev_df <- data.frame(
        species_id = abbrev,
        species_name = species_names
    )
    
    # If there are duplicated names with 5 characters, append LETTER to the end
    if(n > 5) {
        abbrev <- substr(list_names, 1, 5)
        abbrev_df <- data.frame(
            species_name = species_names,
            species_id = abbrev,
            count = as.numeric(ave(
                as.character(abbrev), abbrev, FUN = seq_along
            ))
        )
        abbrev_df$species_id <- ifelse(
            abbrev_df$count > 1, 
            paste0(substr(abbrev_df$species_id, 1, 4), LETTERS[abbrev_df$count]),
            abbrev_df$species_id
        )
        abbrev_df <- abbrev_df[, c("species_id", "species_name")]
    }
    
    return(abbrev_df)
}

#' Wrapper to check if command is found in PATH
#'
#' @param cmd Command to test.
#' @param args Arguments for command.
#'
#' @author Fabricio Almeida-Silva
#' @return Logical indicating whether the command is in PATH or not.
#' @noRd
is_valid <- function(cmd = NULL, args = NULL) {
    found <- tryCatch(
        system2(cmd, args = args, stdout = FALSE, stderr = FALSE),
        error = function(e) return(FALSE),
        warning = function(w) return(FALSE)
    )
    if(!isFALSE(found)) {
        found <- TRUE
    }
    return(found)
}


#' Check if DIAMOND is installed
#'
#' @return Logical indicating whether DIAMOND is installed or not.
#' @export
#' @rdname diamond_is_installed
#' @examples
#' diamond_is_installed()
diamond_is_installed <- function() {
    valid <- is_valid(cmd = "diamond", args = "help")
    return(valid)
}


#' Check if last is installed
#'
#' @return Logical indicating whether last is installed or not.
#' @export
#' @rdname last_is_installed
#' @examples
#' last_is_installed()
last_is_installed <- function() {
    valid <- is_valid(cmd = "lastal", args = "-h")
    return(valid)
}


#' Get IQ-TREE version
#'
#' @return Numeric indicating IQ-TREE version, with either 1 or 2.
#' @export
#' @rdname iqtree_version
#' @examples
#' iqtree_version()
iqtree_version <- function() {
    
    is_v1 <- is_valid(cmd = "iqtree", args = "-h")
    is_v2 <- is_valid(cmd = "iqtree2", args = "-h")
    
    v <- NA
    if(is_v1) { 
        v <- 1 
    } else if(is_v2) {
        v <- 2
    }
    return(v)
}


#' Check if IQTREE is installed
#'
#' @return Logical indicating whether IQTREE is installed or not.
#' @export
#' @rdname iqtree_is_installed
#' @examples
#' iqtree_is_installed()
iqtree_is_installed <- function() {
    
    v <- iqtree_version()
    
    valid <- !is.na(v)
    return(valid)
}


#' Generate custom color palette
#'
#' @param pal Numeric specifying palette number, from 1 to 3.
#'
#' @return Character vector of custom color palette with 20 colors
#' @noRd
custom_palette <- function(pal = 1) {
    pal1 <- c("#1F77B4FF", "#FF7F0EFF", "#2CA02CFF", "#D62728FF",
              "#9467BDFF", "#8C564BFF", "#E377C2FF", "#7F7F7FFF",
              "#BCBD22FF", "#17BECFFF", "#AEC7E8FF", "#FFBB78FF",
              "#98DF8AFF", "#FF9896FF", "#C5B0D5FF", "#C49C94FF",
              "#F7B6D2FF", "#C7C7C7FF", "#DBDB8DFF", "#9EDAE5FF")
    
    pal2 <- c("#3182BDFF", "#E6550DFF", "#31A354FF", "#756BB1FF",
              "#636363FF", "#6BAED6FF", "#FD8D3CFF", "#74C476FF",
              "#9E9AC8FF", "#969696FF", "#9ECAE1FF", "#FDAE6BFF",
              "#A1D99BFF", "#BCBDDCFF", "#BDBDBDFF", "#C6DBEFFF",
              "#FDD0A2FF", "#C7E9C0FF", "#DADAEBFF", "#D9D9D9FF")
    
    pal3 <- c("#393B79FF", "#637939FF", "#8C6D31FF", "#843C39FF",
              "#7B4173FF", "#5254A3FF", "#8CA252FF", "#BD9E39FF",
              "#AD494AFF", "#A55194FF", "#6B6ECFFF", "#B5CF6BFF",
              "#E7BA52FF", "#D6616BFF", "#CE6DBDFF", "#9C9EDEFF",
              "#CEDB9CFF", "#E7CB94FF", "#E7969CFF", "#DE9ED6FF")
    
    l <- list(pal1, pal2, pal3)
    l_final <- l[[pal]]
    return(l_final)
}


#' Wrapper to handle gene color annotation in heatmap
#'
#' @param species_annotation A 2-column data frame with species IDs in 
#' the first column (same as column names of profile matrix), and species
#' annotation (e.g., higher-level taxonomic information) in the second column.
#'
#' @return A list containing a data frame of species annotation and a list with
#' a named character vector of colors for each annotation category.
#' @noRd
#' @examples 
#' species_order <- c(
#'     "vra", "van", "pvu", "gma", "cca", "tpr", "mtr", "adu", "lja",
#'     "Lang", "car", "pmu", "ppe", "pbr", "mdo", "roc", "fve",
#'     "Mnot", "Zjuj", "jcu", "mes", "rco", "lus", "ptr"
#' ) 
#' species_annotation <- data.frame(
#'    Species = species_order,
#'    Family = c(rep("Fabaceae", 11), rep("Rosaceae", 6),
#'               "Moraceae", "Ramnaceae", rep("Euphorbiaceae", 3), 
#'               "Linaceae", "Salicaceae")
#')
#' species_colors(species_annotation)
species_colors <- function(species_annotation) {
    
    # First column to rownames
    rownames(species_annotation) <- species_annotation[,1]
    species_annotation[,1] <- NULL
    
    a_col <- colnames(species_annotation)[1] # Name of annotation column
    n_categories <- length(unique(species_annotation[[a_col]]))
    
    # List of named character vector with annotation colors for each category
    annotation_colors <- list(
        custom_palette(2)[seq_len(n_categories)]
    )
    names(annotation_colors) <- a_col
    names(annotation_colors[[a_col]]) <- unique(species_annotation[[a_col]])
    
    # Save results in a list
    results <- list(
        species_annotation = species_annotation,
        annotation_colors = annotation_colors
    )
    return(results)
}


#' Wrapper to check if input sequences have the expected class
#'
#' @param seq A list of AAStringSet objects, each list element containing
#' protein sequences for a given species. This list must have names 
#' (not NULL), and names of each list element must match the names of
#' list elements in \strong{annotation}.
#' 
#' @return TRUE if the input is valid.
#' @noRd
#' @importFrom methods is
valid_seq <- function(seq) {
    is_valid <- FALSE
    
    if(is(seq, "list") & is(seq[[1]], "AAStringSet")) {
        is_valid <- TRUE
    }
    
    if(!is_valid) { 
        stop("Input sequences must be in a list of AAStringSet objects.") 
    }
    
    return(is_valid)
}


#' Wrapper to check if input annotation has the expected class
#'
#' @param annotation A GRangesList, CompressedGRangesList, or list of
#' GRanges with the annotation for the sequences in \strong{seq}. This list must
#' have names (not NULL), and names of each list element must match the names
#' of list elements in \strong{seq}.
#' 
#' @return TRUE if the input is valid.
#' @noRd
#' @importFrom methods is
valid_annot <- function(annot) {

    if(is(annot, "list") | is(annot, "GRangesList") | 
       is(annot, "CompressedGRangesList")) {
        is_valid <- TRUE
    } else {
        stop("Input annotation must be in a GRangesList, CompressedGRangesList,
             or list of GRanges.") 
    }
    
    return(is_valid)
}


#' Wrapper to check if input BLAST list has the expected class
#'
#' @param blast_list A list of data frames, each data frame having 
#' the tabular output of BLASTp or similar programs, such as DIAMOND.
#' 
#' @return TRUE if the input is valid.
#' @noRd
#' @importFrom methods is
valid_blast <- function(blast_list) {

    if(is(blast_list, "list") & is(blast_list[[1]], "data.frame")) {
        is_valid <- TRUE
    } else {
        stop("BLAST list must be a list of data frames.")
    }
    
    return(is_valid)
}

#----Load data------------------------------------------------------------------
data(proteomes)
data(annotation)
data(blast_list)

## Simulate `proteomes` object without names
proteomes2 <- proteomes
names(proteomes2) <- NULL

## Simulate `proteomes` with names that do not match names in `annotation`
proteomes3 <- proteomes
names(proteomes3)[1] <- "different_name"

## Simulate `annotation` object with less genes than in `proteomes`
annotation2 <- annotation
annotation2$Olucimarinus <- annotation2$Olucimarinus[1:500]

## Simulate `annotation` without "Name" and "gene_id" columns
annotation3 <- annotation
annotation3$Olucimarinus$Name <- NULL
annotation3$Olucimarinus$gene_id <- NULL

## Simulate `annotation` with gene names that are different from `proteomes`
annotation4 <- annotation2
annotation4$Olucimarinus$gene_id[1:5] <- paste0("fakegene", 1:5) 

#----Start tests----------------------------------------------------------------
test_that("check_list_names() checks if names match", {
    
    check <- check_list_names(proteomes, annotation)

    expect_error(
        check_list_names(proteomes2, annotation)
    )
    
    expect_error(
        check_list_names(proteomes3, annotation)
    )
    
    expect_equal(check, TRUE)
})


test_that("check_ngenes() checks if number of seqs is <= gene count", {
    
    check <- check_ngenes(proteomes, annotation)
    
    expect_error(
        check_ngenes(proteomes, annotation2)
    )
    
    expect_equal(check, TRUE)
})


test_that("check_gene_names() checks if sequence names match gene names", {
    
    check <- check_gene_names(proteomes, annotation)
    
    expect_error(
        check_gene_names(proteomes, annotation3)
    )
    
    expect_error(
        check_gene_names(proteomes, annotation4)
    )
    
    expect_equal(check, TRUE)
})


test_that("create_species_id_table() creates a data frame of species IDs", {
    
    species_names <- c("Glycine max", "Glycine soja", "Olucimarinus")
    id1 <- create_species_id_table(species_names)
    
    expect_true(is(id1, "data.frame"))
    expect_equal(ncol(id1), 2)
    expect_true(all(nchar(id1$species_id) <=5))
    
})


test_that("is_valid() returns a logical scalar", {
    valid <- is_valid("echo", "Hello world")
    expect_equal(class(valid), "logical")
})

test_that("diamond_is_installed() returns a logical scalar", {
    diamond <- diamond_is_installed()
    expect_equal(class(diamond), "logical")
})

test_that("iqtree_is_installed() returns a logical scalar", {
    iqtree <- iqtree_is_installed()
    expect_equal(class(iqtree), "logical")
})

test_that("species_colors() returns a list for plot_profiles()", {
    species_order <- c(
        "vra", "van", "pvu", "gma", "cca", "tpr", "mtr", "adu", "lja",
        "Lang", "car", "pmu", "ppe", "pbr", "mdo", "roc", "fve",
        "Mnot", "Zjuj", "jcu", "mes", "rco", "lus", "ptr"
    ) 
    species_annotation <- data.frame(
        Species = species_order,
        Family = c(rep("Fabaceae", 11), rep("Rosaceae", 6),
                   "Moraceae", "Ramnaceae", rep("Euphorbiaceae", 3), 
                   "Linaceae", "Salicaceae")
    )
    l <- species_colors(species_annotation)
    expect_equal(class(l), "list")
    expect_equal(length(l), 2)
    
})

test_that("valid_seq() checks class of seq object", {
    
    check <- valid_seq(proteomes)
    
    expect_error(
        valid_seq(matrix(NA, 2, 2))
    )
    
    expect_equal(check, TRUE)
})

test_that("valid_annot() checks class of annotation object", {
    
    check <- valid_annot(annotation)
    
    expect_error(
        valid_annot(matrix(NA, 2, 2))
    )
    
    expect_equal(check, TRUE)
})


test_that("valid_blast() checks class of blast list object", {
    
    check <- valid_blast(blast_list)
    
    expect_error(
        valid_blast(matrix(NA, 2, 2))
    )
    
    expect_equal(check, TRUE)
})

test_that("iqtree_version() returns IQ-TREE version", {
    
    v <- iqtree_version()
    
    expect_true(v %in% c(1, 2, NA))
})

#' Create a data frame of bidirectional comparisons from unidirectional comparisons
#'
#' @param compare A 2-column data frame with name(s) of target species in 
#' column 1, and name(s) of outgroup species in column 2.
#' 
#' @return A 2-column data frame as in \strong{compare}, but extended to
#' contain bidirectional comparisons. If the data frame in \strong{compare} has
#' N rows, the output data frame should contain 2N rows.
#' 
#' @importFrom stats setNames
#' @export
#' @rdname make_bidirectional
#' @examples
#' spp_outgroup <- data.frame(species = "spA", outgroup = "spB")
#' comp_bi <- make_bidirectional(spp_outgroup)
#' comp_bi
make_bidirectional <- function(compare) {
    
    cnames <- names(compare)
    comp_df <- rbind(compare, setNames(compare[, c(2,1)], cnames))
    
    return(comp_df)
}


#' Collapse a list of bidirectional DIAMOND hits
#' 
#' @param blast_inter A list of data frames containing BLAST/DIAMOND tables 
#' with comparisons between target species and outgroups. 
#' BLASTp, DIAMOND, or similar programs must be run on processed sequence data 
#' as returned by \code{syntenet::process_input()}.
#' @param compare A 2-column data frame with target species name in column 1,
#' and outgroup species name in column 2. Species names must match names of
#' list elements in \strong{blast_inter}.
#'
#' @return A list of data frames with BLAST/DIAMOND tables as 
#' in \strong{blast_inter}, but with bidirectional hits combined. For instance,
#' if \strong{blast_inter} contains elements 'spA_spB' and 'spB_spA', these
#' two data frames are combined into a single data frame following the 
#' order indicated in \strong{outgroups} (i.e. 'column1_column2').
#' 
#' @export
#' @rdname collapse_bidirectional_hits
#' @examples
#' data(blast_list)
#' blast_inter <- blast_list[c(2,3)]
#' compare <- data.frame(species = "Olucimarinus", outgroup = "OspRCC809")
#' chits <- collapse_bidirectional_hits(blast_inter, compare)
collapse_bidirectional_hits <- function(blast_inter, compare) {
    
    final_list <- lapply(seq_len(nrow(compare)), function(n) {
        
        sp1 <- compare[n, 1]
        sp2 <- compare[n, 2]
        
        # Get data frames to combine
        df1 <- blast_inter[[paste0(sp1, "_", sp2)]]
        df2 <- blast_inter[[paste0(sp2, "_", sp1)]]
        
        # Combine data frames
        df_final <- rbind(df1, df2)
        
        return(df_final)
    })
    names(final_list) <- paste0(compare[, 1], "_", compare[, 2])
    
    return(final_list)
}


#' Wrapper to run DIAMOND from an R session
#'
#' @param seq A processed list of AAStringSet objects 
#' as returned by \code{process_input()}.
#' @param top_hits Number of top hits to keep in DIAMOND search. Default: 5.
#' @param verbose Logical indicating if progress messages should be printed.
#' Default: FALSE. 
#' @param outdir Output directory for DIAMOND results. By default, output files
#' are saved to a temporary directory.
#' @param threads Number of threads to use. Default: let DIAMOND auto-detect
#' and use all available virtual cores on the machine.
#' @param compare Character scalar indicating which comparisons
#' should be made when running DIAMOND. 
#' Possible modes are "all" (all-vs-all comparisons), 
#' "intraspecies" (intraspecies comparisons only), or
#' "interspecies" (interspecies comparisons only). Alternatively, users can
#' pass a 2-column data frame as input with the names of species to be 
#' compared.
#' @param ... Any additional arguments to `diamond blastp`.
#'
#' @return A list of data frames containing DIAMOND's tabular output
#' for each pairwise combination of species. For n species, the list length
#' will be \eqn{n^2}.
#' @export
#' @rdname run_diamond
#' @importFrom Biostrings writeXStringSet
#' @importFrom utils read.csv
#' @examples 
#' data(proteomes)
#' data(annotation)
#' seq <- process_input(proteomes, annotation)$seq[1:2]
#' if(diamond_is_installed()) {
#'     diamond_results <- run_diamond(seq)
#' }
run_diamond <- function(seq = NULL, top_hits = 5, verbose = FALSE, 
                        outdir = tempdir(), threads = NULL, 
                        compare = "all", ...) {
    
    valid <- valid_seq(seq)
    check_diamond <- diamond_is_installed()
    if(!check_diamond) {
        stop("Unable to find DIAMOND in PATH.")
    }
    
    # 1. Make dbs for each species
    if(verbose) { message("1. Creating database for each species...\n") }
    dirname <- paste0("diamond_", format(Sys.time(), "%Y-%m-%d_%Hh%M-%S"))
    dbdir <- file.path(outdir, dirname, "dbs")
    seqdir <- file.path(outdir, dirname, "seqs")
    resdir <- file.path(outdir, dirname, "results")
    if(!dir.exists(dbdir)) { dir.create(dbdir, recursive = TRUE) }
    if(!dir.exists(seqdir)) { dir.create(seqdir, recursive = TRUE) }
    if(!dir.exists(resdir)) { dir.create(resdir, recursive = TRUE) }
    
    makedb <- lapply(seq_along(seq), function(x) {
        seqfile <- paste0(file.path(seqdir, names(seq)[x]), ".fasta")
        dbfile <- file.path(dbdir, names(seq)[x])
        Biostrings::writeXStringSet(seq[[x]], filepath = seqfile)
        dbargs <- c("makedb --in ", seqfile, "-d ", dbfile, "--quiet")
        rundb <- system2("diamond", args = dbargs)
    })
    
    if(!is.null(threads)) { threads <- paste0("-p ", threads) }
    # 2. Pairwise BLASTp-like search with DIAMOND
    if(verbose) { message("2. Running pairwise DIAMOND searches...\n")}
    comb_df <- get_comp(names(seq), compare = compare)
    diamond_blastp <- lapply(seq_len(nrow(comb_df)), function(x) {
        query <- paste0(file.path(seqdir, comb_df[x, 1]), ".fasta")
        db <- file.path(dbdir, comb_df[x, 2])
        outfile <- paste(comb_df[x, 1], comb_df[x, 2], sep = "_")
        outfile <- paste0(file.path(resdir, outfile), sep =  ".tsv")
        
        bargs <- c("blastp -q", query, "-d", db, "-o", outfile, threads,
                   "--max-hsps 1 -k", top_hits, "--quiet", ...)
        run_diamond <- system2("diamond", args = bargs)
    })
    
    final_list <- read_diamond(resdir)
    return(final_list)
}


#' Wrapper to run last from an R session
#'
#' @param seq A processed list of AAStringSet objects 
#' as returned by \code{process_input()}.
#' @param verbose Logical indicating if progress messages should be printed.
#' Default: FALSE. 
#' @param outdir Output directory for last results. By default, output files
#' are saved to a temporary directory.
#' @param threads Number of threads to use. Default: 1.
#' @param compare Character scalar indicating which comparisons
#' should be made when running last. 
#' Possible modes are "all" (all-vs-all comparisons), 
#' "intraspecies" (intraspecies comparisons only), or
#' "interspecies" (interspecies comparisons only). Alternatively, users can
#' pass a 2-column data frame as input with the names of species to be 
#' compared.
#' @param lastD last option D: query letters per random alignment. Default: 1e6.
#' @param ... Any additional arguments to `lastal`.
#'
#' @return A list of data frames containing last's tabular output
#' for each pairwise combination of species. For n species, the list length
#' will be \eqn{n^2}.
#' @export
#' @rdname run_last
#' @importFrom Biostrings writeXStringSet
#' @importFrom utils read.csv
#' @examples 
#' data(proteomes)
#' data(annotation)
#' seq <- process_input(proteomes, annotation)$seq[1:2]
#' if(last_is_installed()) {
#'     last_results <- run_last(seq)
#' }
run_last <- function(seq = NULL, verbose = FALSE, 
                        outdir = tempdir(), threads = 1, 
                        compare = "all", lastD=1e6, ...) {
    
    valid <- valid_seq(seq)
    check_last <- last_is_installed()
    if(!check_last) {
        stop("Unable to find last in PATH.")
    }
    
    # 1. Make dbs for each species
    if(verbose) { message("1. Creating database for each species...\n") }
    dirname <- paste0("last_", format(Sys.time(), "%Y-%m-%d_%Hh%M-%S"))
    dbdir <- file.path(outdir, dirname, "dbs")
    seqdir <- file.path(outdir, dirname, "seqs")
    resdir <- file.path(outdir, dirname, "results")
    if(!dir.exists(dbdir)) { dir.create(dbdir, recursive = TRUE) }
    if(!dir.exists(seqdir)) { dir.create(seqdir, recursive = TRUE) }
    if(!dir.exists(resdir)) { dir.create(resdir, recursive = TRUE) }
    
    makedb <- lapply(seq_along(seq), function(x) {
        seqfile <- paste0(file.path(seqdir, names(seq)[x]), ".fasta")
        dbfile <- file.path(dbdir, names(seq)[x])
        Biostrings::writeXStringSet(seq[[x]], filepath = seqfile)
        dbargs <- c("-p", "-cR01", "-P", threads, dbfile, seqfile)
        rundb <- system2("lastdb", args = dbargs)
    })
    
    # 2. Pairwise BLASTp-like search with last
    if(verbose) { message("2. Running pairwise last searches...\n")}
    comb_df <- get_comp(names(seq), compare = compare)
    last_blastp <- lapply(seq_len(nrow(comb_df)), function(x) {
        query <- paste0(file.path(seqdir, comb_df[x, 1]), ".fasta")
        db <- file.path(dbdir, comb_df[x, 2])
        outfile <- paste(comb_df[x, 1], comb_df[x, 2], sep = "_")
        outfile <- paste0(file.path(resdir, outfile), sep =  ".tsv")
        
        bargs <- c("-f", "BlastTab", "-P", threads, "-D", lastD, db, query, ..., ">", outfile)
        run_last <- system2("lastal", args = bargs)
    })
    
    final_list <- read_diamond(resdir)
    return(final_list)
}


#' Export processed sequences as FASTA files
#' 
#' @param seq A processed list of AAStringSet objects 
#' as returned by \code{process_input()}.
#' @param outdir Path to output directory where FASTA files will be stored.
#' 
#' @return Path to exported FASTA files.
#' 
#' @rdname export_sequences
#' @importFrom Biostrings writeXStringSet
#' @export
#' @examples
#' # Load data
#' data(proteomes)
#' data(annotation)
#' 
#' # Process data
#' pdata <- process_input(proteomes, annotation)
#' 
#' # Export data
#' outdir <- file.path(tempdir(), "example_test")
#' export_sequences(pdata$seq, outdir)
export_sequences <- function(seq = NULL, outdir = tempdir()) {
    
    if(!dir.exists(outdir)) { dir.create(outdir, recursive = TRUE) }
    
    # Iterate through sequences and export them
    paths <- unlist(lapply(seq_along(seq), function(x) {
        
        filename <- paste0(file.path(outdir, names(seq)[x]), ".fasta")
        w <- Biostrings::writeXStringSet(
            seq[[x]], 
            filepath = filename,
            compress = FALSE
        )
        return(filename)
    }))
    
    return(paths)
}


#' Read DIAMOND/BLAST tables as a list of data frames
#'
#' @param diamond_dir Path to directory containing the tabular output
#' of DIAMOND or similar programs (e.g., BLAST).
#'
#' @return A list of data frames with the tabular DIAMOND output.
#' 
#' @rdname read_diamond
#' @export
#' @importFrom utils read.csv
#' @examples
#' # Path to output directory
#' diamond_dir <- system.file("extdata", package = "syntenet")
#' 
#' # Read output
#' l <- read_diamond(diamond_dir)
read_diamond <- function(diamond_dir = NULL) {
    
    if(!dir.exists(diamond_dir) | is.null(diamond_dir)) {
        stop("Could not find the directory specified in 'diamond_dir'.")
    }
    
    # Read files as a list of data frames
    result_files <- list.files(diamond_dir, pattern = ".tsv", full.names = TRUE)
    final_list <- lapply(result_files, function(x) {
        df <- read.csv(x, header = FALSE, sep = "\t", comment.char = "#")
        names(df) <- c(
            "query", "db", "perc_identity", "length", "mismatches", 
            "gap_open", "qstart", "qend", "tstart", "tend",
            "evalue", "bitscore"
        )
        return(df)
    })
    names(final_list) <- gsub("\\.tsv", "", basename(result_files))
    
    return(final_list)
}

# Load data
data(proteomes)
data(annotation)
data(blast_list)

# Start tests
test_that("make_bidirectional() creates bidirectional comparisons", {
    spp_outgroup <- data.frame(species = "spA", outgroup = "spB")
    comp_bi <- make_bidirectional(spp_outgroup)
    
    n1 <- as.integer(nrow(spp_outgroup))
    n2 <- as.integer(nrow(comp_bi))
    expect_true(n2 == 2L*n1)
})


test_that("collapse_bidirectional_hits() collapses hits", {
    
    blast_inter <- blast_list[c(2,3)]
    compare <- data.frame(species = "Olucimarinus", outgroup = "OspRCC809")
    chits <- collapse_bidirectional_hits(blast_inter, compare)
    
    expect_equal(length(chits), 1)
})


test_that("run_diamond() runs DIAMOND on the background", {
    
    seq <- process_input(proteomes, annotation)$seq[1:2]
    if(diamond_is_installed()) {
        diamond <- run_diamond(seq, verbose = TRUE, threads = 1)
    } else {
        diamond <- list(
            A = data.frame(),
            B = data.frame(),
            C = data.frame(),
            D = data.frame()
        )
    }
    expect_equal(class(diamond), "list")
    expect_equal(length(diamond), length(seq) ^ 2)
    expect_equal(class(diamond[[1]]), "data.frame")
})


test_that("export_sequences() exports processed sequences to FASTA files", {
    
    # Process data
    pdata <- process_input(proteomes, annotation)

    # Export data
    outdir <- file.path(tempdir(), "example_testthat")
    p <- export_sequences(pdata$seq, outdir)
    
    expect_equal(class(p), "character")
    
})


test_that("read_diamond() creates a list of data frames", {
    
    diamond_dir <- system.file("extdata", package = "syntenet")

    # Read output
    l <- read_diamond(diamond_dir)
    
    expect_error(read_diamond())
    expect_equal(class(l), "list")
    expect_equal(class(l[[1]]), "data.frame")
})

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>",
    crop = NULL
)

## ----load_package, message=FALSE----------------------------------------------
library(syntenet)

## ----data_intra---------------------------------------------------------------
# Load example data sets
data(scerevisiae_annot)
head(scerevisiae_annot)

data(scerevisiae_diamond)
names(scerevisiae_diamond)
head(scerevisiae_diamond$Scerevisiae_Scerevisiae)

## ----intra_syn----------------------------------------------------------------
# Detect intragenome synteny
intra_syn <- intraspecies_synteny(
    scerevisiae_diamond, scerevisiae_annot
)

intra_syn # see where the .collinearity file is

# Get anchor pairs from .collinearity file
anchors <- parse_collinearity(intra_syn)
head(anchors)

## ----parse_blocks-------------------------------------------------------------
# Get synteny block information with `parse_collinearity()`
blocks <- parse_collinearity(intra_syn, as = "blocks")
head(blocks)

## ----parse_all----------------------------------------------------------------
# Get anchors and block data with `parse_collinearity()`
intrasyn_all <- parse_collinearity(intra_syn, as = "all")
head(intrasyn_all)

## ----make_bidirectional-------------------------------------------------------
# Create a data frame with comparisons to be made
comp <- data.frame(
    query = "spA",
    db = "spB"
)
comp

# Make comparisons bidirectional
comp_bi <- make_bidirectional(comp)
comp_bi

## ----run_diamond_compare, eval = FALSE----------------------------------------
# # NOTE: Not executed because object `seq` doesn't exist; for demo only
# dmd_inter <- run_diamond(seq, compare = comp_bi)

## ----example_inter------------------------------------------------------------
# Load list of DIAMOND tables
data(blast_list)
names(blast_list)

algae_inter <- blast_list[c(2,3)] # keep only intergenome comparisons
names(algae_inter)


# Get processed annotation
data(proteomes)     # A list of `AAStringSet` objects
data(annotation)    # A `GRangesList` object

pdata <- process_input(proteomes, annotation)
names(pdata$annotation)

## ----inter_syn----------------------------------------------------------------
# Detect interspecies synteny
intersyn <- interspecies_synteny(algae_inter, pdata$annotation)

intersyn # see where the .collinearity file is

# Parse collinearity file
## 1) Get anchor pairs
algae_anchors <- parse_collinearity(intersyn)
head(algae_anchors)

## 2) Get synteny blocks
algae_blocks <- parse_collinearity(intersyn, as = "blocks")
head(algae_blocks)

## 3) Get all data combined (blocks + anchor pairs)
algae_all <- parse_collinearity(intersyn, as = "all")
head(algae_all)

## ----sessionInfo--------------------------------------------------------------
sessionInfo()

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>",
    crop = NULL ## Related to https://stat.ethz.ch/pipermail/bioc-devel/2020-April/016656.html
)

## ----installation, eval=FALSE-------------------------------------------------
# if(!requireNamespace('BiocManager', quietly = TRUE))
#   install.packages('BiocManager')
# 
# BiocManager::install("syntenet")

## ----load_package, message=FALSE----------------------------------------------
# Load package after installation
library(syntenet)

## ----data---------------------------------------------------------------------
# Protein sequences
data(proteomes)
head(proteomes)

# Annotation (ranges)
data(annotation)
head(annotation)

## ----fasta2AAStringSetlist----------------------------------------------------
# Path to directory containing FASTA files
fasta_dir <- system.file("extdata", "sequences", package = "syntenet")
fasta_dir

dir(fasta_dir) # see the contents of the directory

# Read all FASTA files in `fasta_dir`
aastringsetlist <- fasta2AAStringSetlist(fasta_dir)
aastringsetlist

## ----gff2GRangesList----------------------------------------------------------
# Path to directory containing FASTA files
gff_dir <- system.file("extdata", "annotation", package = "syntenet")
gff_dir

dir(gff_dir) # see the contents of the directory

# Read all FASTA files in `fasta_dir`
grangeslist <- gff2GRangesList(gff_dir)
grangeslist

## ----check_input--------------------------------------------------------------
check_input(proteomes, annotation)

## ----process_input------------------------------------------------------------
pdata <- process_input(proteomes, annotation)

# Looking at the processed data
pdata$seq
pdata$annotation

## ----run_diamond--------------------------------------------------------------
data(blast_list)
if(diamond_is_installed()) {
    blast_list <- run_diamond(seq = pdata$seq)
}

## ----blast_inspect------------------------------------------------------------
# List names
names(blast_list)

# Inspect first data frame
head(blast_list$Olucimarinus_Olucimarinus)

## ----infer_syntenet-----------------------------------------------------------
# Infer synteny network
net <- infer_syntenet(blast_list, pdata$annotation)

# Look at the output
head(net)

## ----create_species_id_table--------------------------------------------------
# Get a 2-column data frame of species IDs and names
id_table <- create_species_id_table(names(proteomes))

id_table

## ----cluster_network----------------------------------------------------------
# Load example data and take a look at it
data(network)
head(network)

# Cluster network
clusters <- cluster_network(network)
head(clusters)

## ----phylogenomic_profile-----------------------------------------------------
# Phylogenomic profiling
profiles <- phylogenomic_profile(clusters)

# Exploring the output
head(profiles)

## ----plot_profiles------------------------------------------------------------
# 1) Create a named vector of custom species order to plot
species_order <- setNames(
    # vector elements
    c(
        "vra", "van", "pvu", "gma", "cca", "tpr", "mtr", "adu", "lja",
        "Lang", "car", "pmu", "ppe", "pbr", "mdo", "roc", "fve",
        "Mnot", "Zjuj", "jcu", "mes", "rco", "lus", "ptr"
    ),
    # vector names
    c(
        "V. radiata", "V. angularis", "P. vulgaris", "G. max", "C. cajan",
        "T. pratense", "M. truncatula", "A. duranensis", "L. japonicus",
        "L. angustifolius", "C. arietinum", "P. mume", "P. persica",
        "P. bretschneideri", "M. domestica", "R. occidentalis", 
        "F. vesca", "M. notabilis", "Z. jujuba",
        "J. curcas", "M. esculenta", "R. communis", 
        "L. usitatissimum", "P. trichocarpa"
    )
)
species_order

# 2) Create a metadata data frame containing the family of each species
species_annotation <- data.frame(
    Species = species_order,
    Family = c(
        rep("Fabaceae", 11), rep("Rosaceae", 6),
        "Moraceae", "Ramnaceae", rep("Euphorbiaceae", 3), 
        "Linaceae", "Salicaceae"
    )
)
head(species_annotation)


# 3) Plot phylogenomic profiles, but using Ruzicka distances
plot_profiles(
    profiles, 
    species_annotation, 
    cluster_species = species_order, 
    dist_function = labdsv::dsvdis,
    dist_params = list(index = "ruzicka")
)

## ----find_GS_clusters---------------------------------------------------------
# Find group-specific clusters
gs_clusters <- find_GS_clusters(profiles, species_annotation)

head(gs_clusters)

# How many family-specific clusters are there?
nrow(gs_clusters)

## ----heatmap_filtered---------------------------------------------------------
# Filter profiles matrix to only include group-specific clusters
idx <- rownames(profiles) %in% gs_clusters$Cluster
p_gs <- profiles[idx, ]

# Plot heatmap
plot_profiles(
    p_gs, species_annotation, 
    cluster_species = species_order, 
    cluster_columns = TRUE
)

## ----plot_network-------------------------------------------------------------
# 1) Visualize a network of first 5 GS-clusters
id <- gs_clusters$Cluster[1:5]
plot_network(network, clusters, cluster_id = id)

# 2) Coloring nodes by family
genes <- unique(c(network$node1, network$node2))
gene_df <- data.frame(
    Gene = genes,
    Species = unlist(lapply(strsplit(genes, "_"), head, 1))
)
gene_df <- merge(gene_df, species_annotation)[, c("Gene", "Family")]
head(gene_df)

plot_network(network, clusters, cluster_id = id, color_by = gene_df)

# 3) Interactive visualization (zoom out and in to explore it)
plot_network(
    network, clusters, cluster_id = id, 
    interactive = TRUE, dim_interactive = c(500, 300)
)

## ----binarize-----------------------------------------------------------------
bt_mat <- binarize_and_transpose(profiles)

# Looking at the first 5 rows and 5 columns of the matrix
bt_mat[1:5, 1:5]

## ----infer_phylogeny----------------------------------------------------------
# Leave only 6 legume species and P. mume as an outgroup for testing purposes
included <- c("gma", "pvu", "vra", "van", "cca", "pmu")
bt_mat <- bt_mat[rownames(bt_mat) %in% included, ]

# Remove non-variable sites
bt_mat <- bt_mat[, colSums(bt_mat) != length(included)]

if(iqtree_is_installed()) {
    phylo <- infer_microsynteny_phylogeny(bt_mat, outgroup = "pmu", 
                                          threads = 1)
}

## ----vis_phylogeny, message = FALSE, warning = FALSE, fig.height = 12, fig.width = 7----
data(angiosperm_phylogeny)

# Plotting the tree
library(ggtree)
ggtree(angiosperm_phylogeny) +
    geom_tiplab(size = 3) +
    xlim(0, 0.3)

## ----faq1---------------------------------------------------------------------
# Add example directory /home/username/bioinfo_tools to PATH
Sys.setenv(
    PATH = paste(
        Sys.getenv("PATH"), "/home/username/bioinfo_tools", sep = ":"
    )
)

## ----faq3-p1------------------------------------------------------------------
# Path to directory containing data
data_dir <- system.file(
    "extdata", "RefSeq_parsing_example", package = "syntenet"
)
dir(data_dir)

# Reading the files to a format that syntenet understands
seqs <- fasta2AAStringSetlist(data_dir)
annot <- gff2GRangesList(data_dir)

# Taking a look at the data
seqs
head(names(seqs$Aalosa))

annot

## ----faq3-p2------------------------------------------------------------------
# Remove whitespace and everything after it
names(seqs$Aalosa) <- gsub(" .*", "", names(seqs$Aalosa))

# Taking a look at the new names
head(names(seqs$Aalosa))

## ----faq3-p3------------------------------------------------------------------
# Show only rows for which `type` is "CDS"
head(annot$Aalosa[annot$Aalosa$type == "CDS"])

## ----faq-p4-------------------------------------------------------------------
# Create a list of data frames containing protein-to-gene ID correspondences
protein2gene <- lapply(annot, function(x) {
    
    # Extract only CDS ranges
    cds_ranges <- x[x$type == "CDS"]
    
    # Create the ID correspondence data frame
    df <- data.frame(
        protein_id = cds_ranges$Name,
        gene_id = cds_ranges$gene
    )
    
    # Remove duplicate rows
    df <- df[!duplicated(df$protein_id), ]
    
    return(df)
})

# Taking a look at the list
protein2gene

## ----faq-p5-------------------------------------------------------------------
# Collapse protein IDs to gene IDs in list of sequences
new_seq <- collapse_protein_ids(seqs, protein2gene)

# Looking at the new sequences
new_seq

## ----sessionInfo--------------------------------------------------------------
sessionInfo()

#' Cluster the synteny network using the Infomap algorithm
#'
#' @param network A network represented as an edge list, which is a 2-column
#' data frame with node 1 in the first column and node 2 in the second column.
#' In a synteny network, node 1 and node are the anchor pairs.
#' @param clust_function Function to be used to cluster the network. It must
#' be one the functions from the \strong{cluster_*} family in 
#' the \strong{igraph} package (e.g., cluster_infomap, cluster_leiden, etc). 
#' Default: igraph::cluster_infomap.
#' @param clust_params A list with additional parameters (if any) to be passed 
#' to the igraph clustering function. Default: NULL (no additional parameters).
#' 
#' @return A 2-column data frame with the following variables:
#' \describe{
#'   \item{Gene}{Gene ID.}
#'   \item{Cluster}{Cluster ID as identified by infomap.}
#' }
#'
#' @importFrom igraph graph_from_data_frame cluster_infomap simplify
#' @export
#' @rdname cluster_network
#' @examples 
#' data(network)
#' clusters <- cluster_network(network[1:500, ])
cluster_network <- function(network = NULL, 
                            clust_function = igraph::cluster_infomap,
                            clust_params = NULL) {
    
    # Create a graph and remove loops
    graph <- igraph::graph_from_data_frame(network, directed = FALSE)
    graph <- igraph::simplify(graph)

    # Cluster the graph    
    cparams <- c(
        list(graph = graph), clust_params
    )
    clusters <- do.call(clust_function, cparams)
    
    # Create a data frame of genes and their corresponding clusters
    clusters_df <- data.frame(
        Gene = clusters$names,
        Cluster = clusters$membership
    )
    return(clusters_df)
}


#' Perform phylogenomic profiling for synteny network clusters
#'
#' @param clusters A 2-column data frame with variables \strong{Gene} and
#' \strong{Cluster} as returned by \code{cluster_network}.
#' 
#' @return A matrix of i rows and j columns containing 
#' the number of genes in cluster i for each species j. The number of rows 
#' is equal to the number of clusters in \strong{clusters}, and 
#' the number of columns is equal to the number of species 
#' in \strong{clusters}.
#'
#' @export
#' @rdname phylogenomic_profile
#' @examples 
#' data(clusters)
#' profiles <- phylogenomic_profile(clusters)
phylogenomic_profile <- function(clusters = NULL) {
    
    # Add species info
    clusters$Species <- vapply(
        strsplit(clusters$Gene, "_"), `[`, 1, FUN.VALUE = character(1)
    )
    # Create a profile matrix
    profile_matrix <- table(clusters$Cluster,clusters$Species)
    profile_matrix <- matrix(
        profile_matrix, ncol = ncol(profile_matrix), 
        dimnames = dimnames(profile_matrix)
    )
    
    return(profile_matrix)
}


#' Find group-specific clusters based on user-defined species classification
#'
#' @param profile_matrix A matrix of phylogenomic profiles obtained
#' with \code{phylogenomic_profile}.
#' @param species_annotation A 2-column data frame with species IDs in 
#' the first column (same as column names of profile matrix), and species
#' annotation (e.g., higher-level taxonomic information) in the second column.
#' @param min_percentage Numeric scalar with the minimum percentage of species
#' in a group to consider group specificity. For instance,
#' if a given cluster is present in only 1 group of species, but in less
#' than \strong{min_percentage} of the species for this group, it will not
#' be considered a group-specific cluster. This filtering criterion is useful 
#' to differentiate group-specific clusters (e.g., family-specific) from 
#' subgroup-specific clusters (e.g., genus-specific). Default: 50.
#' 
#' @return A data frame with the following variables:
#' \describe{
#'   \item{Group}{To which group of species the cluster is specific.}
#'   \item{Percentage}{Percentage of species from the group that are
#'   represented by the cluster.}
#'   \item{Cluster}{Cluster ID.}
#' }
#' @importFrom stats reshape
#' @export
#' @rdname find_GS_clusters
#' @examples
#' data(clusters)
#' profile_matrix <- phylogenomic_profile(clusters)
#' 
#' # Species annotation
#' species_order <- c(
#'     "vra", "van", "pvu", "gma", "cca", "tpr", "mtr", "adu", "lja",
#'     "Lang", "car", "pmu", "ppe", "pbr", "mdo", "roc", "fve",
#'     "Mnot", "Zjuj", "hlu", "jcu", "mes", "rco", "lus", "ptr"
#' )
#' species_annotation <- data.frame(
#'    Species = species_order,
#'    Family = c(rep("Fabaceae", 11), rep("Rosaceae", 6),
#'               "Moraceae", "Ramnaceae", "Cannabaceae",
#'                rep("Euphorbiaceae", 3), "Linaceae", "Salicaceae")
#' )
#' gs_clusters <- find_GS_clusters(profile_matrix, species_annotation)
find_GS_clusters <- function(profile_matrix = NULL, 
                             species_annotation = NULL,
                             min_percentage = 50) {
    if(!is.data.frame(species_annotation)) {
        stop("Species annotation must be a 2-column data frame.")
    }
    diff <- setdiff(colnames(profile_matrix), species_annotation$Species)
    if(length(diff) != 0) {
        diffname <- paste0(diff, collapse = "\n")
        message("Could not find annotation for species: \n", diffname)
    }
    names(species_annotation) <- c("Species", "Group")
    freq_by_group <- as.data.frame(table(species_annotation$Group))
    n <- length(unique(species_annotation$Group))
    
    # Reshape to long format
    bmat <- as.data.frame(profile_matrix)
    bmat[bmat > 0] <- 1
    bmat_long <- reshape(bmat, idvar = "Cluster", ids = rownames(bmat),
                         times = names(bmat), timevar = "Species", 
                         varying = list(names(bmat)), direction = "long")
    names(bmat_long) <- c("Species", "Presence", "Cluster")
    
    # Add group information
    prof_grouped <- merge(bmat_long, species_annotation)
    
    # Look for group-specific clusters
    prof_list <- split(prof_grouped, prof_grouped$Cluster)
    gs_df <- Reduce(rbind, lapply(seq_along(prof_list), function(x) {
        gs <- NULL
        filt_df <- prof_list[[x]][prof_list[[x]]$Presence == 1, ]
        group_count <- as.data.frame(table(filt_df$Group))
        group_count <- merge(freq_by_group, group_count, by = "Var1", 
                             all.x = TRUE)
        group_count$perc <- (group_count$Freq.y / group_count$Freq.x) * 100
        group_count[is.na(group_count)] <- 0
        
        # How many groups have 0% of species?
        perc_zero <- sum(group_count$perc == 0)
        if(perc_zero == n - 1) {
            # How many families are represented by >x% of the genes?
            fgroup_count <- group_count[group_count$perc >= min_percentage, ]
            if(nrow(fgroup_count) == 1) {
                gs <- fgroup_count[, c("Var1", "perc")]
                gs$Cluster <- names(prof_list)[x]
            }
        }
        return(gs)
    }))
    
    if(is.data.frame(gs_df)) {
        names(gs_df) <- c("Group", "Percentage", "Cluster")
        gs_df$Percentage <- round(gs_df$Percentage, 2)
    }
    return(gs_df)
}

#----Setup----------------------------------------------------------------------
data(network)
data(clusters)
set.seed(123)

#----Start tests----------------------------------------------------------------
test_that("cluster_network() clusters genes using the infomap algorithm", {
    cluster <- cluster_network(network[1:100, ])
    expect_equal(class(cluster), "data.frame")
    expect_equal(class(cluster$Gene), "character")
    expect_equal(class(cluster$Cluster), "numeric")
    expect_equal(ncol(cluster), 2)
})


test_that("phylogenomic_profile() returns cluster profiles", {
    
    fclusters <- sample(clusters$Cluster, size = 50, replace = FALSE)
    fclusters <- clusters[clusters$Cluster %in% fclusters, ]
    profiles <- phylogenomic_profile(fclusters)
    
    expect_true("matrix" %in% class(profiles))
    expect_equal(nrow(profiles), length(unique(fclusters$Cluster)))
})

test_that("find_GS_clusters() finds group-specific clusters", {
    
    profile_matrix <- phylogenomic_profile(clusters)
    
    species_order <- c(
        "vra", "van", "pvu", "gma", "cca", "tpr", "mtr", "adu", "lja",
        "Lang", "car", "pmu", "ppe", "pbr", "mdo", "roc", "fve",
        "Mnot", "Zjuj", "hlu", "jcu", "mes", "rco", "lus", "ptr"
    )
    species_annotation <- data.frame(
       Species = species_order,
       Family = c(rep("Fabaceae", 11), rep("Rosaceae", 6),
                  "Moraceae", "Ramnaceae", "Cannabaceae",
                   rep("Euphorbiaceae", 3), "Linaceae", "Salicaceae")
    )
    
    gs_clusters <- find_GS_clusters(profile_matrix, species_annotation)
    
    expect_error(
        find_GS_clusters(profile_matrix, matrix(NA, 2, 2))
    )
    
    expect_equal(class(gs_clusters), "data.frame")
    expect_equal(names(gs_clusters), c("Group", "Percentage", "Cluster"))
})

#' Create a data frame of pairwise comparisons to make in \code{run_diamond}
#' 
#' @param species Character vector of species names.
#' @param compare Character scalar indicating which comparisons
#' should be made when running DIAMOND. 
#' Possible modes are "all" (all-vs-all comparisons), 
#' "intraspecies" (intraspecies comparisons only), or
#' "interspecies" (interspecies comparisons only). Alternatively, users can
#' pass a 2-column data frame as input with the names of species to be 
#' compared.
#'
#' @return A 2-column data frame with columns \strong{sp1} and \strong{sp2}
#' representing the pairwise species comparisons to be made.
#' @importFrom methods is
#' @noRd
#' @examples 
#' species <- c("Olucimarinus", "OspRCC809")
#' get_comp(species)
get_comp <- function(species, compare = "all") {

    # Create a data frame of all possible pairwise comparisons
    comb_df <- expand.grid(species, species, stringsAsFactors = FALSE)
    names(comb_df) <- c("sp1", "sp2")
    
    if(is(compare, "data.frame")) {
        if(ncol(compare) != 2) {
            stop("The data frame of species comparisons must have 2 columns.")
        }
        comb_df <- compare
        names(comb_df) <- c("sp1", "sp2")
        
        # Check if any species in input data frame is not in 'seq'
        sp <- unique(c(comb_df$sp1, comb_df$sp2))
        name_is_missing <- any(!sp %in% species)
        if(name_is_missing) {
            stop("All species names in 'compare' must be in 'seq'.")
        }
        
    } else if(compare == "intraspecies") {
        comb_df <- comb_df[comb_df$sp1 == comb_df$sp2, ]
        
    } else if(compare == "interspecies") {
        comb_df <- comb_df[comb_df$sp1 != comb_df$sp2, ]
        
        
    } else if(compare == "all") {
        comb_df <- comb_df
        
    } else {
        stop("Invalid argument passed to parameter 'compare'.")
    }
    return(comb_df)
}

#' Get edges (anchor pairs) from .collinearity files obtained with MCScan
#' 
#' @param collinearity_paths Character vector of paths to .collinearity files.
#' 
#' @return A data frame with variables:
#' \describe{
#'   \item{Block}{Numeric, synteny block ID}
#'   \item{Anchor1}{Character, gene ID of anchor 1.}
#'   \item{Anchor2}{Character, gene ID of anchor 2.}
#' }
#' 
#' @importFrom utils read.table
#' @noRd
edges_from_collinearity <- function(collinearity_paths = NULL) {
    
    edges <- lapply(seq_along(collinearity_paths), function(x) {
        lines <- readLines(collinearity_paths[x])
        nlines <- length(lines[!startsWith(lines, "#")])
        
        df <- NULL
        if(nlines > 0) {
            df <- read.table(collinearity_paths[x], sep = "\t", comment.char = "#")
            df <- df[, c(1, 2, 3)]
            names(df) <- c("Block", "Anchor1", "Anchor2")
            df$Block <- as.numeric(gsub("-.*", "", df$Block))
            
        }
        return(df)
    })
    
    edges <- Reduce(rbind, edges)
    return(edges)
}

#' Get synteny blocks from .collinearity files obtained with MCScan
#' 
#' @param collinearity_paths Character vector of paths to .collinearity files.
#' 
#' @return A data frame with variables:
#' \describe{
#'   \item{Block}{Numeric, synteny block ID}
#'   \item{Block_score}{Numeric, score of synteny block.}
#'   \item{Chr}{Character, query and target chromosome of the synteny
#'              block formatted as "<querychr>&<targetchr>".}
#'   \item{Orientation}{Character, the orientation of genes within blocks,
#'                      with "plus" indicating that genes are in the same 
#'                      direction, and "minus" indicating that genes are 
#'                      in opposite directions.}
#' }
#' 
#' @noRd
blocks_from_collinearity <- function(collinearity_paths = NULL) {
    
    blocks <- lapply(seq_along(collinearity_paths), function(x) {
        lines <- readLines(collinearity_paths[x])
        nlines <- length(lines[!startsWith(lines, "#")])
        
        block_df <- NULL
        if(nlines > 0) {
            
            # Extract synteny block information
            brows <- lines[grep("^## Alignment", lines)]
            binfo <- strsplit(gsub("## Alignment ", "", brows), split = " ")
            
            # Create a data frame with synteny block information
            block_df <- as.data.frame(do.call(rbind, binfo))[, c(1, 2, 5, 6)]
            names(block_df) <- c("Block", "Block_score", "Chr", "Orientation")
            block_df$Block <- as.numeric(gsub(":.*", "", block_df$Block))
            block_df$Block_score <- as.numeric(
                gsub("score=", "", block_df$Block_score)
            )
        }
        return(block_df)
    })
    
    blocks <- Reduce(rbind, blocks)
    return(blocks)
}


#' Parse .collinearity files obtained with MCScan
#' 
#' The .collinearity files can be obtained with \code{intraspecies_synteny}
#' and \code{interspecies_synteny}, which execute a native version of the
#' MCScan algorithm.
#'
#' @param collinearity_paths Character vector of paths to .collinearity files.
#' @param as Character specifying what to extract. 
#' One of "anchors" (default), "blocks", or "all".
#'
#' @return If \strong{as} is "anchors", a data frame with variables "Anchor1",
#' and "Anchor2". If \strong{as} is "blocks", a data frame with 
#' variables "Block", "Block_score", "Chr", and "Orientation". 
#' If \strong{as} is "all", a data frame with all aforementioned variables,
#' which indicate:
#' \describe{
#'   \item{Block}{Numeric, synteny block ID}
#'   \item{Block_score}{Numeric, score of synteny block.}
#'   \item{Chr}{Character, query and target chromosome of the synteny
#'              block formatted as "<querychr>&<targetchr>".}
#'   \item{Orientation}{Character, the orientation of genes within blocks,
#'                      with "plus" indicating that genes are in the same 
#'                      direction, and "minus" indicating that genes are 
#'                      in opposite directions.}
#'   \item{Anchor1}{Character, gene ID of anchor 1.}
#'   \item{Anchor2}{Character, gene ID of anchor 2.}
#' }
#'
#' @importFrom utils read.table
#' @export
#' @rdname parse_collinearity
#' @examples 
#' collinearity_paths <- system.file(
#'     "extdata", "Scerevisiae.collinearity", package = "syntenet"
#' )
#' net <- parse_collinearity(collinearity_paths)
parse_collinearity <- function(
        collinearity_paths = NULL, as = "anchors"
) {
    
    # Add clean names to file paths (basename)
    fname <- gsub("\\.collinearity", "", basename(collinearity_paths))
    names(collinearity_paths) <- fname
    
    # Get parsed files
    if(as == "anchors") {
        parsed <- edges_from_collinearity(collinearity_paths)
        parsed <- parsed[, c("Anchor1", "Anchor2")]
        
    } else if(as == "blocks") {
        parsed <- blocks_from_collinearity(collinearity_paths)
        
    } else if(as == "all") {
        parsed <- merge(
            blocks_from_collinearity(collinearity_paths),
            edges_from_collinearity(collinearity_paths)
        )
    } else {
        
        stop("Invalid argument passed to 'as' parameter.")
    }
    
    return(parsed)
}


#' Infer synteny network
#' 
#' @param blast_list A list of data frames, each data frame having 
#' the tabular output of BLASTp or similar programs, such as DIAMOND. 
#' This is the output of the function \code{run_diamond()}. If you performed
#' pairwise comparisons on the command line, you can read the tabular output
#' as data frames and combine them in a list. List names must be have species
#' names separated by underscore. For instance, if the first list element is
#' a data frame containing the comparison of speciesA (query) 
#' against speciesB (database), its name must be "speciesA_speciesB".
#' @param annotation A processed GRangesList, CompressedGRangesList, or
#' list of GRanges as returned by \code{process_input()}. 
#' @param outdir Path to the output directory. Default: tempdir().
#' @param anchors Numeric indicating the minimum required number of genes
#' to call a syntenic block. Default: 5.
#' @param max_gaps Numeric indicating the number of upstream and downstream
#' genes to search for anchors. Default: 25.
#' @param is_pairwise specify if only pairwise blocks should be reported
#' Default: TRUE
#' @param verbose Logical indicating if log messages should be printed on
#' screen. Default: FALSE.
#' @param bp_param BiocParallel back-end to be used. 
#' Default: \code{BiocParallel::SerialParam()}.
#' @param ... Any additional arguments to the `MCScanX` algorithm. For a
#' complete list of all available options, see the man page 
#' of \code{rcpp_mcscanx_file()}.
#' 
#' @return A network represented as an edge list.
#'
#' @export
#' @rdname infer_syntenet
#' @importFrom utils write.table
#' @examples
#' # Load data
#' data(proteomes)
#' data(annotation)
#' data(blast_list)
#'
#' # Create processed annotation list
#' annotation <- process_input(proteomes, annotation)$annotation
#'
#' # Infer the synteny network
#' net <- infer_syntenet(blast_list, annotation)
infer_syntenet <- function(
        blast_list = NULL, 
        annotation = NULL, 
        outdir = tempdir(),
        anchors = 5, max_gaps = 25, is_pairwise = TRUE, 
        verbose = FALSE, 
        bp_param = BiocParallel::SerialParam(),
        ...
) {

    valid <- valid_blast(blast_list) & valid_annot(annotation)
    
    # Separate intra from interspecies
    species <- names(annotation)
    equal_comp <- vapply(species, function(x) paste0(x, "_", x), character(1))
    idx_equal <- which(names(blast_list) %in% equal_comp)
    
    #---- 1) Intraspecies synteny detection------------------------------------
    dirname1 <- paste0("intrasyn_", format(Sys.time(), "%Y-%m-%d_%Hh%M-%S"))
    intra_dir <- file.path(outdir, dirname1)
    if(!dir.exists(intra_dir)) { dir.create(intra_dir, recursive = TRUE) }
    blast_intra <- blast_list[idx_equal]
    
    intraspecies <- intraspecies_synteny(
        blast_intra = blast_intra, 
        annotation = annotation,
        intra_dir = intra_dir, 
        anchors = anchors, 
        max_gaps = max_gaps, 
        is_pairwise = is_pairwise,
        verbose = verbose, 
        bp_param = bp_param,
        ...
    )
    
    #---- 2) Interspecies synteny detection------------------------------------
    dirname2 <- paste0("intersyn_", format(Sys.time(), "%Y-%m-%d_%Hh%M-%S"))
    inter_dir <- file.path(outdir, dirname2)
    if(!dir.exists(inter_dir)) { dir.create(inter_dir, recursive = TRUE) }
    blast_inter <- blast_list[-idx_equal]
    
    interspecies <- interspecies_synteny(
        blast_inter = blast_inter, 
        annotation = annotation, 
        inter_dir = inter_dir,
        anchors = anchors, 
        max_gaps = max_gaps, 
        is_pairwise = is_pairwise,
        verbose = verbose, 
        bp_param = bp_param,
        ...
    )
    
    # Create edge list
    syn_files <- c(intraspecies, interspecies)
    edges <- parse_collinearity(syn_files)
    return(edges)
}

#----Setup----------------------------------------------------------------------
data(proteomes)
data(annotation)
data(blast_list)
data(edges)

#----Start tests----------------------------------------------------------------
test_that("get_comp() returns a data frame of comparisons", {
    
    species <- c("sp1", "sp2")
    comp_df <- data.frame(sp1 = "sp1", sp2 = "sp2")
    t1 <- get_comp(species, compare = "all")
    t2 <- get_comp(species, compare = "intraspecies")
    t3 <- get_comp(species, compare = "interspecies")
    t4 <- get_comp(species, compare = comp_df)
    
    expect_error(
        get_comp(species, compare = cbind(comp_df, comp_df))
    )
    
    expect_error(
        get_comp(species, compare = data.frame(sp1 = "sp1", sp2 = "sp3"))
    )
    
    expect_error(
        get_comp(species, compare = "error")
    )
    
    expect_equal(nrow(t1), 4)
    expect_equal(nrow(t2), 2)
    expect_equal(nrow(t3), 2)
    expect_equal(nrow(t4), 1)
    expect_true(all(names(t1) %in% c("sp1", "sp2")))
})


test_that("run_diamond() runs DIAMOND on the background", {
    
    seq <- process_input(proteomes, annotation)$seq[1:2]
    if(diamond_is_installed()) {
        diamond <- run_diamond(seq, verbose = TRUE, threads = 1)
    } else {
        diamond <- list(
            A = data.frame(),
            B = data.frame(),
            C = data.frame(),
            D = data.frame()
        )
    }
    expect_equal(class(diamond), "list")
    expect_equal(length(diamond), length(seq) ^ 2)
    expect_equal(class(diamond[[1]]), "data.frame")
})


test_that("run_last() runs last on the background", {
    
    seq <- process_input(proteomes, annotation)$seq[1:2]
    if(last_is_installed()) {
        last <- run_last(seq, verbose = TRUE, threads = 1)
    } else {
        last <- list(
            A = data.frame(),
            B = data.frame(),
            C = data.frame(),
            D = data.frame()
        )
    }
    expect_equal(class(last), "list")
    expect_equal(length(last), length(seq) ^ 2)
    expect_equal(class(last[[1]]), "data.frame")
})


test_that("intraspecies_synteny() detects intraspecies synteny", {
    
    pdata <- process_input(proteomes, annotation)
    pannotation <- pdata$annotation[1]
    blast_intra <- blast_list[1]

    # Detect intraspecies synteny
    intrasyn <- intraspecies_synteny(blast_intra, pannotation)
    intrasyn2 <- intraspecies_synteny(
        blast_intra, pannotation, verbose = TRUE, is_pairwise = TRUE
    )

    expect_equal(class(intrasyn), "character")
    expect_equal(class(intrasyn2), "character")
    expect_equal(length(intrasyn), 1)
})

test_that("interspecies_synteny() detects interspecies synteny", {
    
    # Get DIAMOND and processed annotation lists
    blast_inter <- blast_list[2]
    pannotation <- process_input(proteomes, annotation)$annotation

    # Detect interspecies synteny
    intersyn <- interspecies_synteny(blast_inter, pannotation)
    
    expect_equal(class(intersyn), "character")
    expect_equal(length(intersyn), 1)
})

test_that("parse_collinearity() reads MCScanX output", {
    file <- system.file("extdata", "Scerevisiae.collinearity", package = "syntenet")
    
    net <- parse_collinearity(file)
    blocks <- parse_collinearity(file, as = "blocks")
    all <- parse_collinearity(file, as = "all")
    
    expect_error(parse_collinearity(file, as = "random_argument"))
    
    expect_equal(class(net), "data.frame")
    expect_equal(ncol(net), 2)
    
    expect_equal(class(blocks), "data.frame")
    expect_equal(ncol(blocks), 4)
    
    expect_equal(class(all), "data.frame")
    expect_equal(ncol(all), 6)
})


test_that("infer_syntenet() on example set find anchors", {
    processed <- process_input(proteomes, annotation) 
    annotation <- processed$annotation
    net <- infer_syntenet(blast_list, annotation)
    
    expect_equal(net, edges)
})

test_that("infer_syntenet() on example set find anchors verbose", {
    processed <- process_input(proteomes, annotation) 
    seq <- processed$seq
    annotation <- processed$annotation
    net <- infer_syntenet(blast_list, annotation, verbose = TRUE)
    
    expect_equal(net, edges)
})

test_that("infer_syntenet() on example set write html", {
    processed <- process_input(proteomes, annotation) 
    seq <- processed$seq
    annotation <- processed$annotation
    net <- infer_syntenet(blast_list, annotation, is_pairwise = FALSE)
    
    expect_equal(net, edges)
})

test_that("infer_syntenet() on example set write html verbose", {
    processed <- process_input(proteomes, annotation) 
    seq <- processed$seq
    annotation <- processed$annotation
    net <- infer_syntenet(blast_list, annotation, is_pairwise = FALSE,
        verbose = TRUE)
    
    expect_equal(net, edges)
})

#' Filtered proteomes of Ostreococcus sp. species
#'
#' Data obtained from Pico-PLAZA 3.0. Only the translated sequences of primary
#' transcripts were included, and only genes from chromosomes 1, 2, and 3.
#'
#' @name proteomes
#' @format A list of AAStringSet objects containing 
#' the elements `Olucimarinus`, `Osp_RCC809`, and `Otauri`.
#' @references
#' Van Bel, M., Silvestri, F., Weitz, E. M., Kreft, L., Botzki, A.,
#' Coppens, F., & Vandepoele, K. (2021). PLAZA 5.0: extending the scope
#' and power of comparative and functional genomics in plants.
#' Nucleic acids research.
#' @examples
#' data(proteomes)
#' @usage data(proteomes)
"proteomes"


#' Filtered genome annotation for Ostreococcus sp. species
#'
#' Data obtained from Pico-PLAZA 3.0. Only annotation data for primary
#' transcripts were included, and only genes for chromosomes 1, 2, and 3.
#'
#' @name annotation
#' @format A CompressedGRangesList containing 
#' the elements `Olucimarinus`, `Osp_RCC809`, and `Otauri`.
#' @references
#' Van Bel, M., Silvestri, F., Weitz, E. M., Kreft, L., Botzki, A.,
#' Coppens, F., & Vandepoele, K. (2021). PLAZA 5.0: extending the scope
#' and power of comparative and functional genomics in plants.
#' Nucleic acids research.
#' @examples
#' data(annotation)
#' @usage data(annotation)
"annotation"


#' List of data frames containing BLAST-like tabular output
#'
#' The object was created by running \code{run_diamond} on the protein
#' sequences for the Ostreococcus algae available in the \strong{proteomes}
#' example data. Hits with <50% identity were filtered out. Code to recreate
#' this data is available at the script/ subdirectory.
#' 
#' @name blast_list
#' @format A list of data frames containing the pairwise comparisons between
#' proteomes of Ostreococcus species.
#' @examples 
#' data(blast_list)
#' @usage data(blast_list)
"blast_list"


#' Synteny network of Ostreococcus genomes represented as an edge list
#'
#' The object was created by running \code{infer_syntenet} on the 
#' \strong{blast_list} example data. Code to recreate this data set is 
#' available at the script/ subdirectory.
#' 
#' @name edges
#' @format A data frame containing anchor pairs between two Ostreococcus
#' proteomes.
#' @examples 
#' data(edges)
#' @usage data(edges)
"edges"


#' Synteny network of BUSCO genes for 25 eudicot species
#'
#' Data obtained from Zhao & Schranz, 2019.
#'
#' @name network
#' @format An edgelist (i.e., a 2-column data frame with node 1 in 
#' column 1 and node 2 in column 2).
#' @references
#' Zhao, T., & Schranz, M. E. (2019). Network-based microsynteny analysis 
#' identifies major differences and genomic outliers in mammalian and 
#' angiosperm genomes. Proceedings of the National Academy of 
#' Sciences, 116(6), 2165-2174.
#' @examples
#' data(network)
#' @usage data(network)
"network"


#' Synteny network clusters of BUSCO genes for 25 eudicot species
#' 
#' Data obtained from Zhao & Schranz, 2019.
#'
#' @name clusters
#' @format A 2-column data frame containing the following variables:
#' \describe{
#'   \item{Gene}{Gene ID}
#'   \item{Cluster}{Cluster ID}
#' }
#' @references
#' Zhao, T., & Schranz, M. E. (2019). Network-based microsynteny analysis 
#' identifies major differences and genomic outliers in mammalian and 
#' angiosperm genomes. Proceedings of the National Academy of 
#' Sciences, 116(6), 2165-2174.
#' @examples
#' data(clusters)
#' @usage data(clusters)
"clusters"


#' Microsynteny-based angiosperm phylogeny.
#'
#' Original tree file obtained from Zhao et al., 2021.
#' The tree is an object of class 'phylo', which can be created by reading
#' the tree file with \code{treeio::read.tree()}.
#'
#' @name angiosperm_phylogeny
#' @format An object of class 'phylo'.
#' @references
#' Zhao, T., Zwaenepoel, A., Xue, J. Y., Kao, S. M., Li, Z., 
#' Schranz, M. E., & Van de Peer, Y. (2021). 
#' Whole-genome microsynteny-based phylogeny of angiosperms. 
#' Nature Communications, 12(1), 1-14.
#' @examples
#' data(angiosperm_phylogeny)
#' @usage data(angiosperm_phylogeny)
"angiosperm_phylogeny"


#' Genome annotation of the yeast species S. cerevisiae
#'
#' Data obtained from Ensembl Fungi. Only annotation data for primary
#' transcripts were included.
#' 
#' @name scerevisiae_annot
#' @format A GRangesList as returned by \code{process_input()} containing 
#' the element \strong{Scerevisiae}.
#' @examples
#' data(scerevisiae_annot)
#' @usage data(scerevisiae_annot)
"scerevisiae_annot"


#' Intraspecies DIAMOND output for S. cerevisiae
#'
#' List obtained with \code{run_diamond()}.
#' 
#' @name scerevisiae_diamond
#' @format A list of data frames (length 1) containing the whole paranome of
#' S. cerevisiae resulting from intragenome similarity searches.
#' @examples 
#' data(scerevisiae_diamond)
#' @usage data(scerevisiae_diamond)
"scerevisiae_diamond"

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

#' @useDynLib syntenet, .registration = TRUE
#' @import Rcpp
#' @title rcpp_mcscanx_file
#' @name rcpp_mcscanx_file
#' @description MCSCanX provides a clustering module for viewing the
#' relationship of collinear segments in multiple genomes (or heavily redundant
#' genomes). It takes the predicted pairwise segments from dynamic programming
#' (DAGchainer in particular) and then tries to build consensus segments from a
#' set of related, overlapping segments.
#' @return NULL, and a .collinearity file is created in the directory
#' specified in \strong{`outdir`}.
#' @param blast_file Character indicating the path to the BLAST/DIAMOND output
#' file.
#' @param gff_file Character indicating the path to the "gff" file, which
#' is a tab-delimited file with 4 columns indicating the chromosome name,
#' gene id, gene start position, and gene end position, respectively.
#' @param prefix Character indicating the prefix to output files. 
#' Default: "out".
#' @param outdir Character indicating the path to the output directory. 
#' Default: "".
#' @param match_score Numeric indicating the match score. Default: 50.
#' @param gap_penalty Numeric indicating the gap penalty. Default: -1.
#' @param match_size Numeric indicating the minimum number of genes 
#' required to call synteny. Default: 5.
#' @param e_value Numeric indicating the minimum e-value allowed. 
#' Default: 1e-5.
#' @param max_gaps Numeric indicating the maximum number of gaps between
#' genes allowed. The unit measure of gaps is number of genes, 
#' so \code{max_gaps = 20} indicates that a maximum of 20 genes can
#' exist between two homologous genes for synteny to be called. Default: 25.
#' @param overlap_window Numeric indicating the overlap window. Default: 5.
#' @param is_pairwise Logical indicating whether only pairwise blocks 
#' should be reported. Default: FALSE.
#' @param in_synteny Numeric indicating the patterns of collinear blocks,
#' where 0 indicates intra and interspecies comparisons, 1 indicates
#' intraspecies comparisons, and 2 indicates interspecies comparisons. 
#' Default: 0.
#' @param species_id_length Integer indicating the length of the species IDs.
#' Default: 3.
#' 0: intra- and inter-species (default); 1: intra-species; 2: inter-species
#' @param verbose Logical indicating whether to print progress messages
#' to the screen. Default: FALSE.
#' @references Wang et al. (2012) MCScanX: a toolkit for detection and
#' evolutionary analysis of gene synteny and collinearity.
#' \emph{Nucleic acids research}. \bold{40.7}, e49-e49.
#' @references Haas et al. (2004) DAGchainer: a tool for mining segmental
#' genome duplications and synteny. \emph{Bioinformatics}. \bold{20.18}
#' 3643-3646.
#' @export rcpp_mcscanx_file
#' @author Kristian K Ullrich and Fabricio Almeida-Silva
rcpp_mcscanx_file <- function(blast_file, gff_file, prefix = "out", outdir = "", match_score = 50L, gap_penalty = -1L, match_size = 5L, e_value = 1e-5, max_gaps = 25L, overlap_window = 5L, is_pairwise = FALSE, in_synteny = 0L, species_id_length = 3L, verbose = FALSE) {
    .Call(`_syntenet_rcpp_mcscanx_file`, blast_file, gff_file, prefix, outdir, match_score, gap_penalty, match_size, e_value, max_gaps, overlap_window, is_pairwise, in_synteny, species_id_length, verbose)
}

#' Read GFF/GTF files in a directory as a GRangesList object
#'
#' @param gff_dir Character indicating the path to the directory containing
#' GFF/GTF files.
#'
#' @return A GRangesList object, where each element represents a different
#' GFF/GTF file.
#' 
#' @export
#' @rdname gff2GRangesList
#' @importFrom GenomicRanges GRangesList
#' @examples 
#' gff_dir <- system.file("extdata", "annotation", package = "syntenet")
#' grangeslist <- gff2GRangesList(gff_dir)
gff2GRangesList <- function(gff_dir) {

    # Get full path to GFF/GFF3/GTF files    
    files <- list.files(gff_dir, pattern = "\\.gff|\\.gtf", full.names = TRUE)

    # Import files
    if(!requireNamespace("rtracklayer", quietly = TRUE)) {
        stop(
            "Package 'rtracklayer' is required to run `gff2GRangesList()`.\n",
            "You can install it with `BiocManager::install('rtracklayer')`."
        )
    }
    grangeslist <- lapply(files, rtracklayer::import)
    grangeslist <- GenomicRanges::GRangesList(grangeslist)
    
    # Name elements based on file names
    names(grangeslist) <- gsub("\\.gff.*|\\.gtf.*", "", basename(files))
    
    return(grangeslist)
}


#' Read FASTA files in a directory as a list of AAStringSet objects
#' 
#' @param fasta_dir Character indicating the path to the directory containing
#' FASTA files.
#' 
#' @return A list of AAStringSet objects, where each element represents a 
#' different FASTA file.
#' 
#' @export
#' @rdname fasta2AAStringSetlist
#' @importFrom Biostrings readAAStringSet 
#' @examples
#' fasta_dir <- system.file("extdata", "sequences", package = "syntenet")
#' aastringsetlist <- fasta2AAStringSetlist(fasta_dir)
fasta2AAStringSetlist <- function(fasta_dir) {
    
    # Get full path to FASTA files    
    files <- list.files(
        fasta_dir, pattern = "\\.fa|\\.fna", full.names = TRUE
    )
    
    # Read FASTA files as a list of `AAStringSet` objects
    seqs <- lapply(files, Biostrings::readAAStringSet)
    
    # Name list elements
    names(seqs) <- gsub("\\.fa.*|\\.fna.*", "", basename(files))
    
    return(seqs)
}

#----Load data------------------------------------------------------------------
fasta_dir <- system.file("extdata", "sequences", package = "syntenet")
gff_dir <- system.file("extdata", "annotation", package = "syntenet")

#----Start tests----------------------------------------------------------------
test_that("gff2GRangesList() reads GFF/GTF files as a GRangesList object", {
    
    grangeslist <- gff2GRangesList(gff_dir)
    
    expect_true(grepl("GRangesList", class(grangeslist)))
})

test_that("fasta2AAStringSetlist() reads FASTA files as list of AAStringSet", {
    
    aastringsetlist <- fasta2AAStringSetlist(fasta_dir)
    
    expect_equal(class(aastringsetlist), "list")
    expect_true("AAStringSet" %in% class(aastringsetlist[[1]]))
})

#' Binarize and transpose the phylogenomic profile matrix
#' 
#' @param profile_matrix A matrix with phylogenomic profiles obtained
#' with \code{phylogenomic_profile}.
#' 
#' @return A binary and transposed version of the profiles matrix.
#'
#' @export
#' @rdname binarize_and_transpose
#' @examples 
#' data(clusters)
#' profile_matrix <- phylogenomic_profile(clusters)
#' tmat <- binarize_and_transpose(profile_matrix)
binarize_and_transpose <- function(profile_matrix = NULL) {
    
    M <- profile_matrix

    # Make sure M is a numeric matrix of non-negative integer values
    if(!is.null(M)) {
        check_nm <- is.matrix(M) & is.numeric(M)
        check_nninteger <- as.vector(M >= 0 & M == round(M))
        bad_input <- any(c(check_nm, check_nninteger) == FALSE)
        if(bad_input) {
            stop("Input must be a matrix of non-negative integer values.")
        }
    }
    
    M[is.na(M)] <- 0
    M[M > 1] <- 1
    tM <- t(M)
    return(tM)
}


#' Save the transposed binary profiles matrix to a file in PHYLIP format
#'
#' @param transposed_profiles A binary and transposed profile matrix. 
#' The profile matrix can be obtained with \code{phylogenomic_profile()}.
#' @param outdir Path to output directory. By default, files are saved in
#' a temporary directory, so they will be deleted when the R session closes.
#' If you want to keep the files, specify a custom output directory.
#' 
#' @return Character specifying the path to the PHYLIP file.
#' @importFrom utils write.table
#' @export
#' @rdname profiles2phylip
#' @examples
#' data(clusters)
#' profile_matrix <- phylogenomic_profile(clusters)
#' tmat <- binarize_and_transpose(profile_matrix)
#' profiles2phylip(tmat)
profiles2phylip <- function(transposed_profiles = NULL, outdir = tempdir()) {
    
    dt <- format(Sys.time(), "%d_%b_%Y_%Hh%M")
    pat <- paste0("microsynteny_phylogeny_", dt)
    if(!dir.exists(outdir)) { 
        stop("Could not find directory specified in 'outdir'.") 
    }
    matrix_file <- file.path(outdir, paste0(pat, ".phy"))
    
    header <- paste(nrow(transposed_profiles), ncol(transposed_profiles))
    writeLines(header, matrix_file)
    write.table(transposed_profiles, file = matrix_file,
                col.names = FALSE, quote = FALSE, append = TRUE)
    
    return(matrix_file)
}

#' Infer microsynteny-based phylogeny with IQTREE
#'
#' @param transposed_profiles A binary and transposed profile matrix. 
#' The profile matrix can be obtained with \code{phylogenomic_profile()}.
#' @param bootr Numeric scalar with the number of bootstrap replicates.
#' Default: 1000.
#' @param alrtboot Numeric scalar with the number of replicates for
#' the SH-like approximate likelihood ratio test. Default: 1000.
#' @param threads Numeric scalar indicating the number of threads to use or
#' "AUTO", which allows IQTREE to automatically choose the best number 
#' of threads to use. Default: "AUTO".
#' @param model Substitution model to use. If you are unsure, pick the default.
#' Default: "MK+FO+R".
#' @param outdir Path to output directory. By default, files are saved in
#' a temporary directory, so they will be deleted when the R session closes.
#' If you want to keep the files, specify a custom output directory.
#' @param outgroup Name of outgroup clade to group the phylogeny. 
#' Default: NULL (unrooted phylogeny).
#' @param verbose Logical indicating if progress messages should be prompted.
#' Default: FALSE.
#' 
#' @return A character vector of paths to output files.
#' @export
#' @rdname infer_microsynteny_phylogeny
#' @examples 
#' data(clusters)
#' profile_matrix <- phylogenomic_profile(clusters)
#' tmat <- binarize_and_transpose(profile_matrix)
#' 
#' # Leave only some legumes and P. mume as an outgroup for testing purposes
#' included <- c("gma", "pvu", "vra", "van", "cca", "pmu")
#' tmat <- tmat[rownames(tmat) %in% included, ]
#' 
#' # Remove non-variable sites
#' tmat <- tmat[, colSums(tmat) != length(included)]
#' 
#' if(iqtree_is_installed()) {
#'     phylo <- infer_microsynteny_phylogeny(tmat, outgroup = "pmu", 
#'                                           threads = 1)
#' }
#'  
infer_microsynteny_phylogeny <- function(transposed_profiles = NULL,
                                         bootr = 1000, alrtboot = 1000,
                                         threads = "AUTO",
                                         model = "MK+FO+R",
                                         outdir = tempdir(),
                                         outgroup = NULL,
                                         verbose = FALSE) {
    
    if(!iqtree_is_installed()) { stop("Could not find IQ-TREE in PATH.") }
    
    # Create a file containing a header and the matrix below it
    matrix_file <- profiles2phylip(transposed_profiles, outdir)
    
    # Handle arguments
    stdout <- FALSE
    if(verbose) { stdout <- "" }
    
    root <- NULL
    if(!is.null(outgroup)) {
        root <- paste0("-o ", outgroup)
    }
    
    # Run IQ-TREE
    if(iqtree_version() == 1) { # IQ-TREE v1
        iqtree_args <- c(
            "-s ", matrix_file, "-bb", bootr, "-alrt", alrtboot, "-nt", threads, root, "-m", model, "-st MORPH -redo"
        )
        iqtree <- system2("iqtree", args = iqtree_args, stdout = stdout)
    } else { # IQ-TREE v2
        iqtree_args <- c(
            "-s ", matrix_file, 
            "-B", bootr, 
            "-alrt", alrtboot, 
            "-T", threads, 
            root,
            "-m", model,
            "-st MORPH -redo"
        )
        iqtree <- system2("iqtree2", args = iqtree_args, stdout = stdout)
    }
    
    paths <- list.files(
        path = outdir, pattern = basename(matrix_file), full.names = TRUE
    )
    return(paths)
}

#----Load data------------------------------------------------------------------
data(clusters)
profile_matrix <- phylogenomic_profile(clusters)
set.seed(123)

#----Start tests----------------------------------------------------------------
test_that("binarize_and_transpose() binarizes and transposes the profiles", {
    
    tmat <- binarize_and_transpose(profile_matrix)
    
    expect_error(
        binarize_and_transpose(matrix(NA, 2, 2))
    )
    
    expect_true("matrix" %in% class(tmat))
    expect_equal(nrow(profile_matrix), ncol(tmat))
    expect_equal(ncol(profile_matrix), nrow(tmat))
    expect_equal(length(which(tmat > 1)), 0)
})

test_that("profiles2phylip() returns path to a PHYLIP file", {
    
    tmat <- binarize_and_transpose(profile_matrix)
    path <- profiles2phylip(tmat)
    
    expect_error(
        profiles2phylip(tmat, outdir = file.path(tempdir(), "errordir"))
    )
    
    expect_equal(class(path), "character")
    expect_equal(length(path), 1)
})


test_that("infer_microsynteny_phylogeny() infers a phylogeny", {
    
    tmat <- binarize_and_transpose(profile_matrix)
    # Leave only some legumes and P. mume as an outgroup
    included <- c("gma", "pvu", "vra", "van", "cca", "pmu")
    tmat <- tmat[rownames(tmat) %in% included, ]
    
    # Remove non-variable sites
    tmat <- tmat[, colSums(tmat) != length(included)]
    
    phylo <- character(length = 10)
    if(iqtree_is_installed()) {
        phylo <- infer_microsynteny_phylogeny(
            tmat, outgroup = "pmu", threads = 1, verbose = TRUE
        )
    } 
    expect_equal(class(phylo), "character")
    expect_equal(length(phylo), 10)
})

---
title: "syntenet as a synteny detection tool"
author: 
  - name: Fabricio Almeida-Silva
    affiliation: 
    - VIB-UGent Center for Plant Systems Biology, Ghent, Belgium
    - Department of Plant Biotechnology and Bioinformatics, Ghent University, Ghent, Belgium
  - name: Tao Zhao
    affiliation:
    - State Key Laboratory of Crop Stress Biology for Arid Areas/Shaanxi Key Laboratory of Apple, College of Horticulture, Northwest A&F University, Yangling, China
  - name: Kristian K Ullrich
    affiliation:
    - Department of Evolutionary Biology, Max Planck Institute For Evolutionary Biology, Ploen, Germany
  - name: Yves Van de Peer
    affiliation: 
    - VIB-UGent Center for Plant Systems Biology, Ghent, Belgium
    - Department of Plant Biotechnology and Bioinformatics, Ghent University, Ghent, Belgium
    - College of Horticulture, Academy for Advanced Interdisciplinary Studies, Nanjing Agricultural University, Nanjing, China
    - Center for Microbial Ecology and Genomics, Department of Biochemistry, Genetics and Microbiology, University of Pretoria, Pretoria, South Africa
output: 
  BiocStyle::html_document:
    toc: true
    toc_float: true
    number_sections: yes
bibliography: vignette_bibliography.bib
vignette: >
  %\VignetteIndexEntry{syntenet as a synteny detection tool}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>",
    crop = NULL
)
```

# Introduction

Although `r BiocStyle::Biocpkg("syntenet")` was designed for large-scale
synteny analyses (i.e., with tens of genomes), users might also be interested
in using `r BiocStyle::Biocpkg("syntenet")` as a simple synteny detection
tool. For example, one can use `r BiocStyle::Biocpkg("syntenet")` to
identify syntenic regions in a genome (intraspecies synteny),
or to identify syntenic regions between the genomes of two species 
(interspecies synteny). To detect synteny, `r BiocStyle::Biocpkg("syntenet")` 
uses its own implementation of the MCScanX algorithm [@wang2012mcscanx], 
ported to R thanks to the `r BiocStyle::CRANpkg("Rcpp")` package for 
R and C++ integration.
In this vignette, we will guide you on how to perform intra and interspecies
synteny with `r BiocStyle::Biocpkg("syntenet")`.

```{r load_package, message=FALSE}
library(syntenet)
```

# Detecting intragenome synteny

To detect intragenome (or intraspecies) synteny, you will use the function
`intraspecies_synteny`. As input, you will need:

1. A `GRangesList` object containing the processed annotation for your 
species of interest, as returned by `process_input()`.

2. A list of data frames with the output of similarity search programs 
(e.g., DIAMOND, BLAST, etc). **Only intragenome comparisons must be included.**
This can be obtained with `run_diamond(seq, compare = "intraspecies")`.


To demonstrate the usage of `intraspecies_synteny()`, let's identify syntenic
regions in the genome of *Saccharomyces cerevisiae*. The processed annotation
and DIAMOND output are stored in the example data sets `scerevisiae_annot`
and `scerevisiae_diamond`.

```{r data_intra}
# Load example data sets
data(scerevisiae_annot)
head(scerevisiae_annot)

data(scerevisiae_diamond)
names(scerevisiae_diamond)
head(scerevisiae_diamond$Scerevisiae_Scerevisiae)
```

Once we have the data, we can detect synteny with `intraspecies_synteny()`.
This function returns the path to the *.collinearity* files generated by 
MCScanX [@wang2012mcscanx], which can be read and parsed with the
`parse_collinearity()` function.

```{r intra_syn}
# Detect intragenome synteny
intra_syn <- intraspecies_synteny(
    scerevisiae_diamond, scerevisiae_annot
)

intra_syn # see where the .collinearity file is

# Get anchor pairs from .collinearity file
anchors <- parse_collinearity(intra_syn)
head(anchors)
```

By default, the output of `parse_collinearity()` is a 2-column data 
frame with anchor pairs in syntenic regions. You can also extract data on each
synteny block (or synteny region) by specifying `as = 'blocks'`.

```{r parse_blocks}
# Get synteny block information with `parse_collinearity()`
blocks <- parse_collinearity(intra_syn, as = "blocks")
head(blocks)
```

You can even extract both anchor pairs and synteny block data at once in
a single data frame.

```{r parse_all}
# Get anchors and block data with `parse_collinearity()`
intrasyn_all <- parse_collinearity(intra_syn, as = "all")
head(intrasyn_all)
```

# Detecting intergenome synteny

To detect intergenome (or interspecies) synteny, you will use the function
`interspecies_synteny`, which works very similarly to `intraspecies_synteny()`.
As input, you will need:

1. A `GRangesList` object containing the processed annotation for your 
species of interest (2+ species), as returned by `process_input()`.

2. A list of data frames with the output of similarity search programs 
(e.g., DIAMOND, BLAST, etc). **Only intergenome comparisons must be included.**
This can be obtained with `run_diamond(seq, compare = compare_df)`.

Importantly, to detect intergenome synteny, the MCScanX algorithm requires
bidirectional BLAST hits. For instance, if you're trying to detect synteny
between `spA` and `spB`, you need to perform DIAMOND/BLAST searches in both
directions: `spA_spB` and `spB_spA`. You can create a data frame specifying
these comparisons as follows:

```{r make_bidirectional}
# Create a data frame with comparisons to be made
comp <- data.frame(
    query = "spA",
    db = "spB"
)
comp

# Make comparisons bidirectional
comp_bi <- make_bidirectional(comp)
comp_bi
```

Then, you can peform similarity searches for these comparisons with:

```{r run_diamond_compare, eval = FALSE}
# NOTE: Not executed because object `seq` doesn't exist; for demo only
dmd_inter <- run_diamond(seq, compare = comp_bi)
```

The code above would generate a list of two data frames with DIAMOND tables: 
one named `spA_spB`, and another one named `spB_spA`. 

To demonstrate what this looks like, we will load the example data set 
we will use to detect intergenome synteny. Here, we will detect syntenic
regions between the genomes of the algae *Ostreococcus lucimarinus* and 
*Ostreococcus sp RCC809*. The list of DIAMOND tables is stored in object
`blast_list`, and we will create the processed annotation from objects
`proteomes` and `annotation`.

```{r example_inter}
# Load list of DIAMOND tables
data(blast_list)
names(blast_list)

algae_inter <- blast_list[c(2,3)] # keep only intergenome comparisons
names(algae_inter)


# Get processed annotation
data(proteomes)     # A list of `AAStringSet` objects
data(annotation)    # A `GRangesList` object

pdata <- process_input(proteomes, annotation)
names(pdata$annotation)
```

Now that we have a list of DIAMOND tables with bidirectional comparisons 
between *Olucimarinus* and *OspRCC809*, as well as annotation for these two
genomes, we can detect synteny with `interspecies_synteny()`. Like
`intraspecies_synteny()`, this function will return the path to 
a *.collinearity* file generated by MCScanX, and we can parse this file
with `parse_collinearity()`.

```{r inter_syn}
# Detect interspecies synteny
intersyn <- interspecies_synteny(algae_inter, pdata$annotation)

intersyn # see where the .collinearity file is

# Parse collinearity file
## 1) Get anchor pairs
algae_anchors <- parse_collinearity(intersyn)
head(algae_anchors)

## 2) Get synteny blocks
algae_blocks <- parse_collinearity(intersyn, as = "blocks")
head(algae_blocks)

## 3) Get all data combined (blocks + anchor pairs)
algae_all <- parse_collinearity(intersyn, as = "all")
head(algae_all)
```


# Session information {.unnumbered}

This document was created under the following conditions:

```{r sessionInfo}
sessionInfo()
```

# References {.unnumbered}

Data acquisition
================

# Data in data/

## annotation.rda

Here, we will get the annotation for the *Ostreococcus sp.* algae
species from Pico-PLAZA 3.0 and filter them to include only chromosomes
1, 2, and 3 for these species for package size issues.

``` r
library(GenomicRanges)
links <- c(
    Olucimarinus = "ftp://ftp.psb.ugent.be/pub/plaza/plaza_pico_03/GFF/olu/annotation.selected_transcript.all_features.olu.gff3.gz",
    OspRCC809 = "ftp://ftp.psb.ugent.be/pub/plaza/plaza_pico_03/GFF/orcc809/annotation.selected_transcript.all_features.orcc809.gff3.gz"
)

annotation <- lapply(links, rtracklayer::import)
annotation <- lapply(annotation, function(x) {
    a <- x[x$type == "gene", ]
    a$Parent <- NULL
    a$name <- NULL
    a$phase <- NULL
    a$score <- NULL
    a$source <- NULL
    a$pid <- NULL
    if("chr_1" %in% seqnames(a)) {
        a <- keepSeqlevels(a, c("chr_1", "chr_2", "chr_3"),
                           pruning.mode = "coarse")
    } else {
        a <- keepSeqlevels(a, c("Chr_1", "Chr_2", "Chr_3"),
                           pruning.mode = "coarse")
    }
    return(a)
})
annotation <- GenomicRanges::GRangesList(annotation)

usethis::use_data(
    annotation, compress = "xz", overwrite = TRUE
)
```

## proteomes.rda

The protein sequences of the *Ostreococcus sp.* algae species (from
primary transcripts only) were obtained from Pico-PLAZA 3.0 and stored
in a list of AAStringSet objects. As above, only chromosomes 1, 2, and 3
were kept.

``` r
links <- c(
    Olucimarinus = "ftp://ftp.psb.ugent.be/pub/plaza/plaza_pico_03/Fasta/proteome.selected_transcript.olu.fasta.gz",
    OspRCC809 = "ftp://ftp.psb.ugent.be/pub/plaza/plaza_pico_03/Fasta/proteome.selected_transcript.orcc809.fasta.gz"
)

filt_genes <- unlist(lapply(annotation, function(x) return(x$gene_id)))

proteomes <- lapply(links, function(x) {
    seq <- Biostrings::readAAStringSet(x)
    names(seq) <- gsub("\\.[0-9] .*", "", names(seq))
    seq <- seq[names(seq) %in% filt_genes]
    return(seq)
})

usethis::use_data(
    proteomes, compress = "xz", overwrite = TRUE
)

# For some reason, the dramatic reduction in gene number does not affect
# much the size of the proteomes.rda file. Let's solve it by exporting 
# each sequence to a file, reading them, and saving again.
library(Biostrings)
data("proteomes")
dir <- tempdir()
seq1 <- proteomes$Olucimarinus
seq2 <- proteomes$OspRCC809

# Save .fa.gz files to tempdir
writeXStringSet(seq1, filepath = file.path(dir, "seq1.fa"), compress = "gzip")
writeXStringSet(seq2, filepath = file.path(dir, "seq2.fa"), compress = "gzip")

# Read files as AAStringSet objetcs and save them to a list
aa1 <- readAAStringSet(file.path(dir, "seq1.fa"))
aa2 <- readAAStringSet(file.path(dir, "seq2.fa"))
proteomes <- list(Olucimarinus = aa1, OspRCC809 = aa2)

usethis::use_data(
    proteomes, compress = "xz", overwrite = TRUE
)
```

## blast_list.rda

This object contains the blast list inferred from the example data, and
it is used in the vignette.

``` r
# Load data
data(proteomes)
data(annotation)

# Process data
pdata <- process_input(proteomes, annotation)

# Get blast list
blast_list <- run_diamond(seq = pdata$seq)
blast_list <- lapply(blast_list, function(x) return(x[x$perc_identity >= 50, ]))

# Save object
usethis::use_data(blast_list, compress = "xz", overwrite = TRUE)
```

## network.rda

The synteny network for BUSCO genes was downloaded from [this Dataverse
repository](https://dataverse.harvard.edu/dataset.xhtml?persistentId=doi:10.7910/DVN/BDMA7A),
which is associated with the paper [Network-based microsynteny analysis
identifies major differences and genomic outliers in mammalian and
angiosperm genomes](https://doi.org/10.1073/pnas.1801757116). The
dataset was filtered to only include the following species:

- *Vigna radiata*
- *Vigna angularis*
- *Phaseolus vulgaris*
- *Glycine max*
- *Cajanus cajan*
- *Trifolium pratense*
- *Medicago truncatula*
- *Arachis duranensis*
- *Lotus japonicus*
- *Lupinus angustifolius*
- *Cicer arietinum*
- *Prunus mume*
- *Prunus persica*
- *Pyrus x bretschneideri*
- *Malus domestica*
- *Rubus occidentalis*
- *Fragaris vesca*
- *Morus notabilis*
- *Ziziphus jujuba*
- *Jatropha curcas*
- *Manihot esculenta*
- *Ricinus communis*
- *Linum usitatissimum*
- *Populus trichocarpa*

``` r
#----Download file--------------------------------------------------------------
net_file <- file.path(tempdir(), "busco_network.txt.gz")
download.file("https://dataverse.harvard.edu/api/access/datafile/:persistentId?persistentId=doi:10.7910/DVN/BDMA7A/7JWAFI", destfile = net_file)

#----Read file and filter it----------------------------------------------------
busco_network <- readr::read_delim(net_file, show_col_types = FALSE, 
                           col_names = FALSE, delim = " ")

species <- unique(substr(c(busco_network$X1, busco_network$X2), 1, 5))
species <- unique(gsub("_.*", "", species))
species <- sort(unique(gsub("[0-9]*$", "", species)))

selected_species <- c(
    "vra", "van", "pvu", "gma", "cca", "tpr", "mtr", "adu", "lja",
    "Lang", "car", # Fabaceae
    "pmu", "ppe", "pbr", "mdo", "roc", "fve",
    "Mnot", "Zjuj", "hlu", 
    "jcu", "mes", "rco", "lus", "ptr"
)
s_patt <- paste0("^", selected_species)
s_patt <- paste0(s_patt, collapse = "|")

# Filtered network
busco_network <- busco_network[grepl(s_patt, busco_network$X1), ]
busco_network <- busco_network[grepl(s_patt, busco_network$X2), ]

# Clean names (i.e., separate species IDs by _)
not_underscore <- unique(substr(c(busco_network$X1, busco_network$X2), 1, 5))
not_underscore <- not_underscore[!grepl("_", not_underscore)]

busco_network$X1 <- gsub("Lang", "Lang_", busco_network$X1)
busco_network$X2 <- gsub("Lang", "Lang_", busco_network$X2)

busco_network$X1 <- gsub("Mnot", "Mnot_", busco_network$X1)
busco_network$X2 <- gsub("Mnot", "Mnot_", busco_network$X2)

busco_network$X1 <- gsub("Zjuj", "Zjuj_", busco_network$X1)
busco_network$X2 <- gsub("Zjuj", "Zjuj_", busco_network$X2)

network <- busco_network
network <- as.data.frame(network)
names(network) <- c("node1", "node2")

usethis::use_data(network, compress = "xz", overwrite = TRUE)
```

## edges.rda

``` r
# Load data
data(proteomes)
data(annotation)
data(blast_list)

# Process input data
pdata <- process_input(proteomes, annotation)

# Infer network
edges <- infer_syntenet(blast_list, pdata$annotation)

usethis::use_data(edges, compress = "xz")
```

## clusters.rda

The synteny network clusters for BUSCO genes were downloaded from [this
Dataverse
repository](https://dataverse.harvard.edu/dataset.xhtml?persistentId=doi:10.7910/DVN/BDMA7A),
which is associated with the paper [Network-based microsynteny analysis
identifies major differences and genomic outliers in mammalian and
angiosperm genomes](https://doi.org/10.1073/pnas.1801757116). The
dataset was filtered to only include the species mentioned above.

``` r
#----Download file--------------------------------------------------------------
cfile <- file.path(tempdir(), "clusters.txt.gz")
download.file("https://dataverse.harvard.edu/api/access/datafile/:persistentId?persistentId=doi:10.7910/DVN/BDMA7A/THEQD7", destfile = cfile)

#----Read file and filter it----------------------------------------------------
clusters <- readr::read_tsv(cfile, show_col_types = FALSE)
clusters <- clusters[grepl(s_patt, clusters$names), ]
names(clusters) <- c("Gene", "Cluster")

clusters$Gene <- gsub("Lang", "Lang_", clusters$Gene)
clusters$Gene <- gsub("Mnot", "Mnot_", clusters$Gene)
clusters$Gene <- gsub("Zjuj", "Zjuj_", clusters$Gene)

clusters <- as.data.frame(clusters)
usethis::use_data(clusters, compress = "xz", overwrite = TRUE)
```

## angiosperm_phylogeny.rda

Phylogenomic profiles for the whole genome of 123 angiosperm species
were downloaded from [this Dataverse
repository](https://dataverse.harvard.edu/dataset.xhtml?persistentId=doi:10.7910/DVN/7ZZWIH),
which is associated with the publication [Whole-genome
microsynteny-based phylogeny of
angiosperms](https://doi.org/10.1038/s41467-021-23665-0).

``` r
download.file(
    url = "https://dataverse.harvard.edu/api/access/datafile/:persistentId?persistentId=doi:10.7910/DVN/7ZZWIH/HFZBGE",
    destfile = file.path(tempdir(), "profiles.tar.gz"))

system2("tar", args = c("-zxvf", file))

#----Load file------------------------------------------------------------------
prof <- read.table("123genome_profiled_allsize", header = TRUE, sep = " ",
                   row.names = 1)
unlink("123genome_profiled_allsize")

#----Binarize and transpose profiles--------------------------------------------
transposed_profiles <- binarize_and_transpose(prof)

#----Infer microsynteny-based phylogeny-----------------------------------------
# Using Amborella trichopoda as an outgroup
phylo <- infer_microsynteny_phylogeny(transposed_profiles, outgroup = "atr")

angiosperm_phylogeny <- treeio::read.tree(phylo[10])

# Replace abbreviations with species names
angiosperm_phylogeny$tip.label
angiosperm_phylogeny$tip.label <- stringr::str_replace_all(
    angiosperm_phylogeny$tip.label, 
    c("^pmu" = "Prunus mume",
      "^ppe" = "Prunus persica",
      "^pbr" = "Pyrus x bretschneideri",
      "^Mald" = "Malus domestica",
      "^Rchi" = "Rosa chinensis",
      "^fve" = "Fragaria vesca",
      "^roc" = "Rubus occidentalis",
      "^Dryd" = "Dryas drummondii",
      "^Pand" = "Parasponia andersonii",
      "^Tori" = "Trema orientale",
      "^Mnot" = "Morus notabilis",
      "^Zjuj" = "Ziziphus jujuba",
      "^cme" = "Cucumis melo",
      "^csa" = "Cucumis sativus",
      "^cla" = "Citrullus lanatus",
      "^Cuma" = "Cucurbita maxima",
      "^Begf" = "Begonia fuchsioides",
      "^Datg" = "Datisca glomerata",
      "^van" = "Vigna angularis",
      "^vra" = "Vigna radiata",
      "^pvu" = "Phaseolus vulgaris",
      "^gma" = "Glycine max",
      "^cca" = "Cajanus cajan",
      "^mtr" = "Medicago truncatula",
      "^tpr" = "Trifolium pratense",
      "^car" = "Cicer arietinum",
      "^lja" = "Lotus japonicus",
      "^Lang" = "Lupinus angustifolius",
      "^Anan" = "Ammopiptanthus nanus",
      "^adu" = "Arachis duranensis",
      "^Cgla" = "Casuarina pendula",
      "^Bpen" = "Betula pendula",
      "^Cill" = "Carya illinoinensis",
      "^Qrob" = "Quercus robur",
      "^bnp" = "Brassica napus",
      "^bol" = "Brassica oleracea",
      "^bra" = "Brassica rapa",
      "^spa" = "Schrenkiella parvula",
      "^tsa" = "Thellungiella salsuginea",
      "^thh" = "Thellungiella halophila",
      "^ath" = "Arabidopsis thaliana",
      "^Alyr" = "Arabidopsis lyrata",
      "^Csat" = "Camelina sativa",
      "^cru" = "Capsella rubella",
      "^Bost" = "Boechera stricta",
      "^Lmey" = "Lepidium meyenii",
      "^Alp" = "Arabis alpina",
      "^aar" = "Aethionema arabicum",
      "^tha" = "Tarenaya hassleriana",
      "^cgy" = "Cleome gynandra",
      "^cpa" = "Carica papaya",
      "^Ghir" = "Gossypium hirsutum",
      "^Goba" = "Gossypium barbadense",
      "^gra" = "Gossypium raimondii",
      "^Dzib" = "Durio zibethinus",
      "^tca" = "Theobroma cacao",
      "^csi" = "Citrus sinensis",
      "^Cmax" = "Citrus maxima",
      "^Xsor" = "Xanthoceras sorbifolium",
      "^rco" = "Ricinus communis",
      "^mes" = "Manihot esculenta",
      "^ptr" = "Populus trichocarpa",
      "^lus" = "Linum usitatissimum",
      "^Pgra" = "Punica granatum",
      "^egr" = "Eucalyptus grandis",
      "^Cach" = "Capsicum chinense",
      "^Caba" = "Capsicum baccatum",
      "^can" = "Capsicum annuum",
      "^sly" = "Solanum lycopersicum",
      "^spe" = "Solanum pennellii",
      "^stu" = "Solanum tuberosum",
      "^pax" = "Petunia axillaris",
      "^Cuca" = "Cuscuta campestris",
      "^Inil" = "Ipomoea nil",
      "^coc" = "Coffea canephora",
      "^mgu" = "Mimulus guttatus",
      "^sin" = "Sesamum indicum",
      "^Oeur" = "Olea europaea",
      "^HanX" = "Helianthus annuus",
      "^Lsat" = "Lactuca sativa",
      "^dca" = "Daucus carota",
      "^Aeri" = "Actinidia eriantha",
      "^ach" = "Actinidia chinensis",
      "^bvu" = "Beta vulgaris",
      "^Cqui" = "Chenopodium quinoa",
      "^Ahyp" = "Amaranthus hypochondriacus",
      "^Kalf" = "Kalanchoe fedtschenkoi",
      "^Mole" = "Malania oleifera",
      "^vvi" = "Vitis vinifera",
      "^nnu" = "Nelumbo nucifera",
      "^Mcor" = "Macleaya cordata",
      "^Psom" = "Papaver somniferum",
      "^Aqco" = "Aquilegia coerulea",
      "^Sevi" = "Setaria viridis",
      "^sit" = "Setaria italica",
      "^Ecru" = "Echinochloa crus-galli",
      "^Sacc" = "Saccharum officinarum",
      "^sbi" = "Sorghum bicolor",
      "^Zmay" = "Zea mays",
      "^oth" = "Oropetium thomaeum",
      "^osa" = "Oryza sativa",
      "^ogl" = "Oryza glaberrima",
      "^oru" = "Oryza rufipogon",
      "^Opun" = "Oryza punctata",
      "^lpe" = "Leersia perrieri",
      "^HORV" = "Hordeum vulgare",
      "^Trdc" = "Triticum turgidum",
      "^bdi" = "Brachypodium distachyon",
      "^aco" = "Ananas comosus",
      "^mac" = "Musa acuminata",
      "^Pdac" = "Phoenix dactylifera",
      "^egu" = "Elaeis guineensis",
      "^Ashe" = "Apostasia shenzhenica",
      "^peq" = "Phalaenopsis equestris",
      "^Aoff" = "Asparagus officinalis",
      "^Xvis" = "Xerophyta viscosa",
      "^zom" = "Zostera marina",
      "^spo" = "Spirodela polyrhiza",
      "^CKAN" = "Cinnamomum kanehirae",
      "^Peam" = "Persea americana",
      "^Lchi" = "Liriodendron chinense",
      "^Nymp" = "Nymphaea colorata",
      "^atr" = "Amborella trichopoda"
      )
)


usethis::use_data(angiosperm_phylogeny, compress = "xz", overwrite = TRUE)
```

## scerevisiae_annot.rda + scerevisiae_diamond.rda

These files contain a processed annotation list (as returned by
`process_input()`) and a list of intragenome DIAMOND comparisons.

``` r
library(doubletrouble)

# Load data from {doubletrouble}
data("yeast_annot")
data("yeast_seq")
data("diamond_intra")

pdata <- process_input(yeast_seq, yeast_annot)

# Create objects
scerevisiae_annot <- pdata$annotation[1]
scerevisiae_diamond <- list(
    Scerevisiae_Scerevisiae = diamond_intra$Scerevisiae_Scerevisiae |>
        dplyr::filter(evalue <= 1e-10)
)

# Save objects
usethis::use_data(scerevisiae_annot, compress = "xz")
usethis::use_data(scerevisiae_diamond, compress = "xz", overwrite = TRUE)
```

# Data in inst/extdata

## Scerevisiae.collinearity

This file contains the intragenome synteny blocks for *S. cerevisiae*.

``` r
# Load data
data(scerevisiae_annot)
data(scerevisiae_diamond)

# Detect intragenome synteny
intra_syn <- intraspecies_synteny(
    scerevisiae_diamond, scerevisiae_annot
)

# Move file to inst/extdata
fs::file_move(
    path = intra_syn,
    new_path = here::here("inst", "extdata")
)

data(proteomes)
data(annotation)
processed <- process_input(proteomes, annotation) 
if(diamond_is_installed()) {
    blast_list <- run_diamond(processed$seq, ... = "--sensitive")
}

net <- infer_syntenet(blast_list, processed$annotation)

# Move file
olu_col <- file.path(tempdir(), "intraspecies_synteny", "Olu.collinearity")
out <- here::here("inst", "extdata")

fs::file_move(path = olu_col, new_path = out)
```

## sequences/

This directory contains FASTA files that correspond to a subset of the
sequences in the example data set `proteomes`.

``` r
data(proteomes)

# Create a list of vector with IDs of genes to subset - here, first 100 genes
genes_subset <- lapply(proteomes, function(x) names(x)[1:100]) 

# Path to sequences/ directory
seq_dir <- here::here("inst", "extdata", "sequences")

# Export sequences as FASTA files
Biostrings::writeXStringSet(
    proteomes$Olucimarinus[genes_subset$Olucimarinus], 
    filepath = file.path(seq_dir, "Olucimarinus.fa.gz"),
    compress = TRUE
)

Biostrings::writeXStringSet(
    proteomes$OspRCC809[genes_subset$OspRCC809], 
    filepath = file.path(seq_dir, "OspRCC809.fa.gz"), 
    compress = TRUE
)
```

## annotation/

This directory contains GFF3 files that correspond to a subset of the
ranges in the example data set `annotation`.

``` r
data(annotation)

# Path to annotation/ directory
annot_dir <- here::here("inst", "extdata", "annotation")

# Export ranges as GFF3 files
rtracklayer::export(
    annotation$Olucimarinus[annotation$Olucimarinus$gene_id %in% 
                                genes_subset$Olucimarinus, ], 
    con = file.path(annot_dir, "Olucimarinus.gff3.gz")
)
rtracklayer::export(
    annotation$OspRCC809[annotation$OspRCC809$gene_id %in%
                             genes_subset$OspRCC809, ], 
    con = file.path(annot_dir, "OspRCC809.gff3.gz")
)
```

## RefSeq_parsing_example/

This directory contains a subset of 10 genes from the *Alosa alosa*
genome (FASTA and GFF3 files) to demonstrate how to parse RefSeq data.

Gene annotation (GFF3) and whole-genome protein sequences (FASTA) were
downloaded from [NCBI’s Data
Hub](https://www.ncbi.nlm.nih.gov/data-hub/taxonomy/278164/) and
processed using the code below.

- *Aalosa.gff3.gz*

``` bash
# Bash

head -n 474 genomic.gff | awk '$3 == "gene" || $3 == "CDS" || $1 ~ /^#/' > Aalosa.gff3
gzip Aalosa.gff3
```

- *Aalosa.fa*

``` r
# Read files
ranges <- rtracklayer::import(here::here(
    "inst", "extdata", "RefSeq_parsing_example", "Aalosa.gff3.gz"
))

sequence <- Biostrings::readAAStringSet(
    "~/Downloads/protein.faa"
)

# Get proteins to extract from FASTA file
proteins <- unique(ranges[ranges$type == "CDS"]$Name)

# Subset sequences
pmatch <- paste0(proteins, collapse = "|")
final_seqs <- sequence[grep(pmatch, names(sequence))]

# Write to file
Biostrings::writeXStringSet(
    final_seqs,
    file = here::here(
        "inst", "extdata", "RefSeq_parsing_example", "Aalosa.fa.gz"
    ),
    compress = TRUE
)
```

---
title: "Inference and analysis of synteny networks"
author: 
  - name: Fabricio Almeida-Silva
    affiliation: 
    - VIB-UGent Center for Plant Systems Biology, Ghent, Belgium
    - Department of Plant Biotechnology and Bioinformatics, Ghent University, Ghent, Belgium
  - name: Tao Zhao
    affiliation:
    - State Key Laboratory of Crop Stress Biology for Arid Areas/Shaanxi Key Laboratory of Apple, College of Horticulture, Northwest A&F University, Yangling, China
  - name: Kristian K Ullrich
    affiliation:
    - Department of Evolutionary Biology, Max Planck Institute For Evolutionary Biology, Ploen, Germany
  - name: Yves Van de Peer
    affiliation: 
    - VIB-UGent Center for Plant Systems Biology, Ghent, Belgium
    - Department of Plant Biotechnology and Bioinformatics, Ghent University, Ghent, Belgium
    - College of Horticulture, Academy for Advanced Interdisciplinary Studies, Nanjing Agricultural University, Nanjing, China
    - Center for Microbial Ecology and Genomics, Department of Biochemistry, Genetics and Microbiology, University of Pretoria, Pretoria, South Africa
output: 
  BiocStyle::html_document:
    toc: true
    toc_float: true
    number_sections: yes
bibliography: vignette_bibliography.bib
vignette: >
  %\VignetteIndexEntry{Inference and analysis of synteny networks}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>",
    crop = NULL ## Related to https://stat.ethz.ch/pipermail/bioc-devel/2020-April/016656.html
)
```

# Introduction

The analysis of synteny (i.e., conserved gene content and order in a genomic
segment across species) can help understand the trajectory of duplicated 
genes through evolution. In particular, synteny analyses are widely used to
investigate the evolution of genes derived from whole-genome duplication (WGD)
events, as they can reveal genomic rearrangements that happened following the
duplication of all chromosomes. However, synteny analysis are typically
performed in a pairwise manner, which can be difficult to explore and interpret
when comparing several species. To understand global patterns of synteny,
@zhao2017network proposed a network-based approach to analyze synteny. 
In synteny networks, genes in a given syntenic block are represented as nodes
connected by an edge. Synteny networks have been used to explore, among others,
global synteny patterns in mammalian and angiosperm genomes [@zhao2019network], 
the evolution of MADS-box transcription factors [@zhao2017phylogenomic], 
and infer a microsynteny-based phylogeny for angiosperms [@zhao2021whole].
`r BiocStyle::Biocpkg("syntenet")` is a package that 
can be used to infer synteny networks from
protein sequences and perform downstream network analyses that include:

- **Network clustering** using the Infomap algorithm;

- **Phylogenomic profiling**, which consists in identifying which species 
contain which clusters. This analysis can reveal highly conserved synteny 
clusters and taxon-specific ones (e.g., family- and order-specific clusters);

- **Microsynteny-based phylogeny reconstruction** with maximum likelihood, 
which can be achieved by inferring a phylogeny from a binary matrix of 
phylogenomic profiles with IQ-TREE.


# Installation

`r BiocStyle::Biocpkg("syntenet")` can be installed from Bioconductor
with the following code:

```{r installation, eval=FALSE}
if(!requireNamespace('BiocManager', quietly = TRUE))
  install.packages('BiocManager')

BiocManager::install("syntenet")
```

```{r load_package, message=FALSE}
# Load package after installation
library(syntenet)
```

# Data description

For this vignette, we will use the proteomes and gene annotation of the algae 
species *Ostreococcus lucimarinus* and *Ostreococcus sp RCC809*, which were 
obtained from Pico-PLAZA 3.0 [@vandepoele2013pico]. 

```{r data}
# Protein sequences
data(proteomes)
head(proteomes)

# Annotation (ranges)
data(annotation)
head(annotation)
```

# Importing data to the R session

To detect synteny and infer synteny networks, 
`r BiocStyle::Biocpkg("syntenet")` requires two objects
as input:

- **seq:** A list of `AAStringSet` objects containing the translated sequences
of primary transcripts for each species.
- **annotation:** A `GRangesList` or `CompressedGRangesList` object containing
the coordinates for the genes in **seq**.


If you have whole-genome protein sequences in FASTA files, store all FASTA
files in the same directory and use the function `fasta2AAStringSetlist()` to
read all FASTA files into a list of `AAStringSet` objects.

Likewise, if you have gene annotation in GFF/GFF3/GTF files, 
store all files in the same directory and use the function `gff2GRangesList()`
to read all GFF/GFF3/GTF files into a `GRangesList object`.

For a demonstration, we will read example FASTA and GFF3 files stored in 
subdirectories named **sequences/** and **annotation/**, which are located 
in the `extdata/` directory of this package.

## From FASTA files to a list of `AAStringSet` objects

Here is how you can use `fasta2AAStringSetlist()` to read FASTA files 
in a directory as a list of `AAStringSet` objects.

```{r fasta2AAStringSetlist}
# Path to directory containing FASTA files
fasta_dir <- system.file("extdata", "sequences", package = "syntenet")
fasta_dir

dir(fasta_dir) # see the contents of the directory

# Read all FASTA files in `fasta_dir`
aastringsetlist <- fasta2AAStringSetlist(fasta_dir)
aastringsetlist
```

And that's it! Now you have a list of `AAStringSet` objects.

## From GFF/GTF files to a `GRangesList` object

Here is how you can use `gff2GRangesList()` to read GFF/GFF3/GTF files 
in a directory as a `GRangesList` object.

```{r gff2GRangesList}
# Path to directory containing FASTA files
gff_dir <- system.file("extdata", "annotation", package = "syntenet")
gff_dir

dir(gff_dir) # see the contents of the directory

# Read all FASTA files in `fasta_dir`
grangeslist <- gff2GRangesList(gff_dir)
grangeslist
```

And now you have a `GRangesList` object.

# Data preprocessing

The first part of the pipeline consists in processing the data to make it
match a standard structure. However, before processing the data for synteny 
detection, you must use the function `check_input()` to check if your data can
enter the pipeline. This function checks the input data for 3 
required conditions:

1. Names of **seq** list (i.e., `names(seq)`) match 
the names of **annotation** `GRangesList`/`CompressedGRangesList`
(i.e., `names(annotation)`)

2. For each species (list elements), the number of sequences
in **seq** is not greater than the number of genes 
in **annotation**. This is a way to ensure users do not input
the translated sequences for multiple isoforms of the same gene (generated
by alternative splicing). Ideally, the number of sequences in **seq** 
should be equal to the number of genes in **annotation**, but
this may not always stand true because of non-protein-coding genes.

3. For each species, sequence names (i.e., `names(seq[[x]])`,
equivalent to FASTA headers) match gene names in `annotation`.
By default, `r BiocStyle::Biocpkg("syntenet")` looks for gene IDs
in a column named "gene_id" in the GRanges objects (default field
in GFF3 files). If your gene IDs are in a different column (e.g., "Name"),
you can specify it in the *gene_field* parameter of `check_input()`
and `process_input()`.


Let's check if the example data sets satisfy these 3 criteria:

```{r check_input}
check_input(proteomes, annotation)
```

As you can see, the data passed the checks. Now, let's process them 
with the function `process_input()`.

This function processes the input sequences and annotation to:

1. Remove whitespace and anything after it in sequence names 
(i.e., `names(seq[[x]])`, which is equivalent to FASTA headers), if
there is any.

2. Add a unique species identifier to sequence names. The species 
identifier consists of the first 3-5 strings of the element name.
For instance, if the first element of the **seq** list is named
"Athaliana", each sequence in it will have an identifier "Atha_" added
to the beginning of each gene name (e.g., Atha_AT1G01010).

3. If sequences have an asterisk (*) representing stop codon, remove it.

4. Add a unique species identifier (same as above) to 
gene and chromosome names of each element of the **annotation**
`GRangesList`/`CompressedGRangesList`.

5. Filter each element of the **annotation** 
`GRangesList`/`CompressedGRangesList` to keep only seqnames, 
ranges, and gene ID.


Let's process our input data:

```{r process_input}
pdata <- process_input(proteomes, annotation)

# Looking at the processed data
pdata$seq
pdata$annotation
```

# Synteny network inference

Now that we have our processed data, we can infer the synteny network.
To detect synteny, we need the tabular output from BLASTp [@altschul1997gapped]
or similar programs. To get that, you can use the functions `run_diamond()`,
or `run_last()`, which runs DIAMOND [@buchfink2021sensitive] or last [@kielbasa2011adaptive]
from the R session and automatically parses its output to a list of data frames [^1].

[^1]: **Alternative:** if you want to use a different program for similarity
searches, you can run it on the command line, save the output in
a DIAMOND/BLAST-like tabular format, and read the output files
as a list of data frames with the function `read_diamond()` or `read_last()`
(see the FAQ for details).

Let's demonstrate how `run_diamond()` works. 
Needless to say, you need to have DIAMOND installed in your machine 
and in your PATH to run this function. To check if you have DIAMOND installed,
use the function `diamond_is_installed()` [^2].

[^2]: **Note:** in the code chunk below, the if statement is not required.
We just added it to make sure that the function `run_diamond()` is only
executed if DIAMOND is installed, to avoid problems when building this
vignette in machines that do not have DIAMOND installed. If you want to
reproduce the code in this vignette and do not have DIAMOND installed,
you can use the example output of `run_diamond()` stored in the *blast_list*
object (loaded with `data(blast_list)`).

```{r run_diamond}
data(blast_list)
if(diamond_is_installed()) {
    blast_list <- run_diamond(seq = pdata$seq)
}
```

The output of `run_diamond()` is a list of data frames containing the tabular
output of all-vs-all DIAMOND searches. Let's take a look at it.

```{r blast_inspect}
# List names
names(blast_list)

# Inspect first data frame
head(blast_list$Olucimarinus_Olucimarinus)
```

Now, we can use this list of DIAMOND data frames to detect synteny. Here,
we reimplemented the popular MCScanX algorithm [@wang2012mcscanx], originally
written in C++, using the `r BiocStyle::CRANpkg("Rcpp")` [@eddelbuettel2011rcpp] 
framework for R and C++ integration. This means that
`r BiocStyle::Biocpkg("syntenet")` comes with a native 
version of the MCScanX algorithm, so you can run MCScanX in R without
having to install it yourself. Amazing, huh?

To detect synteny and infer the synteny network, use the 
function `infer_syntenet()`. The output is a network represented as a
so-called **edge list** (i.e., a 2-column data frame with node 1 and node 2
in columns 1 and 2, respectively). 

```{r infer_syntenet}
# Infer synteny network
net <- infer_syntenet(blast_list, pdata$annotation)

# Look at the output
head(net)
```

In a synteny network, each row of the edge list represents an anchor pair. 
In the edge list above, for example, 
the genes `r net[1,1]` and `r net[1,2]` are an anchor pair (i.e., duplicates
derived from a large-scale duplication event).

Note that gene IDs are preceded by IDs created with `process_input()`.
Under the hood, `process_input()` uses the function `create_species_id_table()`
to create unique IDs from the names of the **seq** and **annotation** lists.
To obtain a data frame of all IDs and their corresponding species, you can
use the following code:

```{r create_species_id_table}
# Get a 2-column data frame of species IDs and names
id_table <- create_species_id_table(names(proteomes))

id_table
```

# Phylogenomic profiling

After inferring the synteny network, the first thing you would want to do is
cluster your network and identify which phylogenetic groups are contained
in each cluster. This is what we call **phylogenomic profiling**. This way,
you can identify clade-specific clusters, and highly conserved clusters,
for instance. Here, we will use an example network of BUSCO genes for 
25 eudicot species, which was obtained from @zhao2019network. 

To obtain the phylogenomic profiles, you first need to cluster your network.
This can be done with `cluster_network()`. [^3]

[^3]: **Friendly tip:** `r BiocStyle::Biocpkg("syntenet")` uses the *Infomap* 
algorithm to cluster networks, which has been shown to have the best performance 
[@zhao2019network]. However, you can use any other network clustering
method implemented in the *cluster_* family of functions from the 
`r BiocStyle::CRANpkg("igraph")` package by passing the function directly
to the *clust_function* parameter (see `?cluster_network` for details).
Importantly, the Infomap algorithm (default clustering method) 
assigns each gene to a single cluster. 
However, for some cases (e.g., detection of tandem arrays), 
you may want to use an algorithm that allows community overlap 
(i.e., a gene can be part of more than one cluster). 
If this is your case, we recommend the *clique percolation* algorithm, 
which is implemented in the R package
`r BiocStyle::CRANpkg("CliquePercolation")` [@lange2021cliquepercolation].

```{r cluster_network}
# Load example data and take a look at it
data(network)
head(network)

# Cluster network
clusters <- cluster_network(network)
head(clusters)
```

Now that each gene has been assigned to a cluster, we can identify the
phylogenomic profiles of each cluster. This function returns 
a matrix of phylogenomic profiles, with clusters in rows 
and species in columns.

```{r phylogenomic_profile}
# Phylogenomic profiling
profiles <- phylogenomic_profile(clusters)

# Exploring the output
head(profiles)
```

As a plot is worth a thousand words (or numbers), you can use the function
`plot_profiles()` to visualize the phylogenomic profiles as a heatmap with 
species in rows and synteny network clusters in columns. The heatmap
generated by this function is highly customizable by users. Some
important remarks are:

1. You can add a legend for species metadata (e.g., taxonomic information) 
by passing a 2-column data frame to the parameter *species_annotation*.

2. Columns (network clusters) are grouped with Ward's clustering on a matrix
of distances. The method to compute the distance matrix can be defined by users
in parameters *dist_function* and *dist_params*. By default, it uses
the function `stats::dist()` with parameter `method = "euclidean"`. Likewise,
the function to cluster the distance matrix and additional parameters
can be specified in *clust_function* and *clust_params*. By default,
it uses `stats::hclust` with parameter `method = "ward.D"`.

3. The order in which species are displayed can be defined by users in 
parameter *cluster_species*. We strongly recommend passing a vector of 
species order that matches the species tree, so that patterns can be
explored in a phylogenetic context. Importantly, if the character vector is
named, vector names will be used as species names in the plot. This a nice
way to replace species abbreviations with their full names.


Here, to briefly demonstrate how to play with the parameters we just mentioned
in the 3 remarks above, we will:

- Create a vector with the order in which we want species to be displayed,
with longer species names in vector names.

- Create a metadata data frame containing the family of each species.

- Use the function `dsvdis()` from the `r BiocStyle::CRANpkg("labdsv")` package
to calculate Ruzicka distances when clustering columns.

```{r plot_profiles}
# 1) Create a named vector of custom species order to plot
species_order <- setNames(
    # vector elements
    c(
        "vra", "van", "pvu", "gma", "cca", "tpr", "mtr", "adu", "lja",
        "Lang", "car", "pmu", "ppe", "pbr", "mdo", "roc", "fve",
        "Mnot", "Zjuj", "jcu", "mes", "rco", "lus", "ptr"
    ),
    # vector names
    c(
        "V. radiata", "V. angularis", "P. vulgaris", "G. max", "C. cajan",
        "T. pratense", "M. truncatula", "A. duranensis", "L. japonicus",
        "L. angustifolius", "C. arietinum", "P. mume", "P. persica",
        "P. bretschneideri", "M. domestica", "R. occidentalis", 
        "F. vesca", "M. notabilis", "Z. jujuba",
        "J. curcas", "M. esculenta", "R. communis", 
        "L. usitatissimum", "P. trichocarpa"
    )
)
species_order

# 2) Create a metadata data frame containing the family of each species
species_annotation <- data.frame(
    Species = species_order,
    Family = c(
        rep("Fabaceae", 11), rep("Rosaceae", 6),
        "Moraceae", "Ramnaceae", rep("Euphorbiaceae", 3), 
        "Linaceae", "Salicaceae"
    )
)
head(species_annotation)


# 3) Plot phylogenomic profiles, but using Ruzicka distances
plot_profiles(
    profiles, 
    species_annotation, 
    cluster_species = species_order, 
    dist_function = labdsv::dsvdis,
    dist_params = list(index = "ruzicka")
)
```

The heatmap is a nice way to observe patterns. For instance, you can see some
Rosaceae-specific clusters, Fabaceae-specific clusters, and highly conserved
ones as well.

If you want to explore in more details the group-specific clusters,
you can use the function `find_GS_clusters()`. For that, you only need to
input the profiles matrix and a data frame of species annotation (i.e.,
species groups).

```{r find_GS_clusters}
# Find group-specific clusters
gs_clusters <- find_GS_clusters(profiles, species_annotation)

head(gs_clusters)

# How many family-specific clusters are there?
nrow(gs_clusters)
```

As you can see, there are `r nrow(gs_clusters)` family-specific clusters
in the network. Let's plot a heatmap of group-specific clusters only.

```{r heatmap_filtered}
# Filter profiles matrix to only include group-specific clusters
idx <- rownames(profiles) %in% gs_clusters$Cluster
p_gs <- profiles[idx, ]

# Plot heatmap
plot_profiles(
    p_gs, species_annotation, 
    cluster_species = species_order, 
    cluster_columns = TRUE
)
```

Pretty cool, huh? You can also visualize clusters as a network plot with
the function `plot_network()`. For example, let's visualize the 
group-specific clusters.

```{r plot_network}
# 1) Visualize a network of first 5 GS-clusters
id <- gs_clusters$Cluster[1:5]
plot_network(network, clusters, cluster_id = id)

# 2) Coloring nodes by family
genes <- unique(c(network$node1, network$node2))
gene_df <- data.frame(
    Gene = genes,
    Species = unlist(lapply(strsplit(genes, "_"), head, 1))
)
gene_df <- merge(gene_df, species_annotation)[, c("Gene", "Family")]
head(gene_df)

plot_network(network, clusters, cluster_id = id, color_by = gene_df)

# 3) Interactive visualization (zoom out and in to explore it)
plot_network(
    network, clusters, cluster_id = id, 
    interactive = TRUE, dim_interactive = c(500, 300)
)
```

# Microsynteny-based phylogeny reconstruction

Finally, you can use the information on presence/absence of clusters in each
species to reconstruct a microsynteny-based phylogeny.

To do that, you first need to binarize the profiles matrix (0s and 1s
representing absence and presence, respectively) and transpose it. This can
be done with `binarize_and_tranpose()`.

```{r binarize}
bt_mat <- binarize_and_transpose(profiles)

# Looking at the first 5 rows and 5 columns of the matrix
bt_mat[1:5, 1:5]
```

Now, you can use this transposed binary matrix as input to 
IQ-TREE [@minh2020iq] to infer a phylogeny. To do so, you can use the function
`infer_microsynteny_phylogeny()`, which allows you to run IQ-TREE from 
an R session [^4]. You need to have IQ-TREE installed in your machine and in 
your PATH to run this function. You can check if you have IQ-TREE installed 
with `iqtree_is_installed()`.

[^4]: **Alternative:** if you want to use a different program rather
than IQ-TREE, you can use the function `profiles2phylip()` to write the 
transposed binary matrix to a PHYLIP file and run your favorite program
on the command line. However, when inferring a phylogeny from phylogenomic
profiles, you need to make sure that the program you are using supports 
substitution models for binary data. In IQ-TREE, for instance, using
binary, morphological models requires passing parameters `-st MORPH`.


For the sake of demonstration, we will infer a phylogeny with 
`infer_microsynteny_phylogeny()` using the profiles for BUSCO genes for 
six legume species only. We will also remove non-variable sites (i.e.,
clusters that are present in all species or absent in all species).
However, we're only using this filtered data set for speed issues. 
For real-life applications, the best thing to do is to 
**include phylogenomic profiles for all genes** (not only BUSCO genes).


```{r infer_phylogeny}
# Leave only 6 legume species and P. mume as an outgroup for testing purposes
included <- c("gma", "pvu", "vra", "van", "cca", "pmu")
bt_mat <- bt_mat[rownames(bt_mat) %in% included, ]

# Remove non-variable sites
bt_mat <- bt_mat[, colSums(bt_mat) != length(included)]

if(iqtree_is_installed()) {
    phylo <- infer_microsynteny_phylogeny(bt_mat, outgroup = "pmu", 
                                          threads = 1)
}
```

The output of `infer_microsynteny_phylogeny()` is a character vector with paths
to the output files from IQ-TREE. Usually, you are interested in the file
ending in *.treefile*. This is the species tree in Newick format, and it can
be visualized with your favorite tree viewer. We strongly recommend using
the `read.tree()` function from the Bioconductor package
`r BiocStyle::Biocpkg("treeio")` [@wang2020treeio] to
read the tree, and visualizing it with the `r BiocStyle::Biocpkg("ggtree")` 
Bioc package [@yu2017ggtree].
For example, let's visualize a microsynteny-based phylogeny for 123 angiosperm
species, whose phylogenomic profiles were obtained from @zhao2021whole.

```{r vis_phylogeny, message = FALSE, warning = FALSE, fig.height = 12, fig.width = 7}
data(angiosperm_phylogeny)

# Plotting the tree
library(ggtree)
ggtree(angiosperm_phylogeny) +
    geom_tiplab(size = 3) +
    xlim(0, 0.3)
```

# FAQ {.unnumbered}

## How do I execute an external dependency that is not in my PATH? {.unnumbered}

If you have DIAMOND and/or IQ-TREE installed, but in a directory that is not in 
your PATH, you can add this given directory to your PATH with the function
`Sys.setenv()`. 

For example, suppose your DIAMOND binary is in `/home/username/bioinfo_tools`.
To add this directory to your PATH, you would run:

```{r faq1}
# Add example directory /home/username/bioinfo_tools to PATH
Sys.setenv(
    PATH = paste(
        Sys.getenv("PATH"), "/home/username/bioinfo_tools", sep = ":"
    )
)
```

Note that your R PATH is not the same as your system's PATH. Thus, even if you
add the directory `/home/username/bioinfo_tools` to your system's path (e.g.,
by editing your ~/.bashrc file if you are on Linux), you would still need to
update your R PATH.

## Can I run the DIAMOND searches on the command line and import the results? {.unnumbered}

Yes. This case is quite common for users who have a large amount
of data and want to execute DIAMOND or any similarity search program 
in an HPC cluster (by submitting a job with `qsub` to execute a Bash script). 

To do that, you will have to follow the 3 steps below:

1. Export the processed sequences (as returned by `process_input()`) with the function `export_sequences()`. This function will write the sequences
to FASTA files in the directory specified in the **outdir**
parameter.

2. Navigate (i.e., `cd`) to the directory specified in **outdir**, where
the FASTA files are, and execute all-vs-all similarity searches. The
output files must be named "[species]_[species].tsv", where [species]
indicates the basename of the FASTA files (e.g., "speciesA" for a FASTA
file named *speciesA.fasta*). For DIAMOND, you can use the following 
code (with adaptations, if you prefer):

```{bash eval=FALSE}
#!/bin/bash

# Create output directories `dbs` and `results`
mkdir -p dbs
mkdir -p results

# 1. Make dbs for each species
for seqfile in *.fasta
do
    dbfile="dbs/$(basename "$seqfile" .fasta)"
    diamond makedb --in "$seqfile" -d "$dbfile" --quiet
done

# 2. Perform all-vs-all pairwise similarity searches
species=( $(basename -s .fasta *.fasta) )
for (( i=0; i<${#species[@]}; i++ ))
do
    query="${species[$i]}.fasta"
    for (( j=0; j<${#species[@]}; j++ ))
    do
        db="dbs/${species[$j]}"
        outfile="results/${species[$i]}_${species[$j]}.tsv"
        diamond blastp -q "$query" -d "$db" -o "$outfile" \
          --max-hsps 1 -k 5 --quiet 
    done
done
```

3. Read the output of the similarity searches into a list of 
data frames with the `read_diamond()` function. As input, `read_diamond()`
takes the path to the directory containing the DIAMOND/BLAST output files.

## My sequence names do not match gene IDs in the annotation. What should I do? {.unnumbered}

When the names of your sequences (equivalent to FASTA headers) do not match
the gene IDs in your `GRanges` objects, __syntenet__ throws the following error:

> Sequence names in 'seq' do not match gene names in 'annotation'.

In most (if not all) of the cases, this error happens because users have 
protein IDs as sequence names, and __syntenet__ looks for gene IDs 
(i.e., using rows of the `GRanges` objects for which the 
column *type* is "gene"). This is the case, for instance, for data obtained
from NCBI's RefSeq database. To solve the issue, you need to replace 
**protein/transcript IDs** with **gene IDs** in sequence names, which can
be done with the function `collapse_protein_ids()`. 


To demonstrate how this works, let's explore an example data set containing
protein sequences and gene annotation obtained from RefSeq. The data contains
information on a subset of 16 genes from the fish species *Alosa alosa*, and
it is stored in the `extdata/RefSeq_parsing_example` directory of this
package.

```{r faq3-p1}
# Path to directory containing data
data_dir <- system.file(
    "extdata", "RefSeq_parsing_example", package = "syntenet"
)
dir(data_dir)

# Reading the files to a format that syntenet understands
seqs <- fasta2AAStringSetlist(data_dir)
annot <- gff2GRangesList(data_dir)

# Taking a look at the data
seqs
head(names(seqs$Aalosa))

annot
```

The first problem we can observe is that sequence names have additional text
describing the sequences (e.g., "GSK3-beta..."), and we must have only the IDs.
To solve this issue, we can remove whitespace and everything 
that comes after it.

```{r faq3-p2}
# Remove whitespace and everything after it
names(seqs$Aalosa) <- gsub(" .*", "", names(seqs$Aalosa))

# Taking a look at the new names
head(names(seqs$Aalosa))
```

Great, we removed the unnecessary text! However, there is still
a problem: sequence names start with *XP_....*. If you look closer at the
first row of `annot$Aalosa` (which contains ranges for genes), you will
notice that none of the columns contain such *XP_...* IDs. This is because
RefSeq uses such IDs for CDS, not for genes. Let's check if we can indeed find
the *XP_...* IDs in rows that have "CDS" in the `type` column.

```{r faq3-p3}
# Show only rows for which `type` is "CDS"
head(annot$Aalosa[annot$Aalosa$type == "CDS"])
```

The *XP_...* IDs can be found in the `Name` column. Note also that the gene IDs,
which are what we need for __syntenet__, are in the `gene` column. To collapse
protein IDs to gene IDs with `collapse_protein_ids()`, we will need to create
a list of 2-column data frames containing the correspondence between
protein IDs and gene IDs for each species. In this example, we can do that 
by extracting the columns `Name` and `gene` from rows that represent CDS 
ranges. 

```{r faq-p4}
# Create a list of data frames containing protein-to-gene ID correspondences
protein2gene <- lapply(annot, function(x) {
    
    # Extract only CDS ranges
    cds_ranges <- x[x$type == "CDS"]
    
    # Create the ID correspondence data frame
    df <- data.frame(
        protein_id = cds_ranges$Name,
        gene_id = cds_ranges$gene
    )
    
    # Remove duplicate rows
    df <- df[!duplicated(df$protein_id), ]
    
    return(df)
})

# Taking a look at the list
protein2gene
```

Finally, we can pass the list of sequences and the list of ID correspondences
to `collapse_protein_ids()`, which will return a list of `AAStringSet` objects
with gene IDs in sequence names.

```{r faq-p5}
# Collapse protein IDs to gene IDs in list of sequences
new_seq <- collapse_protein_ids(seqs, protein2gene)

# Looking at the new sequences
new_seq
```

As you can see, protein IDs have been replaced with gene IDs. If there are
multiple proteins for the same gene (i.e., different isoforms), the function
keeps only the longest sequence (also known as protein products of the
primary transcript). This way, the number of sequences will never be greater
than the number of genes, which is what __syntenet__ expects.

# Session information {.unnumbered}

This document was created under the following conditions:

```{r sessionInfo}
sessionInfo()
```

# References {.unnumbered}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/06_dataviz.R
\name{plot_profiles}
\alias{plot_profiles}
\title{Plot a heatmap of phylogenomic profiles}
\usage{
plot_profiles(
  profile_matrix = NULL,
  species_annotation = NULL,
  palette = "Greens",
  dist_function = stats::dist,
  dist_params = list(method = "euclidean"),
  clust_function = stats::hclust,
  clust_params = list(method = "ward.D"),
  cluster_species = FALSE,
  show_colnames = FALSE,
  discretize = TRUE,
  ...
)
}
\arguments{
\item{profile_matrix}{A matrix of phylogenomic profiles obtained
with \code{phylogenomic_profile}.}

\item{species_annotation}{A 2-column data frame with species IDs in
the first column (same as column names of profile matrix), and species
annotation (e.g., higher-level taxonomic information) in the second column.}

\item{palette}{A character vector of colors or a character scalar with
the name of an RColorBrewer palette. Default: "RdYlBu".}

\item{dist_function}{Function to use to calculate a distance matrix for
synteny clusters. Popular examples include \code{stats::dist},
\code{labdsv::dsvdis}, and \code{vegan::vegdist}. Default: stats::dist.}

\item{dist_params}{A list with parameters to be passed to the function
specified in parameter \strong{dist_function}.
Default: list(method = "euclidean").}

\item{clust_function}{Function to use to cluster the distance matrix
returned by the function specified in \code{dist_function}. Examples include
\code{stats::hclust} and \code{Rclusterpp::Rclusterpp.hclust}.
Default: stats::hclust.}

\item{clust_params}{A list with additional parameters (if any) to be
passed to the function specified in parameter \strong{clust_function}.
Default: list(method = "ward.D").}

\item{cluster_species}{Either a logical scalar (TRUE or FALSE) or
a character vector with the order in which species should be arranged.
TRUE or FALSE indicate whether hierarchical clustering should be applied
to rows (species). Ideally, the character vector
should contain the order of species in a phylogenetically meaningful way.
If users pass a named vector, vector names will be used to rename species.
If users have a species tree, they can read it
with \code{treeio::read.tree()}, plot it with \code{ggtree::ggtree()},
and get the species order from the ggtree object
with \code{ggtree::get_taxa_name()}. Default: FALSE.}

\item{show_colnames}{Logical indicating whether to show column names (i.e.,
cluster IDs) or not. Showing cluster IDs can be useful when visualizing
a small subset of them. When visualizing all clusters, cluster IDs are
impossible to read. Default: FALSE.}

\item{discretize}{Logical indicating whether to discretize clusters in
4 categories: 0, 1, 2, and 3+. If FALSE, counts will be
log2 transformed. Default: TRUE.}

\item{...}{Additional parameters to \code{pheatmap::pheatmap()}.}
}
\value{
A pheatmap object.
}
\description{
Plot a heatmap of phylogenomic profiles
}
\examples{
data(clusters)
profile_matrix <- phylogenomic_profile(clusters)
species_order <- c(
    "vra", "van", "pvu", "gma", "cca", "tpr", "mtr", "adu", "lja",
    "Lang", "car", "pmu", "ppe", "pbr", "mdo", "roc", "fve",
    "Mnot", "Zjuj", "jcu", "mes", "rco", "lus", "ptr"
)
species_names <- c(
    "V. radiata", "V. angularis", "P. vulgaris", "G. max", "C. cajan",
    "T. pratense", "M. truncatula", "A. duranensis", "L. japonicus",
    "L. angustifolius", "C. arietinum", "P. mume", "P. persica",
    "P. bretschneideri", "M. domestica", "R. occidentalis", 
    "F. vesca", "M. notabilis", "Z. jujuba", "J. curcas",
    "M. esculenta", "R. communis", "L. usitatissimum", "P. trichocarpa"
)
names(species_order) <- species_names
species_annotation <- data.frame(
   Species = species_order,
   Family = c(rep("Fabaceae", 11), rep("Rosaceae", 6),
              "Moraceae", "Ramnaceae", rep("Euphorbiaceae", 3), 
              "Linaceae", "Salicaceae")
)
p <- plot_profiles(profile_matrix, species_annotation, 
                   cluster_species = species_order)
                   
p <- plot_profiles(profile_matrix, species_annotation, 
                   cluster_species = species_order, 
                   discretize = FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_data_preprocessing.R
\name{process_input}
\alias{process_input}
\title{Process sequence data}
\usage{
process_input(
  seq = NULL,
  annotation = NULL,
  gene_field = "gene_id",
  filter_annotation = FALSE
)
}
\arguments{
\item{seq}{A list of AAStringSet objects, each list element containing
protein sequences for a given species. This list must have names
(not NULL), and names of each list element must match the names of
list elements in \strong{annotation}.}

\item{annotation}{A GRangesList, CompressedGRangesList, or list of
GRanges with the annotation for the sequences in \strong{seq}. This list must
have names (not NULL), and names of each list element must match the names
of list elements in \strong{seq}.}

\item{gene_field}{Character, name of the column in the GRanges objects
that contains gene IDs. Default: "gene_id".}

\item{filter_annotation}{Logical indicating whether \strong{annotation}
should be filtered to keep only genes that are also in \strong{seq}. This
is particularly useful if users want to remove information on
non-protein coding genes from \strong{annotation}, since such genes are
typically not present in sets of whole-genome protein sequences.
Default: FALSE.}
}
\value{
A list of 2 elements:
\describe{
\item{seq}{The processed list of AAStringSet objects from \strong{seq}.}
\item{annotation}{The processed GRangesList or CompressedGRangesList
object from \strong{annotation}.}
}
}
\description{
Process sequence data
}
\details{
This function processes the input sequences and annotation to:
\enumerate{
\item Remove whitespace and anything after it in sequence names
(i.e., \code{names(seq[[x]])}, which is equivalent to FASTA headers), if
there is any.
\item Add a unique species identifier to sequence names. The species
identifier consists of the first 3-5 strings of the element name.
For instance, if the first element of the \strong{seq} list is named
"Athaliana", each sequence in it will have an identifier "Atha_" added
to the beginning of each gene name (e.g., Atha_AT1G01010).
\item If sequences have an asterisk (*) representing stop codon, remove it.
\item Add a unique species identifier (same as above) to
gene and chromosome names of each element of the \strong{annotation}
GRangesList/CompressedGRangesList.
\item Filter each element of the \strong{annotation}
GRangesList/CompressedGRangesList to keep only seqnames, ranges, and
gene ID.
}
}
\examples{
data(annotation)
data(proteomes)
seq <- proteomes
clean_data <- process_input(seq, annotation)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_synteny.R
\name{intraspecies_synteny}
\alias{intraspecies_synteny}
\title{Detect intraspecies synteny}
\usage{
intraspecies_synteny(
  blast_intra = NULL,
  annotation = NULL,
  intra_dir = file.path(tempdir(), "intra"),
  anchors = 5,
  max_gaps = 25,
  is_pairwise = TRUE,
  verbose = FALSE,
  bp_param = BiocParallel::SerialParam(),
  ...
)
}
\arguments{
\item{blast_intra}{A list of BLAST/DIAMOND data frames for
intraspecies comparisons as returned by \code{run_diamond()}.}

\item{annotation}{A processed GRangesList or CompressedGRangesList object
as returned by \code{process_input()}.}

\item{intra_dir}{Path to output directory where .collinearity files will
be stored.}

\item{anchors}{Numeric indicating the minimum required number of genes
to call a syntenic block. Default: 5.}

\item{max_gaps}{Numeric indicating the number of upstream and downstream
genes to search for anchors. Default: 25.}

\item{is_pairwise}{Logical indicating if only pairwise blocks should be
reported. Default: TRUE.}

\item{verbose}{Logical indicating if log messages should be printed on
screen. Default: FALSE.}

\item{bp_param}{BiocParallel back-end to be used.
Default: \code{BiocParallel::SerialParam()}.}

\item{...}{Any additional arguments to the \code{MCScanX} algorithm. For a
complete list of all available options, see the man page
of \code{rcpp_mcscanx_file()}.}
}
\value{
Paths to .collinearity files.
}
\description{
Detect intraspecies synteny
}
\examples{
# Load data
data(scerevisiae_annot)
data(scerevisiae_diamond)

# Detect intragenome synteny
intra_syn <- intraspecies_synteny(
    scerevisiae_diamond, scerevisiae_annot
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/06_dataviz.R
\name{plot_network}
\alias{plot_network}
\title{Plot network}
\usage{
plot_network(
  network = NULL,
  clusters = NULL,
  cluster_id = NULL,
  color_by = "cluster",
  interactive = FALSE,
  dim_interactive = c(600, 600)
)
}
\arguments{
\item{network}{The synteny network represented as an edge list, which is
a 2-column data frame with each member of the anchor pair in a column.}

\item{clusters}{A 2-column data frame with the variables \strong{Gene}
and \strong{Cluster} representing gene ID and cluster ID, respectively,
exactly as returned by \code{cluster_network}.}

\item{cluster_id}{Character scalar or vector with cluster ID. If more than one
cluster is passed as input, clusters are colored differently.}

\item{color_by}{Either "cluster" or a 2-column data frame with
gene IDs in the first column and variable to be used for
coloring (e.g., taxonomic information) in the second column.}

\item{interactive}{Logical scalar indicating whether to display an
interactive network or not. Default: FALSE.}

\item{dim_interactive}{Numeric vector of length 2 with the window
dimensions of the interactive plot. If \strong{interactive} is set to FALSE,
this parameter is ignored.}
}
\value{
A ggplot object with the network.
}
\description{
Plot network
}
\examples{
data(network)
data(clusters)
# Option 1: 1 cluster
cluster_id <- 25
plot_network(network, clusters, cluster_id)

# Option 2: 2 clusters
cluster_id <- c(25, 1089)
plot_network(network, clusters, cluster_id)

# Option 3: custom annotation for coloring
species_order <- c(
    "vra", "van", "pvu", "gma", "cca", "tpr", "mtr", "adu", "lja",
    "Lang", "car", "pmu", "ppe", "pbr", "mdo", "roc", "fve",
    "Mnot", "Zjuj", "jcu", "mes", "rco", "lus", "ptr"
) 
species_annotation <- data.frame(
   Species = species_order,
   Family = c(rep("Fabaceae", 11), rep("Rosaceae", 6),
              "Moraceae", "Ramnaceae", rep("Euphorbiaceae", 3), 
              "Linaceae", "Salicaceae")
)
genes <- unique(c(network$node1, network$node2))
gene_df <- data.frame(
    Gene = genes,
    Species = unlist(lapply(strsplit(genes, "_"), head, 1))
)
gene_df <- merge(gene_df, species_annotation)[, c("Gene", "Family")]

plot_network(network, clusters, cluster_id = 25, color_by = gene_df)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/04_synteny_detection.R
\name{infer_syntenet}
\alias{infer_syntenet}
\title{Infer synteny network}
\usage{
infer_syntenet(
  blast_list = NULL,
  annotation = NULL,
  outdir = tempdir(),
  anchors = 5,
  max_gaps = 25,
  is_pairwise = TRUE,
  verbose = FALSE,
  bp_param = BiocParallel::SerialParam(),
  ...
)
}
\arguments{
\item{blast_list}{A list of data frames, each data frame having
the tabular output of BLASTp or similar programs, such as DIAMOND.
This is the output of the function \code{run_diamond()}. If you performed
pairwise comparisons on the command line, you can read the tabular output
as data frames and combine them in a list. List names must be have species
names separated by underscore. For instance, if the first list element is
a data frame containing the comparison of speciesA (query)
against speciesB (database), its name must be "speciesA_speciesB".}

\item{annotation}{A processed GRangesList, CompressedGRangesList, or
list of GRanges as returned by \code{process_input()}.}

\item{outdir}{Path to the output directory. Default: tempdir().}

\item{anchors}{Numeric indicating the minimum required number of genes
to call a syntenic block. Default: 5.}

\item{max_gaps}{Numeric indicating the number of upstream and downstream
genes to search for anchors. Default: 25.}

\item{is_pairwise}{specify if only pairwise blocks should be reported
Default: TRUE}

\item{verbose}{Logical indicating if log messages should be printed on
screen. Default: FALSE.}

\item{bp_param}{BiocParallel back-end to be used.
Default: \code{BiocParallel::SerialParam()}.}

\item{...}{Any additional arguments to the \code{MCScanX} algorithm. For a
complete list of all available options, see the man page
of \code{rcpp_mcscanx_file()}.}
}
\value{
A network represented as an edge list.
}
\description{
Infer synteny network
}
\examples{
# Load data
data(proteomes)
data(annotation)
data(blast_list)

# Create processed annotation list
annotation <- process_input(proteomes, annotation)$annotation

# Infer the synteny network
net <- infer_syntenet(blast_list, annotation)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{clusters}
\alias{clusters}
\title{Synteny network clusters of BUSCO genes for 25 eudicot species}
\format{
A 2-column data frame containing the following variables:
\describe{
\item{Gene}{Gene ID}
\item{Cluster}{Cluster ID}
}
}
\usage{
data(clusters)
}
\description{
Data obtained from Zhao & Schranz, 2019.
}
\examples{
data(clusters)
}
\references{
Zhao, T., & Schranz, M. E. (2019). Network-based microsynteny analysis
identifies major differences and genomic outliers in mammalian and
angiosperm genomes. Proceedings of the National Academy of
Sciences, 116(6), 2165-2174.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{create_species_id_table}
\alias{create_species_id_table}
\title{Create a data frame of species IDs (3-5-character abbreviations)}
\usage{
create_species_id_table(species_names)
}
\arguments{
\item{species_names}{A character vector of names extracted from
the \strong{seq} or \strong{annotation} lists, which can be extracted with
\code{names(seq)} or \code{names(annotation)}.}
}
\value{
A 2-column data frame with the following variables:
\describe{
\item{species_id}{Character, species ID consisting of 3-5 characters.}
\item{species_name}{Character, original names passed as input.}
}
}
\description{
Create a data frame of species IDs (3-5-character abbreviations)
}
\examples{
# Load 'seq' list (list of AAStringSet objects)
data(proteomes)

# Create ID table
create_species_id_table(names(proteomes))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{blast_list}
\alias{blast_list}
\title{List of data frames containing BLAST-like tabular output}
\format{
A list of data frames containing the pairwise comparisons between
proteomes of Ostreococcus species.
}
\usage{
data(blast_list)
}
\description{
The object was created by running \code{run_diamond} on the protein
sequences for the Ostreococcus algae available in the \strong{proteomes}
example data. Hits with <50\% identity were filtered out. Code to recreate
this data is available at the script/ subdirectory.
}
\examples{
data(blast_list)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_data_import.R
\name{fasta2AAStringSetlist}
\alias{fasta2AAStringSetlist}
\title{Read FASTA files in a directory as a list of AAStringSet objects}
\usage{
fasta2AAStringSetlist(fasta_dir)
}
\arguments{
\item{fasta_dir}{Character indicating the path to the directory containing
FASTA files.}
}
\value{
A list of AAStringSet objects, where each element represents a
different FASTA file.
}
\description{
Read FASTA files in a directory as a list of AAStringSet objects
}
\examples{
fasta_dir <- system.file("extdata", "sequences", package = "syntenet")
aastringsetlist <- fasta2AAStringSetlist(fasta_dir)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/03_similarity_searches.R
\name{run_last}
\alias{run_last}
\title{Wrapper to run last from an R session}
\usage{
run_last(
  seq = NULL,
  verbose = FALSE,
  outdir = tempdir(),
  threads = 1,
  compare = "all",
  lastD = 1e+06,
  ...
)
}
\arguments{
\item{seq}{A processed list of AAStringSet objects
as returned by \code{process_input()}.}

\item{verbose}{Logical indicating if progress messages should be printed.
Default: FALSE.}

\item{outdir}{Output directory for last results. By default, output files
are saved to a temporary directory.}

\item{threads}{Number of threads to use. Default: 1.}

\item{compare}{Character scalar indicating which comparisons
should be made when running last.
Possible modes are "all" (all-vs-all comparisons),
"intraspecies" (intraspecies comparisons only), or
"interspecies" (interspecies comparisons only). Alternatively, users can
pass a 2-column data frame as input with the names of species to be
compared.}

\item{lastD}{last option D: query letters per random alignment. Default: 1e6.}

\item{...}{Any additional arguments to \code{lastal}.}
}
\value{
A list of data frames containing last's tabular output
for each pairwise combination of species. For n species, the list length
will be \eqn{n^2}.
}
\description{
Wrapper to run last from an R session
}
\examples{
data(proteomes)
data(annotation)
seq <- process_input(proteomes, annotation)$seq[1:2]
if(last_is_installed()) {
    last_results <- run_last(seq)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/03_similarity_searches.R
\name{read_diamond}
\alias{read_diamond}
\title{Read DIAMOND/BLAST tables as a list of data frames}
\usage{
read_diamond(diamond_dir = NULL)
}
\arguments{
\item{diamond_dir}{Path to directory containing the tabular output
of DIAMOND or similar programs (e.g., BLAST).}
}
\value{
A list of data frames with the tabular DIAMOND output.
}
\description{
Read DIAMOND/BLAST tables as a list of data frames
}
\examples{
# Path to output directory
diamond_dir <- system.file("extdata", package = "syntenet")

# Read output
l <- read_diamond(diamond_dir)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/05_phylogenomic_profiling.R
\name{find_GS_clusters}
\alias{find_GS_clusters}
\title{Find group-specific clusters based on user-defined species classification}
\usage{
find_GS_clusters(
  profile_matrix = NULL,
  species_annotation = NULL,
  min_percentage = 50
)
}
\arguments{
\item{profile_matrix}{A matrix of phylogenomic profiles obtained
with \code{phylogenomic_profile}.}

\item{species_annotation}{A 2-column data frame with species IDs in
the first column (same as column names of profile matrix), and species
annotation (e.g., higher-level taxonomic information) in the second column.}

\item{min_percentage}{Numeric scalar with the minimum percentage of species
in a group to consider group specificity. For instance,
if a given cluster is present in only 1 group of species, but in less
than \strong{min_percentage} of the species for this group, it will not
be considered a group-specific cluster. This filtering criterion is useful
to differentiate group-specific clusters (e.g., family-specific) from
subgroup-specific clusters (e.g., genus-specific). Default: 50.}
}
\value{
A data frame with the following variables:
\describe{
\item{Group}{To which group of species the cluster is specific.}
\item{Percentage}{Percentage of species from the group that are
represented by the cluster.}
\item{Cluster}{Cluster ID.}
}
}
\description{
Find group-specific clusters based on user-defined species classification
}
\examples{
data(clusters)
profile_matrix <- phylogenomic_profile(clusters)

# Species annotation
species_order <- c(
    "vra", "van", "pvu", "gma", "cca", "tpr", "mtr", "adu", "lja",
    "Lang", "car", "pmu", "ppe", "pbr", "mdo", "roc", "fve",
    "Mnot", "Zjuj", "hlu", "jcu", "mes", "rco", "lus", "ptr"
)
species_annotation <- data.frame(
   Species = species_order,
   Family = c(rep("Fabaceae", 11), rep("Rosaceae", 6),
              "Moraceae", "Ramnaceae", "Cannabaceae",
               rep("Euphorbiaceae", 3), "Linaceae", "Salicaceae")
)
gs_clusters <- find_GS_clusters(profile_matrix, species_annotation)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/07_microsynteny-based_phylogeny.R
\name{infer_microsynteny_phylogeny}
\alias{infer_microsynteny_phylogeny}
\title{Infer microsynteny-based phylogeny with IQTREE}
\usage{
infer_microsynteny_phylogeny(
  transposed_profiles = NULL,
  bootr = 1000,
  alrtboot = 1000,
  threads = "AUTO",
  model = "MK+FO+R",
  outdir = tempdir(),
  outgroup = NULL,
  verbose = FALSE
)
}
\arguments{
\item{transposed_profiles}{A binary and transposed profile matrix.
The profile matrix can be obtained with \code{phylogenomic_profile()}.}

\item{bootr}{Numeric scalar with the number of bootstrap replicates.
Default: 1000.}

\item{alrtboot}{Numeric scalar with the number of replicates for
the SH-like approximate likelihood ratio test. Default: 1000.}

\item{threads}{Numeric scalar indicating the number of threads to use or
"AUTO", which allows IQTREE to automatically choose the best number
of threads to use. Default: "AUTO".}

\item{model}{Substitution model to use. If you are unsure, pick the default.
Default: "MK+FO+R".}

\item{outdir}{Path to output directory. By default, files are saved in
a temporary directory, so they will be deleted when the R session closes.
If you want to keep the files, specify a custom output directory.}

\item{outgroup}{Name of outgroup clade to group the phylogeny.
Default: NULL (unrooted phylogeny).}

\item{verbose}{Logical indicating if progress messages should be prompted.
Default: FALSE.}
}
\value{
A character vector of paths to output files.
}
\description{
Infer microsynteny-based phylogeny with IQTREE
}
\examples{
data(clusters)
profile_matrix <- phylogenomic_profile(clusters)
tmat <- binarize_and_transpose(profile_matrix)

# Leave only some legumes and P. mume as an outgroup for testing purposes
included <- c("gma", "pvu", "vra", "van", "cca", "pmu")
tmat <- tmat[rownames(tmat) \%in\% included, ]

# Remove non-variable sites
tmat <- tmat[, colSums(tmat) != length(included)]

if(iqtree_is_installed()) {
    phylo <- infer_microsynteny_phylogeny(tmat, outgroup = "pmu", 
                                          threads = 1)
}
 
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/04_synteny_detection.R
\name{parse_collinearity}
\alias{parse_collinearity}
\title{Parse .collinearity files obtained with MCScan}
\usage{
parse_collinearity(collinearity_paths = NULL, as = "anchors")
}
\arguments{
\item{collinearity_paths}{Character vector of paths to .collinearity files.}

\item{as}{Character specifying what to extract.
One of "anchors" (default), "blocks", or "all".}
}
\value{
If \strong{as} is "anchors", a data frame with variables "Anchor1",
and "Anchor2". If \strong{as} is "blocks", a data frame with
variables "Block", "Block_score", "Chr", and "Orientation".
If \strong{as} is "all", a data frame with all aforementioned variables,
which indicate:
\describe{
\item{Block}{Numeric, synteny block ID}
\item{Block_score}{Numeric, score of synteny block.}
\item{Chr}{Character, query and target chromosome of the synteny
block formatted as "\if{html}{\out{<querychr>}}&\if{html}{\out{<targetchr>}}".}
\item{Orientation}{Character, the orientation of genes within blocks,
with "plus" indicating that genes are in the same
direction, and "minus" indicating that genes are
in opposite directions.}
\item{Anchor1}{Character, gene ID of anchor 1.}
\item{Anchor2}{Character, gene ID of anchor 2.}
}
}
\description{
The .collinearity files can be obtained with \code{intraspecies_synteny}
and \code{interspecies_synteny}, which execute a native version of the
MCScan algorithm.
}
\examples{
collinearity_paths <- system.file(
    "extdata", "Scerevisiae.collinearity", package = "syntenet"
)
net <- parse_collinearity(collinearity_paths)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/07_microsynteny-based_phylogeny.R
\name{binarize_and_transpose}
\alias{binarize_and_transpose}
\title{Binarize and transpose the phylogenomic profile matrix}
\usage{
binarize_and_transpose(profile_matrix = NULL)
}
\arguments{
\item{profile_matrix}{A matrix with phylogenomic profiles obtained
with \code{phylogenomic_profile}.}
}
\value{
A binary and transposed version of the profiles matrix.
}
\description{
Binarize and transpose the phylogenomic profile matrix
}
\examples{
data(clusters)
profile_matrix <- phylogenomic_profile(clusters)
tmat <- binarize_and_transpose(profile_matrix)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{annotation}
\alias{annotation}
\title{Filtered genome annotation for Ostreococcus sp. species}
\format{
A CompressedGRangesList containing
the elements \code{Olucimarinus}, \code{Osp_RCC809}, and \code{Otauri}.
}
\usage{
data(annotation)
}
\description{
Data obtained from Pico-PLAZA 3.0. Only annotation data for primary
transcripts were included, and only genes for chromosomes 1, 2, and 3.
}
\examples{
data(annotation)
}
\references{
Van Bel, M., Silvestri, F., Weitz, E. M., Kreft, L., Botzki, A.,
Coppens, F., & Vandepoele, K. (2021). PLAZA 5.0: extending the scope
and power of comparative and functional genomics in plants.
Nucleic acids research.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{angiosperm_phylogeny}
\alias{angiosperm_phylogeny}
\title{Microsynteny-based angiosperm phylogeny.}
\format{
An object of class 'phylo'.
}
\usage{
data(angiosperm_phylogeny)
}
\description{
Original tree file obtained from Zhao et al., 2021.
The tree is an object of class 'phylo', which can be created by reading
the tree file with \code{treeio::read.tree()}.
}
\examples{
data(angiosperm_phylogeny)
}
\references{
Zhao, T., Zwaenepoel, A., Xue, J. Y., Kao, S. M., Li, Z.,
Schranz, M. E., & Van de Peer, Y. (2021).
Whole-genome microsynteny-based phylogeny of angiosperms.
Nature Communications, 12(1), 1-14.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{iqtree_version}
\alias{iqtree_version}
\title{Get IQ-TREE version}
\usage{
iqtree_version()
}
\value{
Numeric indicating IQ-TREE version, with either 1 or 2.
}
\description{
Get IQ-TREE version
}
\examples{
iqtree_version()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/05_phylogenomic_profiling.R
\name{cluster_network}
\alias{cluster_network}
\title{Cluster the synteny network using the Infomap algorithm}
\usage{
cluster_network(
  network = NULL,
  clust_function = igraph::cluster_infomap,
  clust_params = NULL
)
}
\arguments{
\item{network}{A network represented as an edge list, which is a 2-column
data frame with node 1 in the first column and node 2 in the second column.
In a synteny network, node 1 and node are the anchor pairs.}

\item{clust_function}{Function to be used to cluster the network. It must
be one the functions from the \strong{cluster_*} family in
the \strong{igraph} package (e.g., cluster_infomap, cluster_leiden, etc).
Default: igraph::cluster_infomap.}

\item{clust_params}{A list with additional parameters (if any) to be passed
to the igraph clustering function. Default: NULL (no additional parameters).}
}
\value{
A 2-column data frame with the following variables:
\describe{
\item{Gene}{Gene ID.}
\item{Cluster}{Cluster ID as identified by infomap.}
}
}
\description{
Cluster the synteny network using the Infomap algorithm
}
\examples{
data(network)
clusters <- cluster_network(network[1:500, ])
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/07_microsynteny-based_phylogeny.R
\name{profiles2phylip}
\alias{profiles2phylip}
\title{Save the transposed binary profiles matrix to a file in PHYLIP format}
\usage{
profiles2phylip(transposed_profiles = NULL, outdir = tempdir())
}
\arguments{
\item{transposed_profiles}{A binary and transposed profile matrix.
The profile matrix can be obtained with \code{phylogenomic_profile()}.}

\item{outdir}{Path to output directory. By default, files are saved in
a temporary directory, so they will be deleted when the R session closes.
If you want to keep the files, specify a custom output directory.}
}
\value{
Character specifying the path to the PHYLIP file.
}
\description{
Save the transposed binary profiles matrix to a file in PHYLIP format
}
\examples{
data(clusters)
profile_matrix <- phylogenomic_profile(clusters)
tmat <- binarize_and_transpose(profile_matrix)
profiles2phylip(tmat)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_data_import.R
\name{gff2GRangesList}
\alias{gff2GRangesList}
\title{Read GFF/GTF files in a directory as a GRangesList object}
\usage{
gff2GRangesList(gff_dir)
}
\arguments{
\item{gff_dir}{Character indicating the path to the directory containing
GFF/GTF files.}
}
\value{
A GRangesList object, where each element represents a different
GFF/GTF file.
}
\description{
Read GFF/GTF files in a directory as a GRangesList object
}
\examples{
gff_dir <- system.file("extdata", "annotation", package = "syntenet")
grangeslist <- gff2GRangesList(gff_dir)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/03_similarity_searches.R
\name{run_diamond}
\alias{run_diamond}
\title{Wrapper to run DIAMOND from an R session}
\usage{
run_diamond(
  seq = NULL,
  top_hits = 5,
  verbose = FALSE,
  outdir = tempdir(),
  threads = NULL,
  compare = "all",
  ...
)
}
\arguments{
\item{seq}{A processed list of AAStringSet objects
as returned by \code{process_input()}.}

\item{top_hits}{Number of top hits to keep in DIAMOND search. Default: 5.}

\item{verbose}{Logical indicating if progress messages should be printed.
Default: FALSE.}

\item{outdir}{Output directory for DIAMOND results. By default, output files
are saved to a temporary directory.}

\item{threads}{Number of threads to use. Default: let DIAMOND auto-detect
and use all available virtual cores on the machine.}

\item{compare}{Character scalar indicating which comparisons
should be made when running DIAMOND.
Possible modes are "all" (all-vs-all comparisons),
"intraspecies" (intraspecies comparisons only), or
"interspecies" (interspecies comparisons only). Alternatively, users can
pass a 2-column data frame as input with the names of species to be
compared.}

\item{...}{Any additional arguments to \verb{diamond blastp}.}
}
\value{
A list of data frames containing DIAMOND's tabular output
for each pairwise combination of species. For n species, the list length
will be \eqn{n^2}.
}
\description{
Wrapper to run DIAMOND from an R session
}
\examples{
data(proteomes)
data(annotation)
seq <- process_input(proteomes, annotation)$seq[1:2]
if(diamond_is_installed()) {
    diamond_results <- run_diamond(seq)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{edges}
\alias{edges}
\title{Synteny network of Ostreococcus genomes represented as an edge list}
\format{
A data frame containing anchor pairs between two Ostreococcus
proteomes.
}
\usage{
data(edges)
}
\description{
The object was created by running \code{infer_syntenet} on the
\strong{blast_list} example data. Code to recreate this data set is
available at the script/ subdirectory.
}
\examples{
data(edges)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{network}
\alias{network}
\title{Synteny network of BUSCO genes for 25 eudicot species}
\format{
An edgelist (i.e., a 2-column data frame with node 1 in
column 1 and node 2 in column 2).
}
\usage{
data(network)
}
\description{
Data obtained from Zhao & Schranz, 2019.
}
\examples{
data(network)
}
\references{
Zhao, T., & Schranz, M. E. (2019). Network-based microsynteny analysis
identifies major differences and genomic outliers in mammalian and
angiosperm genomes. Proceedings of the National Academy of
Sciences, 116(6), 2165-2174.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/05_phylogenomic_profiling.R
\name{phylogenomic_profile}
\alias{phylogenomic_profile}
\title{Perform phylogenomic profiling for synteny network clusters}
\usage{
phylogenomic_profile(clusters = NULL)
}
\arguments{
\item{clusters}{A 2-column data frame with variables \strong{Gene} and
\strong{Cluster} as returned by \code{cluster_network}.}
}
\value{
A matrix of i rows and j columns containing
the number of genes in cluster i for each species j. The number of rows
is equal to the number of clusters in \strong{clusters}, and
the number of columns is equal to the number of species
in \strong{clusters}.
}
\description{
Perform phylogenomic profiling for synteny network clusters
}
\examples{
data(clusters)
profiles <- phylogenomic_profile(clusters)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{iqtree_is_installed}
\alias{iqtree_is_installed}
\title{Check if IQTREE is installed}
\usage{
iqtree_is_installed()
}
\value{
Logical indicating whether IQTREE is installed or not.
}
\description{
Check if IQTREE is installed
}
\examples{
iqtree_is_installed()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_data_preprocessing.R
\name{collapse_protein_ids}
\alias{collapse_protein_ids}
\title{Collapse protein IDs into gene IDs in sequence names of AAStringSet objects}
\usage{
collapse_protein_ids(seq, protein2gene = NULL)
}
\arguments{
\item{seq}{A list of AAStringSet objects, each list element containing
protein sequences for a given species. This list must have names
(not NULL), and names of each list element must match the names of
list elements in \strong{protein2gene}.}

\item{protein2gene}{A list of 2-column data frames containing
protein-to-gene ID correspondences, where the first column contains
protein IDs, and the second column contains gene IDs. Names of list elements
must match names of \strong{seq}.}
}
\value{
A list of AAStringSet objects as in \strong{seq}, but with
protein IDs replaced with gene IDs.
}
\description{
This function can be used if the sequence names of the AAStringSet objects
contain protein IDs instead of gene IDs (what syntenet requires)
}
\details{
For each species, this function will replace the protein IDs in sequence
names with gene IDs using the protein-to-gene correspondence table in
\strong{protein2gene}. After replacing protein IDs with gene IDs, if
there are multiple sequences with the same gene ID (indicating different
isoforms of the same gene), only the longest sequence is kept, so that
the number of sequences is not greater than the number of genes.
}
\examples{
# Load data
seq_path <- system.file(
    "extdata", "RefSeq_parsing_example", package = "syntenet"
)
seq <- fasta2AAStringSetlist(seq_path)
annot <- gff2GRangesList(seq_path)

# Clean sequence names
names(seq$Aalosa) <- gsub(" .*", "", names(seq$Aalosa))

# Create a correspondence data frame
cor_df <- as.data.frame(annot$Aalosa[annot$Aalosa$type == "CDS", ])
cor_df <- cor_df[, c("Name", "gene")]

# Create a list of correspondence data frames
protein2gene <- list(Aalosa = cor_df)

# Collapse IDs
new_seqs <- collapse_protein_ids(seq, protein2gene)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{scerevisiae_diamond}
\alias{scerevisiae_diamond}
\title{Intraspecies DIAMOND output for S. cerevisiae}
\format{
A list of data frames (length 1) containing the whole paranome of
S. cerevisiae resulting from intragenome similarity searches.
}
\usage{
data(scerevisiae_diamond)
}
\description{
List obtained with \code{run_diamond()}.
}
\examples{
data(scerevisiae_diamond)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{proteomes}
\alias{proteomes}
\title{Filtered proteomes of Ostreococcus sp. species}
\format{
A list of AAStringSet objects containing
the elements \code{Olucimarinus}, \code{Osp_RCC809}, and \code{Otauri}.
}
\usage{
data(proteomes)
}
\description{
Data obtained from Pico-PLAZA 3.0. Only the translated sequences of primary
transcripts were included, and only genes from chromosomes 1, 2, and 3.
}
\examples{
data(proteomes)
}
\references{
Van Bel, M., Silvestri, F., Weitz, E. M., Kreft, L., Botzki, A.,
Coppens, F., & Vandepoele, K. (2021). PLAZA 5.0: extending the scope
and power of comparative and functional genomics in plants.
Nucleic acids research.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{scerevisiae_annot}
\alias{scerevisiae_annot}
\title{Genome annotation of the yeast species S. cerevisiae}
\format{
A GRangesList as returned by \code{process_input()} containing
the element \strong{Scerevisiae}.
}
\usage{
data(scerevisiae_annot)
}
\description{
Data obtained from Ensembl Fungi. Only annotation data for primary
transcripts were included.
}
\examples{
data(scerevisiae_annot)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/syntenet-package.R
\docType{package}
\name{syntenet-package}
\alias{syntenet}
\alias{syntenet-package}
\title{syntenet: Inference And Analysis Of Synteny Networks}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

syntenet can be used to infer synteny networks from whole-genome protein sequences and analyze them. Anchor pairs are detected with the MCScanX algorithm, which was ported to this package with the Rcpp framework for R and C++ integration. Anchor pairs from synteny analyses are treated as an undirected unweighted graph (i.e., a synteny network), and users can perform: i. network clustering; ii. phylogenomic profiling (by identifying which species contain which clusters) and; iii. microsynteny-based phylogeny reconstruction with maximum likelihood.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/almeidasilvaf/syntenet}
  \item Report bugs at \url{https://support.bioconductor.org/t/syntenet}
}

}
\author{
\strong{Maintainer}: Fabrício Almeida-Silva \email{fabricio_almeidasilva@hotmail.com} (\href{https://orcid.org/0000-0002-5314-2964}{ORCID})

Authors:
\itemize{
  \item Tao Zhao \email{tao.zhao@nwafu.edu.cn} (\href{https://orcid.org/0000-0001-7302-6445}{ORCID})
  \item Kristian K Ullrich \email{ullrich@evolbio.mpg.de} (\href{https://orcid.org/0000-0003-4308-9626}{ORCID})
  \item Yves Van de Peer \email{yves.vandepeer@psb.vib-ugent.be} (\href{https://orcid.org/0000-0003-4327-3730}{ORCID})
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_synteny.R
\name{interspecies_synteny}
\alias{interspecies_synteny}
\title{Detect interspecies synteny}
\usage{
interspecies_synteny(
  blast_inter = NULL,
  annotation = NULL,
  inter_dir = file.path(tempdir(), "inter"),
  anchors = 5,
  max_gaps = 25,
  is_pairwise = TRUE,
  verbose = FALSE,
  bp_param = BiocParallel::SerialParam(),
  ...
)
}
\arguments{
\item{blast_inter}{A list of BLAST/DIAMOND data frames for
interspecies comparisons as returned by \code{run_diamond()}.}

\item{annotation}{A processed GRangesList or CompressedGRangesList object
as returned by \code{process_input()}.}

\item{inter_dir}{Path to output directory where .collinearity files will
be stored.}

\item{anchors}{Numeric indicating the minimum required number of genes
to call a syntenic block. Default: 5.}

\item{max_gaps}{Numeric indicating the number of upstream and downstream
genes to search for anchors. Default: 25.}

\item{is_pairwise}{specify if only pairwise blocks should be reported
Default: TRUE.}

\item{verbose}{Logical indicating if log messages should be printed on
screen. Default: FALSE.}

\item{bp_param}{BiocParallel back-end to be used.
Default: \code{BiocParallel::SerialParam()}.}

\item{...}{Any additional arguments to the \code{MCScanX} algorithm. For a
complete list of all available options, see the man page
of \code{rcpp_mcscanx_file()}.}
}
\value{
Paths to .collinearity files.
}
\description{
Detect interspecies synteny
}
\examples{
# Load data
data(proteomes)
data(blast_list)
data(annotation)

# Get DIAMOND and processed annotation lists
blast_inter <- blast_list[2]
annotation <- process_input(proteomes, annotation)$annotation

# Detect interspecies synteny
intersyn <- interspecies_synteny(blast_inter, annotation)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{diamond_is_installed}
\alias{diamond_is_installed}
\title{Check if DIAMOND is installed}
\usage{
diamond_is_installed()
}
\value{
Logical indicating whether DIAMOND is installed or not.
}
\description{
Check if DIAMOND is installed
}
\examples{
diamond_is_installed()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/03_similarity_searches.R
\name{collapse_bidirectional_hits}
\alias{collapse_bidirectional_hits}
\title{Collapse a list of bidirectional DIAMOND hits}
\usage{
collapse_bidirectional_hits(blast_inter, compare)
}
\arguments{
\item{blast_inter}{A list of data frames containing BLAST/DIAMOND tables
with comparisons between target species and outgroups.
BLASTp, DIAMOND, or similar programs must be run on processed sequence data
as returned by \code{syntenet::process_input()}.}

\item{compare}{A 2-column data frame with target species name in column 1,
and outgroup species name in column 2. Species names must match names of
list elements in \strong{blast_inter}.}
}
\value{
A list of data frames with BLAST/DIAMOND tables as
in \strong{blast_inter}, but with bidirectional hits combined. For instance,
if \strong{blast_inter} contains elements 'spA_spB' and 'spB_spA', these
two data frames are combined into a single data frame following the
order indicated in \strong{outgroups} (i.e. 'column1_column2').
}
\description{
Collapse a list of bidirectional DIAMOND hits
}
\examples{
data(blast_list)
blast_inter <- blast_list[c(2,3)]
compare <- data.frame(species = "Olucimarinus", outgroup = "OspRCC809")
chits <- collapse_bidirectional_hits(blast_inter, compare)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/03_similarity_searches.R
\name{make_bidirectional}
\alias{make_bidirectional}
\title{Create a data frame of bidirectional comparisons from unidirectional comparisons}
\usage{
make_bidirectional(compare)
}
\arguments{
\item{compare}{A 2-column data frame with name(s) of target species in
column 1, and name(s) of outgroup species in column 2.}
}
\value{
A 2-column data frame as in \strong{compare}, but extended to
contain bidirectional comparisons. If the data frame in \strong{compare} has
N rows, the output data frame should contain 2N rows.
}
\description{
Create a data frame of bidirectional comparisons from unidirectional comparisons
}
\examples{
spp_outgroup <- data.frame(species = "spA", outgroup = "spB")
comp_bi <- make_bidirectional(spp_outgroup)
comp_bi
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{rcpp_mcscanx_file}
\alias{rcpp_mcscanx_file}
\title{rcpp_mcscanx_file}
\usage{
rcpp_mcscanx_file(
  blast_file,
  gff_file,
  prefix = "out",
  outdir = "",
  match_score = 50L,
  gap_penalty = -1L,
  match_size = 5L,
  e_value = 1e-05,
  max_gaps = 25L,
  overlap_window = 5L,
  is_pairwise = FALSE,
  in_synteny = 0L,
  species_id_length = 3L,
  verbose = FALSE
)
}
\arguments{
\item{blast_file}{Character indicating the path to the BLAST/DIAMOND output
file.}

\item{gff_file}{Character indicating the path to the "gff" file, which
is a tab-delimited file with 4 columns indicating the chromosome name,
gene id, gene start position, and gene end position, respectively.}

\item{prefix}{Character indicating the prefix to output files.
Default: "out".}

\item{outdir}{Character indicating the path to the output directory.
Default: "".}

\item{match_score}{Numeric indicating the match score. Default: 50.}

\item{gap_penalty}{Numeric indicating the gap penalty. Default: -1.}

\item{match_size}{Numeric indicating the minimum number of genes
required to call synteny. Default: 5.}

\item{e_value}{Numeric indicating the minimum e-value allowed.
Default: 1e-5.}

\item{max_gaps}{Numeric indicating the maximum number of gaps between
genes allowed. The unit measure of gaps is number of genes,
so \code{max_gaps = 20} indicates that a maximum of 20 genes can
exist between two homologous genes for synteny to be called. Default: 25.}

\item{overlap_window}{Numeric indicating the overlap window. Default: 5.}

\item{is_pairwise}{Logical indicating whether only pairwise blocks
should be reported. Default: FALSE.}

\item{in_synteny}{Numeric indicating the patterns of collinear blocks,
where 0 indicates intra and interspecies comparisons, 1 indicates
intraspecies comparisons, and 2 indicates interspecies comparisons.
Default: 0.}

\item{species_id_length}{Integer indicating the length of the species IDs.
Default: 3.
0: intra- and inter-species (default); 1: intra-species; 2: inter-species}

\item{verbose}{Logical indicating whether to print progress messages
to the screen. Default: FALSE.}
}
\value{
NULL, and a .collinearity file is created in the directory
specified in \strong{\code{outdir}}.
}
\description{
MCSCanX provides a clustering module for viewing the
relationship of collinear segments in multiple genomes (or heavily redundant
genomes). It takes the predicted pairwise segments from dynamic programming
(DAGchainer in particular) and then tries to build consensus segments from a
set of related, overlapping segments.
}
\references{
Wang et al. (2012) MCScanX: a toolkit for detection and
evolutionary analysis of gene synteny and collinearity.
\emph{Nucleic acids research}. \bold{40.7}, e49-e49.

Haas et al. (2004) DAGchainer: a tool for mining segmental
genome duplications and synteny. \emph{Bioinformatics}. \bold{20.18}
3643-3646.
}
\author{
Kristian K Ullrich and Fabricio Almeida-Silva
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_data_preprocessing.R
\name{check_input}
\alias{check_input}
\title{Check if input objects are ready for further analyses}
\usage{
check_input(seq = NULL, annotation = NULL, gene_field = "gene_id")
}
\arguments{
\item{seq}{A list of AAStringSet objects, each list element containing
protein sequences for a given species. This list must have names
(not NULL), and names of each list element must match the names of
list elements in \strong{annotation}.}

\item{annotation}{A GRangesList, CompressedGRangesList, or list of
GRanges with the annotation for the sequences in \strong{seq}. This list must
have names (not NULL), and names of each list element must match the names
of list elements in \strong{seq}.}

\item{gene_field}{Character, name of the column in the GRanges objects
that contains gene IDs. Default: "gene_id".}
}
\value{
TRUE if the objects pass the check.
}
\description{
Check if input objects are ready for further analyses
}
\details{
This function checks the input data for 3 required conditions:
\enumerate{
\item Names of \strong{seq} list (i.e., \code{names(seq)}) match
the names of \strong{annotation} GRangesList/CompressedGRangesList
(i.e., \code{names(annotation)})
\item For each species (list elements), the number of sequences
in \strong{seq} is not greater than the number of genes
in \strong{annotation}. This is a way to ensure users do not input
the translated sequences for multiple isoforms of the same gene (generated
by alternative splicing). Ideally, the number of sequences in \strong{seq}
should be equal to the number of genes in \strong{annotation}, but
this may not always stand true because of non-protein-coding genes.
\item For each species, sequence names (i.e., \code{names(seq[[x]])},
equivalent to FASTA headers) match gene names in \strong{annotation}.
}
}
\examples{
data(annotation) 
data(proteomes)
check_input(proteomes, annotation)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/03_similarity_searches.R
\name{export_sequences}
\alias{export_sequences}
\title{Export processed sequences as FASTA files}
\usage{
export_sequences(seq = NULL, outdir = tempdir())
}
\arguments{
\item{seq}{A processed list of AAStringSet objects
as returned by \code{process_input()}.}

\item{outdir}{Path to output directory where FASTA files will be stored.}
}
\value{
Path to exported FASTA files.
}
\description{
Export processed sequences as FASTA files
}
\examples{
# Load data
data(proteomes)
data(annotation)

# Process data
pdata <- process_input(proteomes, annotation)

# Export data
outdir <- file.path(tempdir(), "example_test")
export_sequences(pdata$seq, outdir)
}
